/*     */ package org.apache.batik.ext.awt.image.codec.imageio;
/*     */ 
/*     */ import java.awt.geom.Rectangle2D;
/*     */ import java.awt.image.BufferedImage;
/*     */ import java.awt.image.ColorModel;
/*     */ import java.awt.image.WritableRaster;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.util.Iterator;
/*     */ import javax.imageio.ImageIO;
/*     */ import javax.imageio.ImageReader;
/*     */ import javax.imageio.stream.ImageInputStream;
/*     */ import org.apache.batik.ext.awt.image.GraphicsUtil;
/*     */ import org.apache.batik.ext.awt.image.renderable.DeferRable;
/*     */ import org.apache.batik.ext.awt.image.renderable.Filter;
/*     */ import org.apache.batik.ext.awt.image.renderable.RedRable;
/*     */ import org.apache.batik.ext.awt.image.rendered.Any2sRGBRed;
/*     */ import org.apache.batik.ext.awt.image.rendered.CachableRed;
/*     */ import org.apache.batik.ext.awt.image.rendered.FormatRed;
/*     */ import org.apache.batik.ext.awt.image.spi.ImageTagRegistry;
/*     */ import org.apache.batik.ext.awt.image.spi.MagicNumberRegistryEntry;
/*     */ import org.apache.batik.ext.awt.image.spi.MagicNumberRegistryEntry.MagicNumber;
/*     */ import org.apache.batik.util.ParsedURL;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class AbstractImageIORegistryEntry
/*     */   extends MagicNumberRegistryEntry
/*     */ {
/*  64 */   public AbstractImageIORegistryEntry(String name, String[] exts, String[] mimeTypes, MagicNumber[] magicNumbers) { super(name, 1100.0F, exts, mimeTypes, magicNumbers); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  79 */   public AbstractImageIORegistryEntry(String name, String ext, String mimeType, int offset, byte[] magicNumber) { super(name, 1100.0F, ext, mimeType, offset, magicNumber); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Filter handleStream(InputStream inIS, ParsedURL origURL, boolean needRawData) {
/*     */     Object[] errParam;
/*     */     String errCode;
/*  95 */     DeferRable dr = new DeferRable();
/*  96 */     InputStream is = inIS;
/*     */ 
/*     */     
/*  99 */     if (origURL != null) {
/* 100 */       errCode = "url.format.unreadable";
/* 101 */       errParam = new Object[] { getFormatName(), origURL };
/*     */     } else {
/* 103 */       errCode = "stream.format.unreadable";
/* 104 */       errParam = new Object[] { getFormatName() };
/*     */     } 
/*     */     
/* 107 */     Thread t = new Thread(this, is, dr, errCode, errParam) { private final InputStream val$is; private final DeferRable val$dr;
/*     */         public void run() {
/*     */           Filter filt;
/*     */           try {
/* 111 */             Iterator iter = ImageIO.getImageReadersByMIMEType(this.this$0.getMimeTypes().get(0).toString());
/*     */             
/* 113 */             if (!iter.hasNext()) {
/* 114 */               throw new UnsupportedOperationException("No image reader for " + this.this$0.getFormatName() + " available!");
/*     */             }
/*     */ 
/*     */             
/* 118 */             ImageReader reader = (ImageReader)iter.next();
/* 119 */             ImageInputStream imageIn = ImageIO.createImageInputStream(this.val$is);
/* 120 */             reader.setInput(imageIn, true);
/*     */             
/* 122 */             int imageIndex = 0;
/* 123 */             this.val$dr.setBounds(new Rectangle2D.Double(0.0D, 0.0D, reader.getWidth(imageIndex), reader.getHeight(imageIndex)));
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */             
/* 130 */             BufferedImage bi = reader.read(imageIndex);
/* 131 */             CachableRed cr = GraphicsUtil.wrap(bi);
/* 132 */             Any2sRGBRed any2sRGBRed = new Any2sRGBRed(cr);
/* 133 */             FormatRed formatRed = new FormatRed(any2sRGBRed, GraphicsUtil.sRGB_Unpre);
/* 134 */             WritableRaster wr = (WritableRaster)formatRed.getData();
/* 135 */             ColorModel cm = formatRed.getColorModel();
/* 136 */             BufferedImage image = new BufferedImage(cm, wr, cm.isAlphaPremultiplied(), null);
/*     */             
/* 138 */             CachableRed cachableRed = GraphicsUtil.wrap(image);
/* 139 */             filt = new RedRable(cachableRed);
/* 140 */           } catch (IOException ioe) {
/*     */             
/* 142 */             filt = ImageTagRegistry.getBrokenLinkImage(this.this$0, this.val$errCode, this.val$errParam);
/*     */           
/*     */           }
/* 145 */           catch (ThreadDeath td) {
/* 146 */             filt = ImageTagRegistry.getBrokenLinkImage(this.this$0, this.val$errCode, this.val$errParam);
/*     */ 
/*     */             
/* 149 */             this.val$dr.setSource(filt);
/* 150 */             throw td;
/* 151 */           } catch (Throwable t) {
/* 152 */             filt = ImageTagRegistry.getBrokenLinkImage(this.this$0, this.val$errCode, this.val$errParam);
/*     */           } 
/*     */ 
/*     */ 
/*     */           
/* 157 */           this.val$dr.setSource(filt);
/*     */         } private final String val$errCode; private final Object[] val$errParam; private final AbstractImageIORegistryEntry this$0; }
/*     */       ;
/* 160 */     t.start();
/* 161 */     return dr;
/*     */   }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\imageio\AbstractImageIORegistryEntry.class
 * Java compiler version: 2 (46.0)
 * JD-Core Version:       1.0.7
 */