/*     */ package org.apache.batik.ext.awt.image.codec.imageio;
/*     */ 
/*     */ import java.awt.image.RenderedImage;
/*     */ import java.io.IOException;
/*     */ import java.io.OutputStream;
/*     */ import java.util.Iterator;
/*     */ import javax.imageio.IIOImage;
/*     */ import javax.imageio.ImageIO;
/*     */ import javax.imageio.ImageTypeSpecifier;
/*     */ import javax.imageio.ImageWriteParam;
/*     */ import javax.imageio.ImageWriter;
/*     */ import javax.imageio.event.IIOWriteWarningListener;
/*     */ import javax.imageio.metadata.IIOInvalidTreeException;
/*     */ import javax.imageio.metadata.IIOMetadata;
/*     */ import javax.imageio.metadata.IIOMetadataNode;
/*     */ import javax.imageio.stream.ImageOutputStream;
/*     */ import org.apache.batik.ext.awt.image.spi.ImageWriter;
/*     */ import org.apache.batik.ext.awt.image.spi.ImageWriterParams;
/*     */ import org.w3c.dom.Node;
/*     */ import org.w3c.dom.NodeList;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ImageIOImageWriter
/*     */   implements ImageWriter, IIOWriteWarningListener
/*     */ {
/*     */   private String targetMIME;
/*     */   
/*  56 */   public ImageIOImageWriter(String mime) { this.targetMIME = mime; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  63 */   public void writeImage(RenderedImage image, OutputStream out) throws IOException { writeImage(image, out, null); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public void writeImage(RenderedImage image, OutputStream out, ImageWriterParams params) throws IOException {
/*  73 */     Iterator iter = ImageIO.getImageWritersByMIMEType(getMIMEType());
/*  74 */     ImageWriter iiowriter = null;
/*     */     try {
/*  76 */       iiowriter = (ImageWriter)iter.next();
/*  77 */       if (iiowriter != null) {
/*  78 */         iiowriter.addIIOWriteWarningListener(this);
/*     */         
/*  80 */         ImageOutputStream imgout = null; try {
/*     */           ImageTypeSpecifier type;
/*  82 */           imgout = ImageIO.createImageOutputStream(out);
/*  83 */           ImageWriteParam iwParam = getDefaultWriteParam(iiowriter, image, params);
/*     */ 
/*     */           
/*  86 */           if (iwParam.getDestinationType() != null) {
/*  87 */             type = iwParam.getDestinationType();
/*     */           } else {
/*  89 */             type = ImageTypeSpecifier.createFromRenderedImage(image);
/*     */           } 
/*     */ 
/*     */           
/*  93 */           IIOMetadata meta = iiowriter.getDefaultImageMetadata(type, iwParam);
/*     */ 
/*     */           
/*  96 */           if (params != null && meta != null) {
/*  97 */             meta = updateMetadata(meta, params);
/*     */           }
/*     */ 
/*     */           
/* 101 */           iiowriter.setOutput(imgout);
/* 102 */           IIOImage iioimg = new IIOImage(image, null, meta);
/* 103 */           iiowriter.write(null, iioimg, iwParam);
/*     */         } finally {
/* 105 */           if (imgout != null) {
/* 106 */             System.err.println("closing");
/* 107 */             imgout.close();
/*     */           } 
/*     */         } 
/*     */       } else {
/* 111 */         throw new UnsupportedOperationException("No ImageIO codec for writing " + getMIMEType() + " is available!");
/*     */       } 
/*     */     } finally {
/*     */       
/* 115 */       if (iiowriter != null) {
/* 116 */         System.err.println("disposing");
/* 117 */         iiowriter.dispose();
/*     */       } 
/*     */     } 
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected ImageWriteParam getDefaultWriteParam(ImageWriter iiowriter, RenderedImage image, ImageWriterParams params) {
/* 132 */     ImageWriteParam param = iiowriter.getDefaultWriteParam();
/* 133 */     System.err.println("Param: " + params);
/* 134 */     if (params != null && params.getCompressionMethod() != null) {
/* 135 */       param.setCompressionMode(2);
/* 136 */       param.setCompressionType(params.getCompressionMethod());
/*     */     } 
/* 138 */     return param;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected IIOMetadata updateMetadata(IIOMetadata meta, ImageWriterParams params) {
/* 148 */     String stdmeta = "javax_imageio_1.0";
/* 149 */     if (meta.isStandardMetadataFormatSupported()) {
/* 150 */       IIOMetadataNode root = (IIOMetadataNode)meta.getAsTree("javax_imageio_1.0");
/* 151 */       IIOMetadataNode dim = getChildNode(root, "Dimension");
/*     */       
/* 153 */       if (params.getResolution() != null) {
/* 154 */         IIOMetadataNode child = getChildNode(dim, "HorizontalPixelSize");
/* 155 */         if (child == null) {
/* 156 */           child = new IIOMetadataNode("HorizontalPixelSize");
/* 157 */           dim.appendChild(child);
/*     */         } 
/* 159 */         child.setAttribute("value", Double.toString(params.getResolution().doubleValue() / 25.4D));
/*     */         
/* 161 */         child = getChildNode(dim, "VerticalPixelSize");
/* 162 */         if (child == null) {
/* 163 */           child = new IIOMetadataNode("VerticalPixelSize");
/* 164 */           dim.appendChild(child);
/*     */         } 
/* 166 */         child.setAttribute("value", Double.toString(params.getResolution().doubleValue() / 25.4D));
/*     */       } 
/*     */       
/*     */       try {
/* 170 */         meta.mergeTree("javax_imageio_1.0", root);
/* 171 */       } catch (IIOInvalidTreeException e) {
/* 172 */         throw new RuntimeException("Cannot update image metadata: " + e.getMessage());
/*     */       } 
/*     */     } 
/*     */     
/* 176 */     return meta;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   protected static IIOMetadataNode getChildNode(Node n, String name) {
/* 186 */     NodeList nodes = n.getChildNodes();
/* 187 */     for (int i = 0; i < nodes.getLength(); i++) {
/* 188 */       Node child = nodes.item(i);
/* 189 */       if (name.equals(child.getNodeName())) {
/* 190 */         return (IIOMetadataNode)child;
/*     */       }
/*     */     } 
/* 193 */     return null;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 200 */   public String getMIMEType() { return this.targetMIME; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 208 */   public void warningOccurred(ImageWriter source, int imageIndex, String warning) { System.err.println("Problem while writing image using ImageI/O: " + warning); }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\imageio\ImageIOImageWriter.class
 * Java compiler version: 2 (46.0)
 * JD-Core Version:       1.0.7
 */