/*      */ package org.apache.batik.ext.awt.image.codec.png;
/*      */ 
/*      */ import java.awt.Rectangle;
/*      */ import java.awt.image.ColorModel;
/*      */ import java.awt.image.IndexColorModel;
/*      */ import java.awt.image.Raster;
/*      */ import java.awt.image.RenderedImage;
/*      */ import java.awt.image.SampleModel;
/*      */ import java.io.DataOutputStream;
/*      */ import java.io.IOException;
/*      */ import java.io.OutputStream;
/*      */ import java.util.Date;
/*      */ import java.util.GregorianCalendar;
/*      */ import java.util.TimeZone;
/*      */ import java.util.zip.Deflater;
/*      */ import java.util.zip.DeflaterOutputStream;
/*      */ import org.apache.batik.ext.awt.image.codec.util.ImageEncoderImpl;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class PNGImageEncoder
/*      */   extends ImageEncoderImpl
/*      */ {
/*      */   private static final int PNG_COLOR_GRAY = 0;
/*      */   private static final int PNG_COLOR_RGB = 2;
/*      */   private static final int PNG_COLOR_PALETTE = 3;
/*      */   private static final int PNG_COLOR_GRAY_ALPHA = 4;
/*      */   private static final int PNG_COLOR_RGB_ALPHA = 6;
/*  272 */   private static final byte[] magic = { -119, 80, 78, 71, 13, 10, 26, 10 };
/*      */   
/*      */   private PNGEncodeParam param;
/*      */   
/*      */   private RenderedImage image;
/*      */   
/*      */   private int width;
/*      */   
/*      */   private int height;
/*      */   
/*      */   private int bitDepth;
/*      */   
/*      */   private int bitShift;
/*      */   
/*      */   private int numBands;
/*      */   
/*      */   private int colorType;
/*      */   
/*      */   private int bpp;
/*      */   private boolean skipAlpha = false;
/*      */   private boolean compressGray = false;
/*      */   private boolean interlace;
/*  294 */   private byte[] redPalette = null;
/*  295 */   private byte[] greenPalette = null;
/*  296 */   private byte[] bluePalette = null;
/*  297 */   private byte[] alphaPalette = null;
/*      */   private DataOutputStream dataOutput;
/*      */   private byte[] prevRow;
/*      */   private byte[] currRow;
/*      */   private byte[][] filteredRows;
/*      */   
/*  303 */   public PNGImageEncoder(OutputStream output, PNGEncodeParam param) { super(output, param);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  329 */     this.prevRow = null;
/*  330 */     this.currRow = null;
/*      */     
/*  332 */     this.filteredRows = null;
/*      */     if (param != null)
/*      */       this.param = param; 
/*  335 */     this.dataOutput = new DataOutputStream(output); } private void writeMagic() throws IOException { this.dataOutput.write(magic); } private static int clamp(int val, int maxValue) { return (val > maxValue) ? maxValue : val; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void encodePass(OutputStream os, Raster ras, int xOffset, int yOffset, int xSkip, int ySkip) throws IOException {
/*  342 */     int minX = ras.getMinX();
/*  343 */     int minY = ras.getMinY();
/*  344 */     int width = ras.getWidth();
/*  345 */     int height = ras.getHeight();
/*      */     
/*  347 */     xOffset *= this.numBands;
/*  348 */     xSkip *= this.numBands;
/*      */     
/*  350 */     int samplesPerByte = 8 / this.bitDepth;
/*      */     
/*  352 */     int numSamples = width * this.numBands;
/*  353 */     int[] samples = new int[numSamples];
/*      */     
/*  355 */     int pixels = (numSamples - xOffset + xSkip - 1) / xSkip;
/*  356 */     int bytesPerRow = pixels * this.numBands;
/*  357 */     if (this.bitDepth < 8) {
/*  358 */       bytesPerRow = (bytesPerRow + samplesPerByte - 1) / samplesPerByte;
/*  359 */     } else if (this.bitDepth == 16) {
/*  360 */       bytesPerRow *= 2;
/*      */     } 
/*      */     
/*  363 */     if (bytesPerRow == 0) {
/*      */       return;
/*      */     }
/*      */     
/*  367 */     this.currRow = new byte[bytesPerRow + this.bpp];
/*  368 */     this.prevRow = new byte[bytesPerRow + this.bpp];
/*      */     
/*  370 */     this.filteredRows = new byte[5][bytesPerRow + this.bpp];
/*      */     
/*  372 */     int maxValue = (1 << this.bitDepth) - 1;
/*      */     
/*  374 */     for (int row = minY + yOffset; row < minY + height; row += ySkip) {
/*  375 */       int s, s, s, mask; ras.getPixels(minX, row, width, 1, samples);
/*      */       
/*  377 */       if (this.compressGray) {
/*  378 */         int shift = 8 - this.bitDepth;
/*  379 */         for (int i = 0; i < width; i++) {
/*  380 */           samples[i] = samples[i] >> shift;
/*      */         }
/*      */       } 
/*      */       
/*  384 */       int count = this.bpp;
/*  385 */       int pos = 0;
/*  386 */       int tmp = 0;
/*      */       
/*  388 */       switch (this.bitDepth) {
/*      */         case 1:
/*      */         case 2:
/*      */         case 4:
/*  392 */           mask = samplesPerByte - 1;
/*  393 */           for (s = xOffset; s < numSamples; s += xSkip) {
/*  394 */             int val = clamp(samples[s] >> this.bitShift, maxValue);
/*  395 */             tmp = tmp << this.bitDepth | val;
/*      */             
/*  397 */             if (pos++ == mask) {
/*  398 */               this.currRow[count++] = (byte)tmp;
/*  399 */               tmp = 0;
/*  400 */               pos = 0;
/*      */             } 
/*      */           } 
/*      */ 
/*      */           
/*  405 */           if (pos != 0) {
/*  406 */             tmp <<= (samplesPerByte - pos) * this.bitDepth;
/*  407 */             this.currRow[count++] = (byte)tmp;
/*      */           } 
/*      */           break;
/*      */         
/*      */         case 8:
/*  412 */           for (s = xOffset; s < numSamples; s += xSkip) {
/*  413 */             for (int b = 0; b < this.numBands; b++) {
/*  414 */               this.currRow[count++] = (byte)clamp(samples[s + b] >> this.bitShift, maxValue);
/*      */             }
/*      */           } 
/*      */           break;
/*      */ 
/*      */         
/*      */         case 16:
/*  421 */           for (s = xOffset; s < numSamples; s += xSkip) {
/*  422 */             for (int b = 0; b < this.numBands; b++) {
/*  423 */               int val = clamp(samples[s + b] >> this.bitShift, maxValue);
/*  424 */               this.currRow[count++] = (byte)(val >> 8);
/*  425 */               this.currRow[count++] = (byte)(val & 0xFF);
/*      */             } 
/*      */           } 
/*      */           break;
/*      */       } 
/*      */ 
/*      */       
/*  432 */       int filterType = this.param.filterRow(this.currRow, this.prevRow, this.filteredRows, bytesPerRow, this.bpp);
/*      */ 
/*      */ 
/*      */       
/*  436 */       os.write(filterType);
/*  437 */       os.write(this.filteredRows[filterType], this.bpp, bytesPerRow);
/*      */ 
/*      */       
/*  440 */       byte[] swap = this.currRow;
/*  441 */       this.currRow = this.prevRow;
/*  442 */       this.prevRow = swap;
/*      */     } 
/*      */   } private void writeIHDR() throws IOException { ChunkStream cs = new ChunkStream("IHDR"); cs.writeInt(this.width); cs.writeInt(this.height); cs.writeByte((byte)this.bitDepth); cs.writeByte((byte)this.colorType); cs.writeByte(0); cs.writeByte(0);
/*      */     cs.writeByte(this.interlace ? 1 : 0);
/*      */     cs.writeToStream(this.dataOutput);
/*  447 */     cs.close(); } private void writeIDAT() throws IOException { IDATOutputStream ios = new IDATOutputStream(this.dataOutput, 8192);
/*  448 */     DeflaterOutputStream dos = new DeflaterOutputStream(ios, new Deflater(9));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  458 */     Raster ras = this.image.getData(new Rectangle(this.image.getMinX(), this.image.getMinY(), this.image.getWidth(), this.image.getHeight()));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  473 */     if (this.skipAlpha) {
/*  474 */       int numBands = ras.getNumBands() - 1;
/*  475 */       int[] bandList = new int[numBands];
/*  476 */       for (int i = 0; i < numBands; i++) {
/*  477 */         bandList[i] = i;
/*      */       }
/*  479 */       ras = ras.createChild(0, 0, ras.getWidth(), ras.getHeight(), 0, 0, bandList);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  485 */     if (this.interlace) {
/*      */       
/*  487 */       encodePass(dos, ras, 0, 0, 8, 8);
/*      */       
/*  489 */       encodePass(dos, ras, 4, 0, 8, 8);
/*      */       
/*  491 */       encodePass(dos, ras, 0, 4, 4, 8);
/*      */       
/*  493 */       encodePass(dos, ras, 2, 0, 4, 4);
/*      */       
/*  495 */       encodePass(dos, ras, 0, 2, 2, 4);
/*      */       
/*  497 */       encodePass(dos, ras, 1, 0, 2, 2);
/*      */       
/*  499 */       encodePass(dos, ras, 0, 1, 1, 2);
/*      */     } else {
/*  501 */       encodePass(dos, ras, 0, 0, 1, 1);
/*      */     } 
/*      */     
/*  504 */     dos.finish();
/*  505 */     ios.flush(); }
/*      */ 
/*      */   
/*      */   private void writeIEND() throws IOException {
/*  509 */     ChunkStream cs = new ChunkStream("IEND");
/*  510 */     cs.writeToStream(this.dataOutput);
/*  511 */     cs.close();
/*      */   }
/*      */   
/*  514 */   private static final float[] srgbChroma = { 0.3127F, 0.329F, 0.64F, 0.33F, 0.3F, 0.6F, 0.15F, 0.06F };
/*      */ 
/*      */ 
/*      */   
/*      */   private void writeCHRM() throws IOException {
/*  519 */     if (this.param.isChromaticitySet() || this.param.isSRGBIntentSet()) {
/*  520 */       float[] arrayOfFloat; ChunkStream cs = new ChunkStream("cHRM");
/*      */ 
/*      */       
/*  523 */       if (!this.param.isSRGBIntentSet()) {
/*  524 */         arrayOfFloat = this.param.getChromaticity();
/*      */       } else {
/*  526 */         arrayOfFloat = srgbChroma;
/*      */       } 
/*      */       
/*  529 */       for (int i = 0; i < 8; i++) {
/*  530 */         cs.writeInt((int)(arrayOfFloat[i] * 100000.0F));
/*      */       }
/*  532 */       cs.writeToStream(this.dataOutput);
/*  533 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeGAMA() throws IOException {
/*  538 */     if (this.param.isGammaSet() || this.param.isSRGBIntentSet()) {
/*  539 */       float f; ChunkStream cs = new ChunkStream("gAMA");
/*      */ 
/*      */       
/*  542 */       if (!this.param.isSRGBIntentSet()) {
/*  543 */         f = this.param.getGamma();
/*      */       } else {
/*  545 */         f = 0.45454544F;
/*      */       } 
/*      */ 
/*      */       
/*  549 */       cs.writeInt((int)(f * 100000.0F));
/*  550 */       cs.writeToStream(this.dataOutput);
/*  551 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeICCP() throws IOException {
/*  556 */     if (this.param.isICCProfileDataSet()) {
/*  557 */       ChunkStream cs = new ChunkStream("iCCP");
/*  558 */       byte[] ICCProfileData = this.param.getICCProfileData();
/*  559 */       cs.write(ICCProfileData);
/*  560 */       cs.writeToStream(this.dataOutput);
/*  561 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeSBIT() throws IOException {
/*  566 */     if (this.param.isSignificantBitsSet()) {
/*  567 */       ChunkStream cs = new ChunkStream("sBIT");
/*  568 */       int[] significantBits = this.param.getSignificantBits();
/*  569 */       int len = significantBits.length;
/*  570 */       for (int i = 0; i < len; i++) {
/*  571 */         cs.writeByte(significantBits[i]);
/*      */       }
/*  573 */       cs.writeToStream(this.dataOutput);
/*  574 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeSRGB() throws IOException {
/*  579 */     if (this.param.isSRGBIntentSet()) {
/*  580 */       ChunkStream cs = new ChunkStream("sRGB");
/*      */       
/*  582 */       int intent = this.param.getSRGBIntent();
/*  583 */       cs.write(intent);
/*  584 */       cs.writeToStream(this.dataOutput);
/*  585 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writePLTE() throws IOException {
/*  590 */     if (this.redPalette == null) {
/*      */       return;
/*      */     }
/*      */     
/*  594 */     ChunkStream cs = new ChunkStream("PLTE");
/*  595 */     for (int i = 0; i < this.redPalette.length; i++) {
/*  596 */       cs.writeByte(this.redPalette[i]);
/*  597 */       cs.writeByte(this.greenPalette[i]);
/*  598 */       cs.writeByte(this.bluePalette[i]);
/*      */     } 
/*      */     
/*  601 */     cs.writeToStream(this.dataOutput);
/*  602 */     cs.close();
/*      */   }
/*      */   
/*      */   private void writeBKGD() throws IOException {
/*  606 */     if (this.param.isBackgroundSet()) {
/*  607 */       int rgb[], index, gray; ChunkStream cs = new ChunkStream("bKGD");
/*      */       
/*  609 */       switch (this.colorType) {
/*      */         case 0:
/*      */         case 4:
/*  612 */           gray = ((PNGEncodeParam.Gray)this.param).getBackgroundGray();
/*  613 */           cs.writeShort(gray);
/*      */           break;
/*      */         
/*      */         case 3:
/*  617 */           index = ((PNGEncodeParam.Palette)this.param).getBackgroundPaletteIndex();
/*      */           
/*  619 */           cs.writeByte(index);
/*      */           break;
/*      */         
/*      */         case 2:
/*      */         case 6:
/*  624 */           rgb = ((PNGEncodeParam.RGB)this.param).getBackgroundRGB();
/*  625 */           cs.writeShort(rgb[0]);
/*  626 */           cs.writeShort(rgb[1]);
/*  627 */           cs.writeShort(rgb[2]);
/*      */           break;
/*      */       } 
/*      */       
/*  631 */       cs.writeToStream(this.dataOutput);
/*  632 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeHIST() throws IOException {
/*  637 */     if (this.param.isPaletteHistogramSet()) {
/*  638 */       ChunkStream cs = new ChunkStream("hIST");
/*      */       
/*  640 */       int[] hist = this.param.getPaletteHistogram();
/*  641 */       for (int i = 0; i < hist.length; i++) {
/*  642 */         cs.writeShort(hist[i]);
/*      */       }
/*      */       
/*  645 */       cs.writeToStream(this.dataOutput);
/*  646 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeTRNS() throws IOException {
/*  651 */     if (this.param.isTransparencySet() && this.colorType != 4 && this.colorType != 6) {
/*      */ 
/*      */       
/*  654 */       ChunkStream cs = new ChunkStream("tRNS");
/*      */       
/*  656 */       if (this.param instanceof PNGEncodeParam.Palette) {
/*  657 */         byte[] t = ((PNGEncodeParam.Palette)this.param).getPaletteTransparency();
/*      */         
/*  659 */         for (int i = 0; i < t.length; i++) {
/*  660 */           cs.writeByte(t[i]);
/*      */         }
/*  662 */       } else if (this.param instanceof PNGEncodeParam.Gray) {
/*  663 */         int t = ((PNGEncodeParam.Gray)this.param).getTransparentGray();
/*  664 */         cs.writeShort(t);
/*  665 */       } else if (this.param instanceof PNGEncodeParam.RGB) {
/*  666 */         int[] t = ((PNGEncodeParam.RGB)this.param).getTransparentRGB();
/*  667 */         cs.writeShort(t[0]);
/*  668 */         cs.writeShort(t[1]);
/*  669 */         cs.writeShort(t[2]);
/*      */       } 
/*      */       
/*  672 */       cs.writeToStream(this.dataOutput);
/*  673 */       cs.close();
/*  674 */     } else if (this.colorType == 3) {
/*  675 */       int lastEntry = Math.min(255, this.alphaPalette.length - 1);
/*      */       int nonOpaque;
/*  677 */       for (nonOpaque = lastEntry; nonOpaque >= 0 && 
/*  678 */         this.alphaPalette[nonOpaque] == -1; nonOpaque--);
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  683 */       if (nonOpaque >= 0) {
/*  684 */         ChunkStream cs = new ChunkStream("tRNS");
/*  685 */         for (int i = 0; i <= nonOpaque; i++) {
/*  686 */           cs.writeByte(this.alphaPalette[i]);
/*      */         }
/*  688 */         cs.writeToStream(this.dataOutput);
/*  689 */         cs.close();
/*      */       } 
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writePHYS() throws IOException {
/*  695 */     if (this.param.isPhysicalDimensionSet()) {
/*  696 */       ChunkStream cs = new ChunkStream("pHYs");
/*      */       
/*  698 */       int[] dims = this.param.getPhysicalDimension();
/*  699 */       cs.writeInt(dims[0]);
/*  700 */       cs.writeInt(dims[1]);
/*  701 */       cs.writeByte((byte)dims[2]);
/*      */       
/*  703 */       cs.writeToStream(this.dataOutput);
/*  704 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeSPLT() throws IOException {
/*  709 */     if (this.param.isSuggestedPaletteSet()) {
/*  710 */       ChunkStream cs = new ChunkStream("sPLT");
/*      */       
/*  712 */       System.out.println("sPLT not supported yet.");
/*      */       
/*  714 */       cs.writeToStream(this.dataOutput);
/*  715 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeTIME() throws IOException {
/*  720 */     if (this.param.isModificationTimeSet()) {
/*  721 */       ChunkStream cs = new ChunkStream("tIME");
/*      */       
/*  723 */       Date date = this.param.getModificationTime();
/*  724 */       TimeZone gmt = TimeZone.getTimeZone("GMT");
/*      */       
/*  726 */       GregorianCalendar cal = new GregorianCalendar(gmt);
/*  727 */       cal.setTime(date);
/*      */       
/*  729 */       int year = cal.get(1);
/*  730 */       int month = cal.get(2);
/*  731 */       int day = cal.get(5);
/*  732 */       int hour = cal.get(11);
/*  733 */       int minute = cal.get(12);
/*  734 */       int second = cal.get(13);
/*      */       
/*  736 */       cs.writeShort(year);
/*  737 */       cs.writeByte(month + 1);
/*  738 */       cs.writeByte(day);
/*  739 */       cs.writeByte(hour);
/*  740 */       cs.writeByte(minute);
/*  741 */       cs.writeByte(second);
/*      */       
/*  743 */       cs.writeToStream(this.dataOutput);
/*  744 */       cs.close();
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeTEXT() throws IOException {
/*  749 */     if (this.param.isTextSet()) {
/*  750 */       String[] text = this.param.getText();
/*      */       
/*  752 */       for (int i = 0; i < text.length / 2; i++) {
/*  753 */         byte[] keyword = text[2 * i].getBytes();
/*  754 */         byte[] value = text[2 * i + 1].getBytes();
/*      */         
/*  756 */         ChunkStream cs = new ChunkStream("tEXt");
/*      */         
/*  758 */         cs.write(keyword, 0, Math.min(keyword.length, 79));
/*  759 */         cs.write(0);
/*  760 */         cs.write(value);
/*      */         
/*  762 */         cs.writeToStream(this.dataOutput);
/*  763 */         cs.close();
/*      */       } 
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writeZTXT() throws IOException {
/*  769 */     if (this.param.isCompressedTextSet()) {
/*  770 */       String[] text = this.param.getCompressedText();
/*      */       
/*  772 */       for (int i = 0; i < text.length / 2; i++) {
/*  773 */         byte[] keyword = text[2 * i].getBytes();
/*  774 */         byte[] value = text[2 * i + 1].getBytes();
/*      */         
/*  776 */         ChunkStream cs = new ChunkStream("zTXt");
/*      */         
/*  778 */         cs.write(keyword, 0, Math.min(keyword.length, 79));
/*  779 */         cs.write(0);
/*  780 */         cs.write(0);
/*      */         
/*  782 */         DeflaterOutputStream dos = new DeflaterOutputStream(cs);
/*  783 */         dos.write(value);
/*  784 */         dos.finish();
/*      */         
/*  786 */         cs.writeToStream(this.dataOutput);
/*  787 */         cs.close();
/*      */       } 
/*      */     } 
/*      */   }
/*      */   
/*      */   private void writePrivateChunks() throws IOException {
/*  793 */     int numChunks = this.param.getNumPrivateChunks();
/*  794 */     for (int i = 0; i < numChunks; i++) {
/*  795 */       String type = this.param.getPrivateChunkType(i);
/*  796 */       byte[] data = this.param.getPrivateChunkData(i);
/*      */       
/*  798 */       ChunkStream cs = new ChunkStream(type);
/*  799 */       cs.write(data);
/*  800 */       cs.writeToStream(this.dataOutput);
/*  801 */       cs.close();
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private PNGEncodeParam.Gray createGrayParam(byte[] redPalette, byte[] greenPalette, byte[] bluePalette, byte[] alphaPalette) {
/*  817 */     PNGEncodeParam.Gray param = new PNGEncodeParam.Gray();
/*  818 */     int numTransparent = 0;
/*      */     
/*  820 */     int grayFactor = 255 / ((1 << this.bitDepth) - 1);
/*  821 */     int entries = 1 << this.bitDepth;
/*  822 */     for (int i = 0; i < entries; i++) {
/*  823 */       byte red = redPalette[i];
/*  824 */       if (red != i * grayFactor || red != greenPalette[i] || red != bluePalette[i])
/*      */       {
/*      */         
/*  827 */         return null;
/*      */       }
/*      */ 
/*      */       
/*  831 */       byte alpha = alphaPalette[i];
/*  832 */       if (alpha == 0) {
/*  833 */         param.setTransparentGray(i);
/*      */         
/*  835 */         numTransparent++;
/*  836 */         if (numTransparent > 1) {
/*  837 */           return null;
/*      */         }
/*  839 */       } else if (alpha != -1) {
/*  840 */         return null;
/*      */       } 
/*      */     } 
/*      */     
/*  844 */     return param;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void encode(RenderedImage im) throws IOException {
/*  854 */     this.image = im;
/*  855 */     this.width = this.image.getWidth();
/*  856 */     this.height = this.image.getHeight();
/*      */     
/*  858 */     SampleModel sampleModel = this.image.getSampleModel();
/*      */     
/*  860 */     int[] sampleSize = sampleModel.getSampleSize();
/*      */ 
/*      */     
/*  863 */     this.bitDepth = -1;
/*  864 */     this.bitShift = 0;
/*      */ 
/*      */     
/*  867 */     if (this.param instanceof PNGEncodeParam.Gray) {
/*  868 */       PNGEncodeParam.Gray paramg = (PNGEncodeParam.Gray)this.param;
/*  869 */       if (paramg.isBitDepthSet()) {
/*  870 */         this.bitDepth = paramg.getBitDepth();
/*      */       }
/*      */       
/*  873 */       if (paramg.isBitShiftSet()) {
/*  874 */         this.bitShift = paramg.getBitShift();
/*      */       }
/*      */     } 
/*      */ 
/*      */     
/*  879 */     if (this.bitDepth == -1) {
/*      */ 
/*      */       
/*  882 */       this.bitDepth = sampleSize[0];
/*      */       
/*  884 */       for (int i = 1; i < sampleSize.length; i++) {
/*  885 */         if (sampleSize[i] != this.bitDepth) {
/*  886 */           throw new RuntimeException();
/*      */         }
/*      */       } 
/*      */ 
/*      */       
/*  891 */       if (this.bitDepth > 2 && this.bitDepth < 4) {
/*  892 */         this.bitDepth = 4;
/*  893 */       } else if (this.bitDepth > 4 && this.bitDepth < 8) {
/*  894 */         this.bitDepth = 8;
/*  895 */       } else if (this.bitDepth > 8 && this.bitDepth < 16) {
/*  896 */         this.bitDepth = 16;
/*  897 */       } else if (this.bitDepth > 16) {
/*  898 */         throw new RuntimeException();
/*      */       } 
/*      */     } 
/*      */     
/*  902 */     this.numBands = sampleModel.getNumBands();
/*  903 */     this.bpp = this.numBands * ((this.bitDepth == 16) ? 2 : 1);
/*      */     
/*  905 */     ColorModel colorModel = this.image.getColorModel();
/*  906 */     if (colorModel instanceof IndexColorModel) {
/*  907 */       if (this.bitDepth < 1 || this.bitDepth > 8) {
/*  908 */         throw new RuntimeException();
/*      */       }
/*  910 */       if (sampleModel.getNumBands() != 1) {
/*  911 */         throw new RuntimeException();
/*      */       }
/*      */       
/*  914 */       IndexColorModel icm = (IndexColorModel)colorModel;
/*  915 */       int size = icm.getMapSize();
/*      */       
/*  917 */       this.redPalette = new byte[size];
/*  918 */       this.greenPalette = new byte[size];
/*  919 */       this.bluePalette = new byte[size];
/*  920 */       this.alphaPalette = new byte[size];
/*      */       
/*  922 */       icm.getReds(this.redPalette);
/*  923 */       icm.getGreens(this.greenPalette);
/*  924 */       icm.getBlues(this.bluePalette);
/*  925 */       icm.getAlphas(this.alphaPalette);
/*      */       
/*  927 */       this.bpp = 1;
/*      */       
/*  929 */       if (this.param == null) {
/*  930 */         this.param = createGrayParam(this.redPalette, this.greenPalette, this.bluePalette, this.alphaPalette);
/*      */       }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  937 */       if (this.param == null) {
/*  938 */         this.param = new PNGEncodeParam.Palette();
/*      */       }
/*      */       
/*  941 */       if (this.param instanceof PNGEncodeParam.Palette) {
/*      */         
/*  943 */         PNGEncodeParam.Palette parami = (PNGEncodeParam.Palette)this.param;
/*  944 */         if (parami.isPaletteSet()) {
/*  945 */           int[] palette = parami.getPalette();
/*  946 */           size = palette.length / 3;
/*      */           
/*  948 */           int index = 0;
/*  949 */           for (int i = 0; i < size; i++) {
/*  950 */             this.redPalette[i] = (byte)palette[index++];
/*  951 */             this.greenPalette[i] = (byte)palette[index++];
/*  952 */             this.bluePalette[i] = (byte)palette[index++];
/*  953 */             this.alphaPalette[i] = -1;
/*      */           } 
/*      */         } 
/*  956 */         this.colorType = 3;
/*  957 */       } else if (this.param instanceof PNGEncodeParam.Gray) {
/*  958 */         this.redPalette = this.greenPalette = this.bluePalette = this.alphaPalette = null;
/*  959 */         this.colorType = 0;
/*      */       } else {
/*  961 */         throw new RuntimeException();
/*      */       } 
/*  963 */     } else if (this.numBands == 1) {
/*  964 */       if (this.param == null) {
/*  965 */         this.param = new PNGEncodeParam.Gray();
/*      */       }
/*  967 */       this.colorType = 0;
/*  968 */     } else if (this.numBands == 2) {
/*  969 */       if (this.param == null) {
/*  970 */         this.param = new PNGEncodeParam.Gray();
/*      */       }
/*      */       
/*  973 */       if (this.param.isTransparencySet()) {
/*  974 */         this.skipAlpha = true;
/*  975 */         this.numBands = 1;
/*  976 */         if (sampleSize[0] == 8 && this.bitDepth < 8) {
/*  977 */           this.compressGray = true;
/*      */         }
/*  979 */         this.bpp = (this.bitDepth == 16) ? 2 : 1;
/*  980 */         this.colorType = 0;
/*      */       } else {
/*  982 */         if (this.bitDepth < 8) {
/*  983 */           this.bitDepth = 8;
/*      */         }
/*  985 */         this.colorType = 4;
/*      */       } 
/*  987 */     } else if (this.numBands == 3) {
/*  988 */       if (this.param == null) {
/*  989 */         this.param = new PNGEncodeParam.RGB();
/*      */       }
/*  991 */       this.colorType = 2;
/*  992 */     } else if (this.numBands == 4) {
/*  993 */       if (this.param == null) {
/*  994 */         this.param = new PNGEncodeParam.RGB();
/*      */       }
/*  996 */       if (this.param.isTransparencySet()) {
/*  997 */         this.skipAlpha = true;
/*  998 */         this.numBands = 3;
/*  999 */         this.bpp = (this.bitDepth == 16) ? 6 : 3;
/* 1000 */         this.colorType = 2;
/*      */       } else {
/* 1002 */         this.colorType = 6;
/*      */       } 
/*      */     } 
/*      */     
/* 1006 */     this.interlace = this.param.getInterlacing();
/*      */     
/* 1008 */     writeMagic();
/*      */     
/* 1010 */     writeIHDR();
/*      */     
/* 1012 */     writeCHRM();
/* 1013 */     writeGAMA();
/* 1014 */     writeICCP();
/* 1015 */     writeSBIT();
/* 1016 */     writeSRGB();
/*      */     
/* 1018 */     writePLTE();
/*      */     
/* 1020 */     writeHIST();
/* 1021 */     writeTRNS();
/* 1022 */     writeBKGD();
/*      */     
/* 1024 */     writePHYS();
/* 1025 */     writeSPLT();
/* 1026 */     writeTIME();
/* 1027 */     writeTEXT();
/* 1028 */     writeZTXT();
/*      */     
/* 1030 */     writePrivateChunks();
/*      */     
/* 1032 */     writeIDAT();
/*      */     
/* 1034 */     writeIEND();
/*      */     
/* 1036 */     this.dataOutput.flush();
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\png\PNGImageEncoder.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */