/*      */ package org.apache.batik.ext.awt.image.codec.png;
/*      */ 
/*      */ import java.awt.Color;
/*      */ import java.awt.Point;
/*      */ import java.awt.Rectangle;
/*      */ import java.awt.color.ColorSpace;
/*      */ import java.awt.image.ColorModel;
/*      */ import java.awt.image.ComponentColorModel;
/*      */ import java.awt.image.DataBuffer;
/*      */ import java.awt.image.DataBufferByte;
/*      */ import java.awt.image.DataBufferUShort;
/*      */ import java.awt.image.IndexColorModel;
/*      */ import java.awt.image.Raster;
/*      */ import java.awt.image.SampleModel;
/*      */ import java.awt.image.WritableRaster;
/*      */ import java.io.BufferedInputStream;
/*      */ import java.io.ByteArrayInputStream;
/*      */ import java.io.DataInputStream;
/*      */ import java.io.IOException;
/*      */ import java.io.InputStream;
/*      */ import java.io.SequenceInputStream;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Collections;
/*      */ import java.util.Date;
/*      */ import java.util.GregorianCalendar;
/*      */ import java.util.HashMap;
/*      */ import java.util.List;
/*      */ import java.util.Map;
/*      */ import java.util.TimeZone;
/*      */ import java.util.zip.Inflater;
/*      */ import java.util.zip.InflaterInputStream;
/*      */ import org.apache.batik.ext.awt.image.GraphicsUtil;
/*      */ import org.apache.batik.ext.awt.image.codec.util.PropertyUtil;
/*      */ import org.apache.batik.ext.awt.image.rendered.AbstractRed;
/*      */ import org.apache.batik.ext.awt.image.rendered.CachableRed;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class PNGRed
/*      */   extends AbstractRed
/*      */ {
/*      */   public static final int PNG_COLOR_GRAY = 0;
/*      */   public static final int PNG_COLOR_RGB = 2;
/*      */   public static final int PNG_COLOR_PALETTE = 3;
/*      */   public static final int PNG_COLOR_GRAY_ALPHA = 4;
/*      */   public static final int PNG_COLOR_RGB_ALPHA = 6;
/*      */   
/*      */   static class PNGChunk
/*      */   {
/*      */     int length;
/*      */     int type;
/*      */     byte[] data;
/*      */     int crc;
/*      */     String typeString;
/*      */     
/*      */     public PNGChunk(int length, int type, byte[] data, int crc) {
/*   73 */       this.length = length;
/*   74 */       this.type = type;
/*   75 */       this.data = data;
/*   76 */       this.crc = crc;
/*      */       
/*   78 */       this.typeString = "";
/*   79 */       this.typeString += (char)(type >> 24);
/*   80 */       this.typeString += (char)(type >> 16 & 0xFF);
/*   81 */       this.typeString += (char)(type >> 8 & 0xFF);
/*   82 */       this.typeString += (char)(type & 0xFF);
/*      */     }
/*      */ 
/*      */     
/*   86 */     public int getLength() { return this.length; }
/*      */ 
/*      */ 
/*      */     
/*   90 */     public int getType() { return this.type; }
/*      */ 
/*      */ 
/*      */     
/*   94 */     public String getTypeString() { return this.typeString; }
/*      */ 
/*      */ 
/*      */     
/*   98 */     public byte[] getData() { return this.data; }
/*      */ 
/*      */ 
/*      */     
/*  102 */     public byte getByte(int offset) { return this.data[offset]; }
/*      */ 
/*      */ 
/*      */     
/*  106 */     public int getInt1(int offset) { return this.data[offset] & 0xFF; }
/*      */ 
/*      */ 
/*      */     
/*  110 */     public int getInt2(int offset) { return (this.data[offset] & 0xFF) << 8 | this.data[offset + 1] & 0xFF; }
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  115 */     public int getInt4(int offset) { return (this.data[offset] & 0xFF) << 24 | (this.data[offset + 1] & 0xFF) << 16 | (this.data[offset + 2] & 0xFF) << 8 | this.data[offset + 3] & 0xFF; }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     public String getString4(int offset) {
/*  122 */       s = new String();
/*  123 */       s = s + (char)this.data[offset];
/*  124 */       s = s + (char)this.data[offset + 1];
/*  125 */       s = s + (char)this.data[offset + 2];
/*  126 */       return s + (char)this.data[offset + 3];
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  131 */     public boolean isType(String typeName) { return this.typeString.equals(typeName); }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  141 */   private static final String[] colorTypeNames = { "Grayscale", "Error", "Truecolor", "Index", "Grayscale with alpha", "Error", "Truecolor with alpha" };
/*      */   
/*      */   public static final int PNG_FILTER_NONE = 0;
/*      */   
/*      */   public static final int PNG_FILTER_SUB = 1;
/*      */   
/*      */   public static final int PNG_FILTER_UP = 2;
/*      */   
/*      */   public static final int PNG_FILTER_AVERAGE = 3;
/*      */   
/*      */   public static final int PNG_FILTER_PAETH = 4;
/*  152 */   private int[][] bandOffsets = { null, { 0 }, { 0, 1 }, { 0, 1, 2 }, { 0, 1, 2, 3 } };
/*      */   
/*      */   private int bitDepth;
/*      */   
/*      */   private int colorType;
/*      */   
/*      */   private int compressionMethod;
/*      */   
/*      */   private int filterMethod;
/*      */   
/*      */   private int interlaceMethod;
/*      */   
/*      */   private int paletteEntries;
/*      */   
/*      */   private byte[] redPalette;
/*      */   
/*      */   private byte[] greenPalette;
/*      */   
/*      */   private byte[] bluePalette;
/*      */   
/*      */   private byte[] alphaPalette;
/*      */   
/*      */   private int bkgdRed;
/*      */   
/*      */   private int bkgdGreen;
/*      */   
/*      */   private int bkgdBlue;
/*      */   private int grayTransparentAlpha;
/*      */   private int redTransparentAlpha;
/*      */   private int greenTransparentAlpha;
/*      */   private int blueTransparentAlpha;
/*      */   private int maxOpacity;
/*  184 */   private int[] significantBits = null;
/*      */ 
/*      */ 
/*      */   
/*      */   private boolean suppressAlpha = false;
/*      */ 
/*      */ 
/*      */   
/*      */   private boolean expandPalette = false;
/*      */ 
/*      */   
/*      */   private boolean output8BitGray = false;
/*      */ 
/*      */   
/*      */   private boolean outputHasAlphaPalette = false;
/*      */ 
/*      */   
/*      */   private boolean performGammaCorrection = false;
/*      */ 
/*      */   
/*      */   private boolean expandGrayAlpha = false;
/*      */ 
/*      */   
/*      */   private boolean generateEncodeParam = false;
/*      */ 
/*      */   
/*  210 */   private PNGDecodeParam decodeParam = null;
/*      */ 
/*      */   
/*  213 */   private PNGEncodeParam encodeParam = null;
/*      */   
/*      */   private boolean emitProperties = true;
/*      */   
/*  217 */   private float fileGamma = 0.45455F;
/*      */   
/*  219 */   private float userExponent = 1.0F;
/*      */   
/*  221 */   private float displayExponent = 2.2F;
/*      */   
/*  223 */   private float[] chromaticity = null;
/*      */   
/*  225 */   private int sRGBRenderingIntent = -1;
/*      */ 
/*      */   
/*  228 */   private int postProcess = 0;
/*      */ 
/*      */ 
/*      */   
/*      */   private static final int POST_NONE = 0;
/*      */ 
/*      */ 
/*      */   
/*      */   private static final int POST_GAMMA = 1;
/*      */ 
/*      */ 
/*      */   
/*      */   private static final int POST_GRAY_LUT = 2;
/*      */ 
/*      */ 
/*      */   
/*      */   private static final int POST_GRAY_LUT_ADD_TRANS = 3;
/*      */ 
/*      */ 
/*      */   
/*      */   private static final int POST_PALETTE_TO_RGB = 4;
/*      */ 
/*      */   
/*      */   private static final int POST_PALETTE_TO_RGBA = 5;
/*      */ 
/*      */   
/*      */   private static final int POST_ADD_GRAY_TRANS = 6;
/*      */ 
/*      */   
/*      */   private static final int POST_ADD_RGB_TRANS = 7;
/*      */ 
/*      */   
/*      */   private static final int POST_REMOVE_GRAY_TRANS = 8;
/*      */ 
/*      */   
/*      */   private static final int POST_REMOVE_RGB_TRANS = 9;
/*      */ 
/*      */   
/*      */   private static final int POST_EXP_MASK = 16;
/*      */ 
/*      */   
/*      */   private static final int POST_GRAY_ALPHA_EXP = 16;
/*      */ 
/*      */   
/*      */   private static final int POST_GAMMA_EXP = 17;
/*      */ 
/*      */   
/*      */   private static final int POST_GRAY_LUT_ADD_TRANS_EXP = 19;
/*      */ 
/*      */   
/*      */   private static final int POST_ADD_GRAY_TRANS_EXP = 22;
/*      */ 
/*      */   
/*  281 */   private List streamVec = new ArrayList();
/*      */   
/*      */   private DataInputStream dataStream;
/*      */   
/*      */   private int bytesPerPixel;
/*      */   
/*      */   private int inputBands;
/*      */   private int outputBands;
/*  289 */   private int chunkIndex = 0;
/*      */   
/*  291 */   private List textKeys = new ArrayList();
/*  292 */   private List textStrings = new ArrayList();
/*      */   
/*  294 */   private List ztextKeys = new ArrayList();
/*  295 */   private List ztextStrings = new ArrayList();
/*      */   
/*      */   private WritableRaster theTile;
/*      */   
/*      */   private Rectangle bounds;
/*      */   
/*  301 */   private Map properties = new HashMap();
/*      */ 
/*      */   
/*  304 */   private int[] gammaLut = null;
/*      */   
/*      */   private void initGammaLut(int bits) {
/*  307 */     double exp = this.userExponent / (this.fileGamma * this.displayExponent);
/*  308 */     int numSamples = 1 << bits;
/*  309 */     int maxOutSample = (bits == 16) ? 65535 : 255;
/*      */     
/*  311 */     this.gammaLut = new int[numSamples];
/*  312 */     for (int i = 0; i < numSamples; i++) {
/*  313 */       double gbright = i / (numSamples - 1);
/*  314 */       double gamma = Math.pow(gbright, exp);
/*  315 */       int igamma = (int)(gamma * maxOutSample + 0.5D);
/*  316 */       if (igamma > maxOutSample) {
/*  317 */         igamma = maxOutSample;
/*      */       }
/*  319 */       this.gammaLut[i] = igamma;
/*      */     } 
/*      */   }
/*      */   
/*  323 */   private final byte[][] expandBits = { null, { 0, -1 }, { 0, 85, -86, -1 }, null, { 0, 17, 34, 51, 68, 85, 102, 119, -120, -103, -86, -69, -52, -35, -18, -1 } };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  334 */   private int[] grayLut = null;
/*      */   
/*      */   private void initGrayLut(int bits) {
/*  337 */     int len = 1 << bits;
/*  338 */     this.grayLut = new int[len];
/*      */     
/*  340 */     if (this.performGammaCorrection) {
/*  341 */       System.arraycopy(this.gammaLut, 0, this.grayLut, 0, len);
/*      */     } else {
/*  343 */       for (int i = 0; i < len; i++) {
/*  344 */         this.grayLut[i] = this.expandBits[bits][i];
/*      */       }
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*  350 */   public PNGRed(InputStream stream) throws IOException { this(stream, null); }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public PNGRed(InputStream stream, PNGDecodeParam decodeParam) throws IOException {
/*  356 */     if (!stream.markSupported()) {
/*  357 */       stream = new BufferedInputStream(stream);
/*      */     }
/*  359 */     DataInputStream distream = new DataInputStream(stream);
/*      */     
/*  361 */     if (decodeParam == null) {
/*  362 */       decodeParam = new PNGDecodeParam();
/*      */     }
/*  364 */     this.decodeParam = decodeParam;
/*      */ 
/*      */     
/*  367 */     this.suppressAlpha = decodeParam.getSuppressAlpha();
/*  368 */     this.expandPalette = decodeParam.getExpandPalette();
/*  369 */     this.output8BitGray = decodeParam.getOutput8BitGray();
/*  370 */     this.expandGrayAlpha = decodeParam.getExpandGrayAlpha();
/*  371 */     if (decodeParam.getPerformGammaCorrection()) {
/*  372 */       this.userExponent = decodeParam.getUserExponent();
/*  373 */       this.displayExponent = decodeParam.getDisplayExponent();
/*  374 */       this.performGammaCorrection = true;
/*  375 */       this.output8BitGray = true;
/*      */     } 
/*  377 */     this.generateEncodeParam = decodeParam.getGenerateEncodeParam();
/*      */     
/*  379 */     if (this.emitProperties) {
/*  380 */       this.properties.put("file_type", "PNG v. 1.0");
/*      */     }
/*      */     
/*      */     try {
/*  384 */       long magic = distream.readLong();
/*  385 */       if (magic != -8552249625308161526L) {
/*  386 */         String msg = PropertyUtil.getString("PNGImageDecoder0");
/*  387 */         throw new RuntimeException(msg);
/*      */       } 
/*      */     } catch (Exception e) {
/*  390 */       e.printStackTrace();
/*  391 */       String msg = PropertyUtil.getString("PNGImageDecoder1");
/*  392 */       throw new RuntimeException(msg);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/*      */       while (true) {
/*  399 */         String chunkType = getChunkType(distream);
/*  400 */         if (chunkType.equals("IHDR")) {
/*  401 */           PNGChunk chunk = readChunk(distream);
/*  402 */           parse_IHDR_chunk(chunk); continue;
/*  403 */         }  if (chunkType.equals("PLTE")) {
/*  404 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  405 */           parse_PLTE_chunk(pNGChunk1); continue;
/*  406 */         }  if (chunkType.equals("IDAT")) {
/*  407 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  408 */           this.streamVec.add(new ByteArrayInputStream(pNGChunk1.getData())); continue;
/*  409 */         }  if (chunkType.equals("IEND")) {
/*  410 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  411 */           parse_IEND_chunk(pNGChunk1); break;
/*      */         } 
/*  413 */         if (chunkType.equals("bKGD")) {
/*  414 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  415 */           parse_bKGD_chunk(pNGChunk1); continue;
/*  416 */         }  if (chunkType.equals("cHRM")) {
/*  417 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  418 */           parse_cHRM_chunk(pNGChunk1); continue;
/*  419 */         }  if (chunkType.equals("gAMA")) {
/*  420 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  421 */           parse_gAMA_chunk(pNGChunk1); continue;
/*  422 */         }  if (chunkType.equals("hIST")) {
/*  423 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  424 */           parse_hIST_chunk(pNGChunk1); continue;
/*  425 */         }  if (chunkType.equals("iCCP")) {
/*  426 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  427 */           parse_iCCP_chunk(pNGChunk1); continue;
/*  428 */         }  if (chunkType.equals("pHYs")) {
/*  429 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  430 */           parse_pHYs_chunk(pNGChunk1); continue;
/*  431 */         }  if (chunkType.equals("sBIT")) {
/*  432 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  433 */           parse_sBIT_chunk(pNGChunk1); continue;
/*  434 */         }  if (chunkType.equals("sRGB")) {
/*  435 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  436 */           parse_sRGB_chunk(pNGChunk1); continue;
/*  437 */         }  if (chunkType.equals("tEXt")) {
/*  438 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  439 */           parse_tEXt_chunk(pNGChunk1); continue;
/*  440 */         }  if (chunkType.equals("tIME")) {
/*  441 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  442 */           parse_tIME_chunk(pNGChunk1); continue;
/*  443 */         }  if (chunkType.equals("tRNS")) {
/*  444 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  445 */           parse_tRNS_chunk(pNGChunk1); continue;
/*  446 */         }  if (chunkType.equals("zTXt")) {
/*  447 */           PNGChunk pNGChunk1 = readChunk(distream);
/*  448 */           parse_zTXt_chunk(pNGChunk1); continue;
/*      */         } 
/*  450 */         PNGChunk pNGChunk = readChunk(distream);
/*      */ 
/*      */         
/*  453 */         String type = pNGChunk.getTypeString();
/*  454 */         byte[] data = pNGChunk.getData();
/*  455 */         if (this.encodeParam != null) {
/*  456 */           this.encodeParam.addPrivateChunk(type, data);
/*      */         }
/*  458 */         if (this.emitProperties) {
/*  459 */           String key = "chunk_" + this.chunkIndex++ + ':' + type;
/*  460 */           this.properties.put(key.toLowerCase(), data);
/*      */         } 
/*      */       } 
/*      */     } catch (Exception e) {
/*  464 */       e.printStackTrace();
/*  465 */       String msg = PropertyUtil.getString("PNGImageDecoder2");
/*  466 */       throw new RuntimeException(msg);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  472 */     if (this.significantBits == null) {
/*  473 */       this.significantBits = new int[this.inputBands];
/*  474 */       for (int i = 0; i < this.inputBands; i++) {
/*  475 */         this.significantBits[i] = this.bitDepth;
/*      */       }
/*      */       
/*  478 */       if (this.emitProperties) {
/*  479 */         this.properties.put("significant_bits", this.significantBits);
/*      */       }
/*      */     } 
/*      */   }
/*      */   
/*      */   private static String getChunkType(DataInputStream distream) {
/*      */     try {
/*  486 */       distream.mark(8);
/*  487 */       distream.readInt();
/*  488 */       int type = distream.readInt();
/*  489 */       distream.reset();
/*      */       
/*  491 */       return "" + (char)(type >> 24 & 0xFF) + (char)(type >> 16 & 0xFF) + (char)(type >> 8 & 0xFF) + (char)(type & 0xFF);
/*      */ 
/*      */     
/*      */     }
/*      */     catch (Exception e) {
/*      */ 
/*      */       
/*  498 */       e.printStackTrace();
/*  499 */       return null;
/*      */     } 
/*      */   }
/*      */   
/*      */   private static PNGChunk readChunk(DataInputStream distream) {
/*      */     try {
/*  505 */       int length = distream.readInt();
/*  506 */       int type = distream.readInt();
/*  507 */       byte[] data = new byte[length];
/*  508 */       distream.readFully(data);
/*  509 */       int crc = distream.readInt();
/*      */       
/*  511 */       return new PNGChunk(length, type, data, crc);
/*      */     } catch (Exception e) {
/*  513 */       e.printStackTrace();
/*  514 */       return null;
/*      */     } 
/*      */   }
/*      */   
/*      */   private void parse_IHDR_chunk(PNGChunk chunk) {
/*  519 */     int width = chunk.getInt4(0);
/*  520 */     int height = chunk.getInt4(4);
/*      */     
/*  522 */     this.bounds = new Rectangle(0, 0, width, height);
/*      */     
/*  524 */     this.bitDepth = chunk.getInt1(8);
/*      */     
/*  526 */     int validMask = 65814;
/*  527 */     if ((1 << this.bitDepth & validMask) == 0) {
/*      */       
/*  529 */       String msg = PropertyUtil.getString("PNGImageDecoder3");
/*  530 */       throw new RuntimeException(msg);
/*      */     } 
/*  532 */     this.maxOpacity = (1 << this.bitDepth) - 1;
/*      */     
/*  534 */     this.colorType = chunk.getInt1(9);
/*  535 */     if (this.colorType != 0 && this.colorType != 2 && this.colorType != 3 && this.colorType != 4 && this.colorType != 6)
/*      */     {
/*      */ 
/*      */ 
/*      */       
/*  540 */       System.out.println(PropertyUtil.getString("PNGImageDecoder4"));
/*      */     }
/*      */     
/*  543 */     if (this.colorType == 2 && this.bitDepth < 8) {
/*      */       
/*  545 */       String msg = PropertyUtil.getString("PNGImageDecoder5");
/*  546 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/*  549 */     if (this.colorType == 3 && this.bitDepth == 16) {
/*      */       
/*  551 */       String msg = PropertyUtil.getString("PNGImageDecoder6");
/*  552 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/*  555 */     if (this.colorType == 4 && this.bitDepth < 8) {
/*      */       
/*  557 */       String msg = PropertyUtil.getString("PNGImageDecoder7");
/*  558 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/*  561 */     if (this.colorType == 6 && this.bitDepth < 8) {
/*      */       
/*  563 */       String msg = PropertyUtil.getString("PNGImageDecoder8");
/*  564 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/*  567 */     if (this.emitProperties) {
/*  568 */       this.properties.put("color_type", colorTypeNames[this.colorType]);
/*      */     }
/*      */     
/*  571 */     if (this.generateEncodeParam) {
/*  572 */       if (this.colorType == 3) {
/*  573 */         this.encodeParam = new PNGEncodeParam.Palette();
/*  574 */       } else if (this.colorType == 0 || this.colorType == 4) {
/*      */         
/*  576 */         this.encodeParam = new PNGEncodeParam.Gray();
/*      */       } else {
/*  578 */         this.encodeParam = new PNGEncodeParam.RGB();
/*      */       } 
/*  580 */       this.decodeParam.setEncodeParam(this.encodeParam);
/*      */     } 
/*      */     
/*  583 */     if (this.encodeParam != null) {
/*  584 */       this.encodeParam.setBitDepth(this.bitDepth);
/*      */     }
/*  586 */     if (this.emitProperties) {
/*  587 */       this.properties.put("bit_depth", new Integer(this.bitDepth));
/*      */     }
/*      */     
/*  590 */     if (this.performGammaCorrection) {
/*      */       
/*  592 */       float gamma = 0.45454544F * this.displayExponent / this.userExponent;
/*  593 */       if (this.encodeParam != null) {
/*  594 */         this.encodeParam.setGamma(gamma);
/*      */       }
/*  596 */       if (this.emitProperties) {
/*  597 */         this.properties.put("gamma", new Float(gamma));
/*      */       }
/*      */     } 
/*      */     
/*  601 */     this.compressionMethod = chunk.getInt1(10);
/*  602 */     if (this.compressionMethod != 0) {
/*      */       
/*  604 */       String msg = PropertyUtil.getString("PNGImageDecoder9");
/*  605 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/*  608 */     this.filterMethod = chunk.getInt1(11);
/*  609 */     if (this.filterMethod != 0) {
/*      */       
/*  611 */       String msg = PropertyUtil.getString("PNGImageDecoder10");
/*  612 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/*  615 */     this.interlaceMethod = chunk.getInt1(12);
/*  616 */     if (this.interlaceMethod == 0) {
/*  617 */       if (this.encodeParam != null) {
/*  618 */         this.encodeParam.setInterlacing(false);
/*      */       }
/*  620 */       if (this.emitProperties) {
/*  621 */         this.properties.put("interlace_method", "None");
/*      */       }
/*  623 */     } else if (this.interlaceMethod == 1) {
/*  624 */       if (this.encodeParam != null) {
/*  625 */         this.encodeParam.setInterlacing(true);
/*      */       }
/*  627 */       if (this.emitProperties) {
/*  628 */         this.properties.put("interlace_method", "Adam7");
/*      */       }
/*      */     } else {
/*      */       
/*  632 */       String msg = PropertyUtil.getString("PNGImageDecoder11");
/*  633 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/*  636 */     this.bytesPerPixel = (this.bitDepth == 16) ? 2 : 1;
/*      */     
/*  638 */     switch (this.colorType) {
/*      */       case 0:
/*  640 */         this.inputBands = 1;
/*  641 */         this.outputBands = 1;
/*      */         
/*  643 */         if (this.output8BitGray && this.bitDepth < 8) {
/*  644 */           this.postProcess = 2; break;
/*  645 */         }  if (this.performGammaCorrection) {
/*  646 */           this.postProcess = 1; break;
/*      */         } 
/*  648 */         this.postProcess = 0;
/*      */         break;
/*      */ 
/*      */       
/*      */       case 2:
/*  653 */         this.inputBands = 3;
/*  654 */         this.bytesPerPixel *= 3;
/*  655 */         this.outputBands = 3;
/*      */         
/*  657 */         if (this.performGammaCorrection) {
/*  658 */           this.postProcess = 1; break;
/*      */         } 
/*  660 */         this.postProcess = 0;
/*      */         break;
/*      */ 
/*      */       
/*      */       case 3:
/*  665 */         this.inputBands = 1;
/*  666 */         this.bytesPerPixel = 1;
/*  667 */         this.outputBands = this.expandPalette ? 3 : 1;
/*      */         
/*  669 */         if (this.expandPalette) {
/*  670 */           this.postProcess = 4; break;
/*      */         } 
/*  672 */         this.postProcess = 0;
/*      */         break;
/*      */ 
/*      */       
/*      */       case 4:
/*  677 */         this.inputBands = 2;
/*  678 */         this.bytesPerPixel *= 2;
/*      */         
/*  680 */         if (this.suppressAlpha) {
/*  681 */           this.outputBands = 1;
/*  682 */           this.postProcess = 8; break;
/*      */         } 
/*  684 */         if (this.performGammaCorrection) {
/*  685 */           this.postProcess = 1;
/*      */         } else {
/*  687 */           this.postProcess = 0;
/*      */         } 
/*  689 */         if (this.expandGrayAlpha) {
/*  690 */           this.postProcess |= 0x10;
/*  691 */           this.outputBands = 4; break;
/*      */         } 
/*  693 */         this.outputBands = 2;
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       case 6:
/*  699 */         this.inputBands = 4;
/*  700 */         this.bytesPerPixel *= 4;
/*  701 */         this.outputBands = !this.suppressAlpha ? 4 : 3;
/*      */         
/*  703 */         if (this.suppressAlpha) {
/*  704 */           this.postProcess = 9; break;
/*  705 */         }  if (this.performGammaCorrection) {
/*  706 */           this.postProcess = 1; break;
/*      */         } 
/*  708 */         this.postProcess = 0;
/*      */         break;
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*      */   private void parse_IEND_chunk(PNGChunk chunk) {
/*      */     ColorModel colorModel;
/*  716 */     int textLen = this.textKeys.size();
/*  717 */     String[] textArray = new String[2 * textLen];
/*  718 */     for (int i = 0; i < textLen; i++) {
/*  719 */       String key = (String)this.textKeys.get(i);
/*  720 */       String val = (String)this.textStrings.get(i);
/*  721 */       textArray[2 * i] = key;
/*  722 */       textArray[2 * i + 1] = val;
/*  723 */       if (this.emitProperties) {
/*  724 */         String uniqueKey = "text_" + i + ':' + key;
/*  725 */         this.properties.put(uniqueKey.toLowerCase(), val);
/*      */       } 
/*      */     } 
/*  728 */     if (this.encodeParam != null) {
/*  729 */       this.encodeParam.setText(textArray);
/*      */     }
/*      */ 
/*      */     
/*  733 */     int ztextLen = this.ztextKeys.size();
/*  734 */     String[] ztextArray = new String[2 * ztextLen];
/*  735 */     for (int i = 0; i < ztextLen; i++) {
/*  736 */       String key = (String)this.ztextKeys.get(i);
/*  737 */       String val = (String)this.ztextStrings.get(i);
/*  738 */       ztextArray[2 * i] = key;
/*  739 */       ztextArray[2 * i + 1] = val;
/*  740 */       if (this.emitProperties) {
/*  741 */         String uniqueKey = "ztext_" + i + ':' + key;
/*  742 */         this.properties.put(uniqueKey.toLowerCase(), val);
/*      */       } 
/*      */     } 
/*  745 */     if (this.encodeParam != null) {
/*  746 */       this.encodeParam.setCompressedText(ztextArray);
/*      */     }
/*      */ 
/*      */     
/*  750 */     InputStream seqStream = new SequenceInputStream(Collections.enumeration(this.streamVec));
/*      */     
/*  752 */     InputStream infStream = new InflaterInputStream(seqStream, new Inflater());
/*      */     
/*  754 */     this.dataStream = new DataInputStream(infStream);
/*      */ 
/*      */     
/*  757 */     int depth = this.bitDepth;
/*  758 */     if (this.colorType == 0 && this.bitDepth < 8 && this.output8BitGray)
/*      */     {
/*  760 */       depth = 8;
/*      */     }
/*  762 */     if (this.colorType == 3 && this.expandPalette) {
/*  763 */       depth = 8;
/*      */     }
/*  765 */     int width = this.bounds.width;
/*  766 */     int height = this.bounds.height;
/*      */     
/*  768 */     int bytesPerRow = (this.outputBands * width * depth + 7) / 8;
/*  769 */     int scanlineStride = (depth == 16) ? (bytesPerRow / 2) : bytesPerRow;
/*      */ 
/*      */     
/*  772 */     this.theTile = createRaster(width, height, this.outputBands, scanlineStride, depth);
/*      */ 
/*      */ 
/*      */     
/*  776 */     if (this.performGammaCorrection && this.gammaLut == null) {
/*  777 */       initGammaLut(this.bitDepth);
/*      */     }
/*  779 */     if (this.postProcess == 2 || this.postProcess == 3 || this.postProcess == 19)
/*      */     {
/*      */       
/*  782 */       initGrayLut(this.bitDepth);
/*      */     }
/*      */     
/*  785 */     decodeImage((this.interlaceMethod == 1));
/*  786 */     SampleModel sm = this.theTile.getSampleModel();
/*      */ 
/*      */     
/*  789 */     if (this.colorType == 3 && !this.expandPalette) {
/*  790 */       if (this.outputHasAlphaPalette) {
/*  791 */         colorModel = new IndexColorModel(this.bitDepth, this.paletteEntries, this.redPalette, this.greenPalette, this.bluePalette, this.alphaPalette);
/*      */ 
/*      */       
/*      */       }
/*      */       else {
/*      */ 
/*      */         
/*  798 */         colorModel = new IndexColorModel(this.bitDepth, this.paletteEntries, this.redPalette, this.greenPalette, this.bluePalette);
/*      */       
/*      */       }
/*      */ 
/*      */     
/*      */     }
/*  804 */     else if (this.colorType == 0 && this.bitDepth < 8 && !this.output8BitGray) {
/*      */       
/*  806 */       byte[] palette = this.expandBits[this.bitDepth];
/*  807 */       colorModel = new IndexColorModel(this.bitDepth, palette.length, palette, palette, palette);
/*      */     
/*      */     }
/*      */     else {
/*      */ 
/*      */       
/*  813 */       colorModel = createComponentColorModel(sm);
/*      */     } 
/*      */ 
/*      */     
/*  817 */     init((CachableRed)null, this.bounds, colorModel, sm, 0, 0, this.properties);
/*      */   }
/*      */   
/*  820 */   private static final int[] GrayBits8 = { 8 };
/*  821 */   private static final ComponentColorModel colorModelGray8 = new ComponentColorModel(ColorSpace.getInstance(1003), GrayBits8, false, false, 1, 0);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  827 */   private static final int[] GrayAlphaBits8 = { 8, 8 };
/*  828 */   private static final ComponentColorModel colorModelGrayAlpha8 = new ComponentColorModel(ColorSpace.getInstance(1003), GrayAlphaBits8, true, false, 3, 0);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  834 */   private static final int[] GrayBits16 = { 16 };
/*  835 */   private static final ComponentColorModel colorModelGray16 = new ComponentColorModel(ColorSpace.getInstance(1003), GrayBits16, false, false, 1, 1);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  841 */   private static final int[] GrayAlphaBits16 = { 16, 16 };
/*  842 */   private static final ComponentColorModel colorModelGrayAlpha16 = new ComponentColorModel(ColorSpace.getInstance(1003), GrayAlphaBits16, true, false, 3, 1);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  848 */   private static final int[] GrayBits32 = { 32 };
/*  849 */   private static final ComponentColorModel colorModelGray32 = new ComponentColorModel(ColorSpace.getInstance(1003), GrayBits32, false, false, 1, 3);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  855 */   private static final int[] GrayAlphaBits32 = { 32, 32 };
/*  856 */   private static final ComponentColorModel colorModelGrayAlpha32 = new ComponentColorModel(ColorSpace.getInstance(1003), GrayAlphaBits32, true, false, 3, 3);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  862 */   private static final int[] RGBBits8 = { 8, 8, 8 };
/*  863 */   private static final ComponentColorModel colorModelRGB8 = new ComponentColorModel(ColorSpace.getInstance(1000), RGBBits8, false, false, 1, 0);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  869 */   private static final int[] RGBABits8 = { 8, 8, 8, 8 };
/*  870 */   private static final ComponentColorModel colorModelRGBA8 = new ComponentColorModel(ColorSpace.getInstance(1000), RGBABits8, true, false, 3, 0);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  876 */   private static final int[] RGBBits16 = { 16, 16, 16 };
/*  877 */   private static final ComponentColorModel colorModelRGB16 = new ComponentColorModel(ColorSpace.getInstance(1000), RGBBits16, false, false, 1, 1);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  883 */   private static final int[] RGBABits16 = { 16, 16, 16, 16 };
/*  884 */   private static final ComponentColorModel colorModelRGBA16 = new ComponentColorModel(ColorSpace.getInstance(1000), RGBABits16, true, false, 3, 1);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  890 */   private static final int[] RGBBits32 = { 32, 32, 32 };
/*  891 */   private static final ComponentColorModel colorModelRGB32 = new ComponentColorModel(ColorSpace.getInstance(1000), RGBBits32, false, false, 1, 3);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  897 */   private static final int[] RGBABits32 = { 32, 32, 32, 32 };
/*  898 */   private static final ComponentColorModel colorModelRGBA32 = new ComponentColorModel(ColorSpace.getInstance(1000), RGBABits32, true, false, 3, 3);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public static ColorModel createComponentColorModel(SampleModel sm) {
/*  914 */     int type = sm.getDataType();
/*  915 */     int bands = sm.getNumBands();
/*  916 */     ComponentColorModel cm = null;
/*      */     
/*  918 */     if (type == 0) {
/*  919 */       switch (bands) {
/*      */         case 1:
/*  921 */           cm = colorModelGray8;
/*      */           break;
/*      */         case 2:
/*  924 */           cm = colorModelGrayAlpha8;
/*      */           break;
/*      */         case 3:
/*  927 */           cm = colorModelRGB8;
/*      */           break;
/*      */         case 4:
/*  930 */           cm = colorModelRGBA8;
/*      */           break;
/*      */       } 
/*  933 */     } else if (type == 1) {
/*  934 */       switch (bands) {
/*      */         case 1:
/*  936 */           cm = colorModelGray16;
/*      */           break;
/*      */         case 2:
/*  939 */           cm = colorModelGrayAlpha16;
/*      */           break;
/*      */         case 3:
/*  942 */           cm = colorModelRGB16;
/*      */           break;
/*      */         case 4:
/*  945 */           cm = colorModelRGBA16;
/*      */           break;
/*      */       } 
/*  948 */     } else if (type == 3) {
/*  949 */       switch (bands) {
/*      */         case 1:
/*  951 */           cm = colorModelGray32;
/*      */           break;
/*      */         case 2:
/*  954 */           cm = colorModelGrayAlpha32;
/*      */           break;
/*      */         case 3:
/*  957 */           cm = colorModelRGB32;
/*      */           break;
/*      */         case 4:
/*  960 */           cm = colorModelRGBA32;
/*      */           break;
/*      */       } 
/*      */     
/*      */     } 
/*  965 */     return cm;
/*      */   }
/*      */   
/*      */   private void parse_PLTE_chunk(PNGChunk chunk) {
/*  969 */     this.paletteEntries = chunk.getLength() / 3;
/*  970 */     this.redPalette = new byte[this.paletteEntries];
/*  971 */     this.greenPalette = new byte[this.paletteEntries];
/*  972 */     this.bluePalette = new byte[this.paletteEntries];
/*      */     
/*  974 */     int pltIndex = 0;
/*      */ 
/*      */     
/*  977 */     if (this.performGammaCorrection) {
/*  978 */       if (this.gammaLut == null) {
/*  979 */         initGammaLut((this.bitDepth == 16) ? 16 : 8);
/*      */       }
/*      */       
/*  982 */       for (int i = 0; i < this.paletteEntries; i++) {
/*  983 */         byte r = chunk.getByte(pltIndex++);
/*  984 */         byte g = chunk.getByte(pltIndex++);
/*  985 */         byte b = chunk.getByte(pltIndex++);
/*      */         
/*  987 */         this.redPalette[i] = (byte)this.gammaLut[r & 0xFF];
/*  988 */         this.greenPalette[i] = (byte)this.gammaLut[g & 0xFF];
/*  989 */         this.bluePalette[i] = (byte)this.gammaLut[b & 0xFF];
/*      */       } 
/*      */     } else {
/*  992 */       for (int i = 0; i < this.paletteEntries; i++) {
/*  993 */         this.redPalette[i] = chunk.getByte(pltIndex++);
/*  994 */         this.greenPalette[i] = chunk.getByte(pltIndex++);
/*  995 */         this.bluePalette[i] = chunk.getByte(pltIndex++);
/*      */       } 
/*      */     }  } private void parse_bKGD_chunk(PNGChunk chunk) {
/*      */     int[] bkgdRGB;
/*      */     int bkgdGray;
/*      */     int bkgdIndex;
/* 1001 */     switch (this.colorType) {
/*      */       case 3:
/* 1003 */         bkgdIndex = chunk.getByte(0) & 0xFF;
/*      */         
/* 1005 */         this.bkgdRed = this.redPalette[bkgdIndex] & 0xFF;
/* 1006 */         this.bkgdGreen = this.greenPalette[bkgdIndex] & 0xFF;
/* 1007 */         this.bkgdBlue = this.bluePalette[bkgdIndex] & 0xFF;
/*      */         
/* 1009 */         if (this.encodeParam != null) {
/* 1010 */           ((PNGEncodeParam.Palette)this.encodeParam).setBackgroundPaletteIndex(bkgdIndex);
/*      */         }
/*      */         break;
/*      */       case 0:
/*      */       case 4:
/* 1015 */         bkgdGray = chunk.getInt2(0);
/* 1016 */         this.bkgdRed = this.bkgdGreen = this.bkgdBlue = bkgdGray;
/*      */         
/* 1018 */         if (this.encodeParam != null) {
/* 1019 */           ((PNGEncodeParam.Gray)this.encodeParam).setBackgroundGray(bkgdGray);
/*      */         }
/*      */         break;
/*      */       case 2:
/*      */       case 6:
/* 1024 */         this.bkgdRed = chunk.getInt2(0);
/* 1025 */         this.bkgdGreen = chunk.getInt2(2);
/* 1026 */         this.bkgdBlue = chunk.getInt2(4);
/*      */         
/* 1028 */         bkgdRGB = new int[3];
/* 1029 */         bkgdRGB[0] = this.bkgdRed;
/* 1030 */         bkgdRGB[1] = this.bkgdGreen;
/* 1031 */         bkgdRGB[2] = this.bkgdBlue;
/* 1032 */         if (this.encodeParam != null) {
/* 1033 */           ((PNGEncodeParam.RGB)this.encodeParam).setBackgroundRGB(bkgdRGB);
/*      */         }
/*      */         break;
/*      */     } 
/*      */ 
/*      */     
/* 1039 */     if (this.emitProperties) {
/* 1040 */       int r = 0, g = 0, b = 0;
/* 1041 */       if (this.colorType == 3 || this.bitDepth == 8) {
/* 1042 */         r = this.bkgdRed;
/* 1043 */         g = this.bkgdGreen;
/* 1044 */         b = this.bkgdBlue;
/* 1045 */       } else if (this.bitDepth < 8) {
/* 1046 */         r = this.expandBits[this.bitDepth][this.bkgdRed];
/* 1047 */         g = this.expandBits[this.bitDepth][this.bkgdGreen];
/* 1048 */         b = this.expandBits[this.bitDepth][this.bkgdBlue];
/* 1049 */       } else if (this.bitDepth == 16) {
/* 1050 */         r = this.bkgdRed >> 8;
/* 1051 */         g = this.bkgdGreen >> 8;
/* 1052 */         b = this.bkgdBlue >> 8;
/*      */       } 
/* 1054 */       this.properties.put("background_color", new Color(r, g, b));
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*      */   private void parse_cHRM_chunk(PNGChunk chunk) {
/* 1060 */     if (this.sRGBRenderingIntent != -1) {
/*      */       return;
/*      */     }
/*      */     
/* 1064 */     this.chromaticity = new float[8];
/* 1065 */     this.chromaticity[0] = chunk.getInt4(0) / 100000.0F;
/* 1066 */     this.chromaticity[1] = chunk.getInt4(4) / 100000.0F;
/* 1067 */     this.chromaticity[2] = chunk.getInt4(8) / 100000.0F;
/* 1068 */     this.chromaticity[3] = chunk.getInt4(12) / 100000.0F;
/* 1069 */     this.chromaticity[4] = chunk.getInt4(16) / 100000.0F;
/* 1070 */     this.chromaticity[5] = chunk.getInt4(20) / 100000.0F;
/* 1071 */     this.chromaticity[6] = chunk.getInt4(24) / 100000.0F;
/* 1072 */     this.chromaticity[7] = chunk.getInt4(28) / 100000.0F;
/*      */     
/* 1074 */     if (this.encodeParam != null) {
/* 1075 */       this.encodeParam.setChromaticity(this.chromaticity);
/*      */     }
/* 1077 */     if (this.emitProperties) {
/* 1078 */       this.properties.put("white_point_x", new Float(this.chromaticity[0]));
/* 1079 */       this.properties.put("white_point_y", new Float(this.chromaticity[1]));
/* 1080 */       this.properties.put("red_x", new Float(this.chromaticity[2]));
/* 1081 */       this.properties.put("red_y", new Float(this.chromaticity[3]));
/* 1082 */       this.properties.put("green_x", new Float(this.chromaticity[4]));
/* 1083 */       this.properties.put("green_y", new Float(this.chromaticity[5]));
/* 1084 */       this.properties.put("blue_x", new Float(this.chromaticity[6]));
/* 1085 */       this.properties.put("blue_y", new Float(this.chromaticity[7]));
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*      */   private void parse_gAMA_chunk(PNGChunk chunk) {
/* 1091 */     if (this.sRGBRenderingIntent != -1) {
/*      */       return;
/*      */     }
/*      */     
/* 1095 */     this.fileGamma = chunk.getInt4(0) / 100000.0F;
/*      */     
/* 1097 */     float exp = this.performGammaCorrection ? (this.displayExponent / this.userExponent) : 1.0F;
/*      */     
/* 1099 */     if (this.encodeParam != null) {
/* 1100 */       this.encodeParam.setGamma(this.fileGamma * exp);
/*      */     }
/* 1102 */     if (this.emitProperties) {
/* 1103 */       this.properties.put("gamma", new Float(this.fileGamma * exp));
/*      */     }
/*      */   }
/*      */   
/*      */   private void parse_hIST_chunk(PNGChunk chunk) {
/* 1108 */     if (this.redPalette == null) {
/* 1109 */       String msg = PropertyUtil.getString("PNGImageDecoder18");
/* 1110 */       throw new RuntimeException(msg);
/*      */     } 
/*      */     
/* 1113 */     int length = this.redPalette.length;
/* 1114 */     int[] hist = new int[length];
/* 1115 */     for (int i = 0; i < length; i++) {
/* 1116 */       hist[i] = chunk.getInt2(2 * i);
/*      */     }
/*      */     
/* 1119 */     if (this.encodeParam != null) {
/* 1120 */       this.encodeParam.setPaletteHistogram(hist);
/*      */     }
/*      */   }
/*      */   
/*      */   private void parse_iCCP_chunk(PNGChunk chunk) {
/* 1125 */     String name = "";
/*      */ 
/*      */     
/* 1128 */     int textIndex = 0; byte b;
/* 1129 */     while ((b = chunk.getByte(textIndex++)) != 0) {
/* 1130 */       name = name + (char)b;
/*      */     }
/*      */   }
/*      */   
/*      */   private void parse_pHYs_chunk(PNGChunk chunk) {
/* 1135 */     int xPixelsPerUnit = chunk.getInt4(0);
/* 1136 */     int yPixelsPerUnit = chunk.getInt4(4);
/* 1137 */     int unitSpecifier = chunk.getInt1(8);
/*      */     
/* 1139 */     if (this.encodeParam != null) {
/* 1140 */       this.encodeParam.setPhysicalDimension(xPixelsPerUnit, yPixelsPerUnit, unitSpecifier);
/*      */     }
/*      */ 
/*      */     
/* 1144 */     if (this.emitProperties) {
/* 1145 */       this.properties.put("x_pixels_per_unit", new Integer(xPixelsPerUnit));
/* 1146 */       this.properties.put("y_pixels_per_unit", new Integer(yPixelsPerUnit));
/* 1147 */       this.properties.put("pixel_aspect_ratio", new Float(xPixelsPerUnit / yPixelsPerUnit));
/*      */       
/* 1149 */       if (unitSpecifier == 1) {
/* 1150 */         this.properties.put("pixel_units", "Meters");
/* 1151 */       } else if (unitSpecifier != 0) {
/*      */         
/* 1153 */         String msg = PropertyUtil.getString("PNGImageDecoder12");
/* 1154 */         throw new RuntimeException(msg);
/*      */       } 
/*      */     } 
/*      */   }
/*      */   
/*      */   private void parse_sBIT_chunk(PNGChunk chunk) {
/* 1160 */     if (this.colorType == 3) {
/* 1161 */       this.significantBits = new int[3];
/*      */     } else {
/* 1163 */       this.significantBits = new int[this.inputBands];
/*      */     } 
/* 1165 */     for (int i = 0; i < this.significantBits.length; i++) {
/* 1166 */       int bits = chunk.getByte(i);
/* 1167 */       int depth = (this.colorType == 3) ? 8 : this.bitDepth;
/* 1168 */       if (bits <= 0 || bits > depth) {
/*      */ 
/*      */         
/* 1171 */         String msg = PropertyUtil.getString("PNGImageDecoder13");
/* 1172 */         throw new RuntimeException(msg);
/*      */       } 
/* 1174 */       this.significantBits[i] = bits;
/*      */     } 
/*      */     
/* 1177 */     if (this.encodeParam != null) {
/* 1178 */       this.encodeParam.setSignificantBits(this.significantBits);
/*      */     }
/* 1180 */     if (this.emitProperties) {
/* 1181 */       this.properties.put("significant_bits", this.significantBits);
/*      */     }
/*      */   }
/*      */   
/*      */   private void parse_sRGB_chunk(PNGChunk chunk) {
/* 1186 */     this.sRGBRenderingIntent = chunk.getByte(0);
/*      */ 
/*      */ 
/*      */     
/* 1190 */     this.fileGamma = 0.45455F;
/*      */     
/* 1192 */     this.chromaticity = new float[8];
/* 1193 */     this.chromaticity[0] = 3.127F;
/* 1194 */     this.chromaticity[1] = 3.29F;
/* 1195 */     this.chromaticity[2] = 6.4F;
/* 1196 */     this.chromaticity[3] = 3.3F;
/* 1197 */     this.chromaticity[4] = 3.0F;
/* 1198 */     this.chromaticity[5] = 6.0F;
/* 1199 */     this.chromaticity[6] = 1.5F;
/* 1200 */     this.chromaticity[7] = 0.6F;
/*      */     
/* 1202 */     if (this.performGammaCorrection) {
/*      */       
/* 1204 */       float gamma = this.fileGamma * this.displayExponent / this.userExponent;
/* 1205 */       if (this.encodeParam != null) {
/* 1206 */         this.encodeParam.setGamma(gamma);
/* 1207 */         this.encodeParam.setChromaticity(this.chromaticity);
/*      */       } 
/* 1209 */       if (this.emitProperties) {
/* 1210 */         this.properties.put("gamma", new Float(gamma));
/* 1211 */         this.properties.put("white_point_x", new Float(this.chromaticity[0]));
/* 1212 */         this.properties.put("white_point_y", new Float(this.chromaticity[1]));
/* 1213 */         this.properties.put("red_x", new Float(this.chromaticity[2]));
/* 1214 */         this.properties.put("red_y", new Float(this.chromaticity[3]));
/* 1215 */         this.properties.put("green_x", new Float(this.chromaticity[4]));
/* 1216 */         this.properties.put("green_y", new Float(this.chromaticity[5]));
/* 1217 */         this.properties.put("blue_x", new Float(this.chromaticity[6]));
/* 1218 */         this.properties.put("blue_y", new Float(this.chromaticity[7]));
/*      */       } 
/*      */     } 
/*      */   }
/*      */   
/*      */   private void parse_tEXt_chunk(PNGChunk chunk) {
/* 1224 */     StringBuffer key = new StringBuffer();
/* 1225 */     StringBuffer value = new StringBuffer();
/*      */ 
/*      */     
/* 1228 */     int textIndex = 0; byte b;
/* 1229 */     while ((b = chunk.getByte(textIndex++)) != 0) {
/* 1230 */       key.append((char)b);
/*      */     }
/*      */     
/* 1233 */     for (int i = textIndex; i < chunk.getLength(); i++) {
/* 1234 */       value.append((char)chunk.getByte(i));
/*      */     }
/*      */     
/* 1237 */     this.textKeys.add(key.toString());
/* 1238 */     this.textStrings.add(value.toString());
/*      */   }
/*      */   
/*      */   private void parse_tIME_chunk(PNGChunk chunk) {
/* 1242 */     int year = chunk.getInt2(0);
/* 1243 */     int month = chunk.getInt1(2) - 1;
/* 1244 */     int day = chunk.getInt1(3);
/* 1245 */     int hour = chunk.getInt1(4);
/* 1246 */     int minute = chunk.getInt1(5);
/* 1247 */     int second = chunk.getInt1(6);
/*      */     
/* 1249 */     TimeZone gmt = TimeZone.getTimeZone("GMT");
/*      */     
/* 1251 */     GregorianCalendar cal = new GregorianCalendar(gmt);
/* 1252 */     cal.set(year, month, day, hour, minute, second);
/*      */     
/* 1254 */     Date date = cal.getTime();
/*      */     
/* 1256 */     if (this.encodeParam != null) {
/* 1257 */       this.encodeParam.setModificationTime(date);
/*      */     }
/* 1259 */     if (this.emitProperties) {
/* 1260 */       this.properties.put("timestamp", date);
/*      */     }
/*      */   }
/*      */   
/*      */   private void parse_tRNS_chunk(PNGChunk chunk) {
/* 1265 */     if (this.colorType == 3) {
/* 1266 */       int entries = chunk.getLength();
/* 1267 */       if (entries > this.paletteEntries) {
/*      */         
/* 1269 */         String msg = PropertyUtil.getString("PNGImageDecoder14");
/* 1270 */         throw new RuntimeException(msg);
/*      */       } 
/*      */ 
/*      */       
/* 1274 */       this.alphaPalette = new byte[this.paletteEntries];
/* 1275 */       for (int i = 0; i < entries; i++) {
/* 1276 */         this.alphaPalette[i] = chunk.getByte(i);
/*      */       }
/*      */ 
/*      */       
/* 1280 */       for (int i = entries; i < this.paletteEntries; i++) {
/* 1281 */         this.alphaPalette[i] = -1;
/*      */       }
/*      */       
/* 1284 */       if (!this.suppressAlpha) {
/* 1285 */         if (this.expandPalette) {
/* 1286 */           this.postProcess = 5;
/* 1287 */           this.outputBands = 4;
/*      */         } else {
/* 1289 */           this.outputHasAlphaPalette = true;
/*      */         } 
/*      */       }
/* 1292 */     } else if (this.colorType == 0) {
/* 1293 */       this.grayTransparentAlpha = chunk.getInt2(0);
/*      */       
/* 1295 */       if (!this.suppressAlpha) {
/* 1296 */         if (this.bitDepth < 8) {
/* 1297 */           this.output8BitGray = true;
/* 1298 */           this.maxOpacity = 255;
/* 1299 */           this.postProcess = 3;
/*      */         } else {
/* 1301 */           this.postProcess = 6;
/*      */         } 
/*      */         
/* 1304 */         if (this.expandGrayAlpha) {
/* 1305 */           this.outputBands = 4;
/* 1306 */           this.postProcess |= 0x10;
/*      */         } else {
/* 1308 */           this.outputBands = 2;
/*      */         } 
/*      */         
/* 1311 */         if (this.encodeParam != null) {
/* 1312 */           ((PNGEncodeParam.Gray)this.encodeParam).setTransparentGray(this.grayTransparentAlpha);
/*      */         }
/*      */       }
/*      */     
/* 1316 */     } else if (this.colorType == 2) {
/* 1317 */       this.redTransparentAlpha = chunk.getInt2(0);
/* 1318 */       this.greenTransparentAlpha = chunk.getInt2(2);
/* 1319 */       this.blueTransparentAlpha = chunk.getInt2(4);
/*      */       
/* 1321 */       if (!this.suppressAlpha) {
/* 1322 */         this.outputBands = 4;
/* 1323 */         this.postProcess = 7;
/*      */         
/* 1325 */         if (this.encodeParam != null) {
/* 1326 */           int[] rgbTrans = new int[3];
/* 1327 */           rgbTrans[0] = this.redTransparentAlpha;
/* 1328 */           rgbTrans[1] = this.greenTransparentAlpha;
/* 1329 */           rgbTrans[2] = this.blueTransparentAlpha;
/* 1330 */           ((PNGEncodeParam.RGB)this.encodeParam).setTransparentRGB(rgbTrans);
/*      */         }
/*      */       
/*      */       } 
/* 1334 */     } else if (this.colorType == 4 || this.colorType == 6) {
/*      */ 
/*      */       
/* 1337 */       String msg = PropertyUtil.getString("PNGImageDecoder15");
/* 1338 */       throw new RuntimeException(msg);
/*      */     } 
/*      */   }
/*      */   
/*      */   private void parse_zTXt_chunk(PNGChunk chunk) {
/* 1343 */     StringBuffer key = new StringBuffer();
/* 1344 */     StringBuffer value = new StringBuffer();
/*      */ 
/*      */     
/* 1347 */     int textIndex = 0; byte b;
/* 1348 */     while ((b = chunk.getByte(textIndex++)) != 0) {
/* 1349 */       key.append((char)b);
/*      */     }
/* 1351 */     chunk.getByte(textIndex++);
/*      */     
/*      */     try {
/* 1354 */       int length = chunk.getLength() - textIndex;
/* 1355 */       byte[] data = chunk.getData();
/* 1356 */       InputStream cis = new ByteArrayInputStream(data, textIndex, length);
/*      */       
/* 1358 */       InputStream iis = new InflaterInputStream(cis);
/*      */       
/*      */       int c;
/* 1361 */       while ((c = iis.read()) != -1) {
/* 1362 */         value.append((char)c);
/*      */       }
/*      */       
/* 1365 */       this.ztextKeys.add(key.toString());
/* 1366 */       this.ztextStrings.add(value.toString());
/*      */     } catch (Exception e) {
/* 1368 */       e.printStackTrace();
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private WritableRaster createRaster(int width, int height, int bands, int scanlineStride, int bitDepth) {
/* 1377 */     WritableRaster ras = null;
/* 1378 */     Point origin = new Point(0, 0);
/* 1379 */     if (bitDepth < 8 && bands == 1) {
/* 1380 */       DataBuffer dataBuffer = new DataBufferByte(height * scanlineStride);
/* 1381 */       ras = Raster.createPackedRaster(dataBuffer, width, height, bitDepth, origin);
/*      */ 
/*      */     
/*      */     }
/* 1385 */     else if (bitDepth <= 8) {
/* 1386 */       DataBufferByte dataBufferByte = new DataBufferByte(height * scanlineStride);
/* 1387 */       ras = Raster.createInterleavedRaster(dataBufferByte, width, height, scanlineStride, bands, this.bandOffsets[bands], origin);
/*      */ 
/*      */     
/*      */     }
/*      */     else {
/*      */ 
/*      */       
/* 1394 */       DataBufferUShort dataBufferUShort = new DataBufferUShort(height * scanlineStride);
/* 1395 */       ras = Raster.createInterleavedRaster(dataBufferUShort, width, height, scanlineStride, bands, this.bandOffsets[bands], origin);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1403 */     return ras;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   private static void decodeSubFilter(byte[] curr, int count, int bpp) {
/* 1409 */     for (int i = bpp; i < count; i++) {
/*      */ 
/*      */       
/* 1412 */       int val = curr[i] & 0xFF;
/* 1413 */       val += (curr[i - bpp] & 0xFF);
/*      */       
/* 1415 */       curr[i] = (byte)val;
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*      */   private static void decodeUpFilter(byte[] curr, byte[] prev, int count) {
/* 1421 */     for (int i = 0; i < count; i++) {
/* 1422 */       int raw = curr[i] & 0xFF;
/* 1423 */       int prior = prev[i] & 0xFF;
/*      */       
/* 1425 */       curr[i] = (byte)(raw + prior);
/*      */     } 
/*      */   }
/*      */ 
/*      */   
/*      */   private static void decodeAverageFilter(byte[] curr, byte[] prev, int count, int bpp) {
/* 1431 */     for (int i = 0; i < bpp; i++) {
/* 1432 */       int raw = curr[i] & 0xFF;
/* 1433 */       int priorRow = prev[i] & 0xFF;
/*      */       
/* 1435 */       curr[i] = (byte)(raw + priorRow / 2);
/*      */     } 
/*      */     
/* 1438 */     for (int i = bpp; i < count; i++) {
/* 1439 */       int raw = curr[i] & 0xFF;
/* 1440 */       int priorPixel = curr[i - bpp] & 0xFF;
/* 1441 */       int priorRow = prev[i] & 0xFF;
/*      */       
/* 1443 */       curr[i] = (byte)(raw + (priorPixel + priorRow) / 2);
/*      */     } 
/*      */   }
/*      */   
/*      */   private static int paethPredictor(int a, int b, int c) {
/* 1448 */     int p = a + b - c;
/* 1449 */     int pa = Math.abs(p - a);
/* 1450 */     int pb = Math.abs(p - b);
/* 1451 */     int pc = Math.abs(p - c);
/*      */     
/* 1453 */     if (pa <= pb && pa <= pc)
/* 1454 */       return a; 
/* 1455 */     if (pb <= pc) {
/* 1456 */       return b;
/*      */     }
/* 1458 */     return c;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private static void decodePaethFilter(byte[] curr, byte[] prev, int count, int bpp) {
/* 1466 */     for (int i = 0; i < bpp; i++) {
/* 1467 */       int raw = curr[i] & 0xFF;
/* 1468 */       int priorRow = prev[i] & 0xFF;
/*      */       
/* 1470 */       curr[i] = (byte)(raw + priorRow);
/*      */     } 
/*      */     
/* 1473 */     for (int i = bpp; i < count; i++) {
/* 1474 */       int raw = curr[i] & 0xFF;
/* 1475 */       int priorPixel = curr[i - bpp] & 0xFF;
/* 1476 */       int priorRow = prev[i] & 0xFF;
/* 1477 */       int priorRowPixel = prev[i - bpp] & 0xFF;
/*      */       
/* 1479 */       curr[i] = (byte)(raw + paethPredictor(priorPixel, priorRow, priorRowPixel));
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void processPixels(int process, Raster src, WritableRaster dst, int xOffset, int step, int y, int width) {
/*      */     int[] workGammaLut;
/*      */     boolean flagGammaCorrection;
/* 1491 */     int srcX, ps[] = src.getPixel(0, 0, (int[])null);
/* 1492 */     int[] pd = dst.getPixel(0, 0, (int[])null);
/*      */     
/* 1494 */     int dstX = xOffset;
/* 1495 */     switch (process) {
/*      */       case 0:
/* 1497 */         for (srcX = 0; srcX < width; srcX++) {
/* 1498 */           src.getPixel(srcX, 0, ps);
/* 1499 */           dst.setPixel(dstX, y, ps);
/* 1500 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 1:
/* 1505 */         for (srcX = 0; srcX < width; srcX++) {
/* 1506 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1508 */           for (int i = 0; i < this.inputBands; i++) {
/* 1509 */             int x = ps[i];
/* 1510 */             ps[i] = this.gammaLut[x];
/*      */           } 
/*      */           
/* 1513 */           dst.setPixel(dstX, y, ps);
/* 1514 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 2:
/* 1519 */         for (srcX = 0; srcX < width; srcX++) {
/* 1520 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1522 */           pd[0] = this.grayLut[ps[0]];
/*      */           
/* 1524 */           dst.setPixel(dstX, y, pd);
/* 1525 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 3:
/* 1530 */         for (srcX = 0; srcX < width; srcX++) {
/* 1531 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1533 */           int val = ps[0];
/* 1534 */           pd[0] = this.grayLut[val];
/* 1535 */           if (val == this.grayTransparentAlpha) {
/* 1536 */             pd[1] = 0;
/*      */           } else {
/* 1538 */             pd[1] = this.maxOpacity;
/*      */           } 
/*      */           
/* 1541 */           dst.setPixel(dstX, y, pd);
/* 1542 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 4:
/* 1547 */         for (srcX = 0; srcX < width; srcX++) {
/* 1548 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1550 */           int val = ps[0];
/* 1551 */           pd[0] = this.redPalette[val];
/* 1552 */           pd[1] = this.greenPalette[val];
/* 1553 */           pd[2] = this.bluePalette[val];
/*      */           
/* 1555 */           dst.setPixel(dstX, y, pd);
/* 1556 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 5:
/* 1561 */         for (srcX = 0; srcX < width; srcX++) {
/* 1562 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1564 */           int val = ps[0];
/* 1565 */           pd[0] = this.redPalette[val];
/* 1566 */           pd[1] = this.greenPalette[val];
/* 1567 */           pd[2] = this.bluePalette[val];
/* 1568 */           pd[3] = this.alphaPalette[val];
/*      */           
/* 1570 */           dst.setPixel(dstX, y, pd);
/* 1571 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 6:
/* 1576 */         for (srcX = 0; srcX < width; srcX++) {
/* 1577 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1579 */           int val = ps[0];
/* 1580 */           if (this.performGammaCorrection) {
/* 1581 */             val = this.gammaLut[val];
/*      */           }
/* 1583 */           pd[0] = val;
/* 1584 */           if (val == this.grayTransparentAlpha) {
/* 1585 */             pd[1] = 0;
/*      */           } else {
/* 1587 */             pd[1] = this.maxOpacity;
/*      */           } 
/*      */           
/* 1590 */           dst.setPixel(dstX, y, pd);
/* 1591 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 7:
/* 1596 */         flagGammaCorrection = this.performGammaCorrection;
/* 1597 */         workGammaLut = this.gammaLut;
/* 1598 */         for (srcX = 0; srcX < width; srcX++) {
/* 1599 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1601 */           int r = ps[0];
/* 1602 */           int g = ps[1];
/* 1603 */           int b = ps[2];
/* 1604 */           if (flagGammaCorrection) {
/* 1605 */             pd[0] = workGammaLut[r];
/* 1606 */             pd[1] = workGammaLut[g];
/* 1607 */             pd[2] = workGammaLut[b];
/*      */           } else {
/* 1609 */             pd[0] = r;
/* 1610 */             pd[1] = g;
/* 1611 */             pd[2] = b;
/*      */           } 
/* 1613 */           if (r == this.redTransparentAlpha && g == this.greenTransparentAlpha && b == this.blueTransparentAlpha) {
/*      */ 
/*      */             
/* 1616 */             pd[3] = 0;
/*      */           } else {
/* 1618 */             pd[3] = this.maxOpacity;
/*      */           } 
/*      */           
/* 1621 */           dst.setPixel(dstX, y, pd);
/* 1622 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 8:
/* 1627 */         for (srcX = 0; srcX < width; srcX++) {
/* 1628 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1630 */           int g = ps[0];
/* 1631 */           if (this.performGammaCorrection) {
/* 1632 */             pd[0] = this.gammaLut[g];
/*      */           } else {
/* 1634 */             pd[0] = g;
/*      */           } 
/*      */           
/* 1637 */           dst.setPixel(dstX, y, pd);
/* 1638 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 9:
/* 1643 */         for (srcX = 0; srcX < width; srcX++) {
/* 1644 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1646 */           int r = ps[0];
/* 1647 */           int g = ps[1];
/* 1648 */           int b = ps[2];
/* 1649 */           if (this.performGammaCorrection) {
/* 1650 */             pd[0] = this.gammaLut[r];
/* 1651 */             pd[1] = this.gammaLut[g];
/* 1652 */             pd[2] = this.gammaLut[b];
/*      */           } else {
/* 1654 */             pd[0] = r;
/* 1655 */             pd[1] = g;
/* 1656 */             pd[2] = b;
/*      */           } 
/*      */           
/* 1659 */           dst.setPixel(dstX, y, pd);
/* 1660 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 17:
/* 1665 */         for (srcX = 0; srcX < width; srcX++) {
/* 1666 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1668 */           int val = ps[0];
/* 1669 */           int alpha = ps[1];
/* 1670 */           int gamma = this.gammaLut[val];
/* 1671 */           pd[0] = gamma;
/* 1672 */           pd[1] = gamma;
/* 1673 */           pd[2] = gamma;
/* 1674 */           pd[3] = alpha;
/*      */           
/* 1676 */           dst.setPixel(dstX, y, pd);
/* 1677 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 16:
/* 1682 */         for (srcX = 0; srcX < width; srcX++) {
/* 1683 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1685 */           int val = ps[0];
/* 1686 */           int alpha = ps[1];
/* 1687 */           pd[0] = val;
/* 1688 */           pd[1] = val;
/* 1689 */           pd[2] = val;
/* 1690 */           pd[3] = alpha;
/*      */           
/* 1692 */           dst.setPixel(dstX, y, pd);
/* 1693 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 22:
/* 1698 */         for (srcX = 0; srcX < width; srcX++) {
/* 1699 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1701 */           int val = ps[0];
/* 1702 */           if (this.performGammaCorrection) {
/* 1703 */             val = this.gammaLut[val];
/*      */           }
/* 1705 */           pd[0] = val;
/* 1706 */           pd[1] = val;
/* 1707 */           pd[2] = val;
/* 1708 */           if (val == this.grayTransparentAlpha) {
/* 1709 */             pd[3] = 0;
/*      */           } else {
/* 1711 */             pd[3] = this.maxOpacity;
/*      */           } 
/*      */           
/* 1714 */           dst.setPixel(dstX, y, pd);
/* 1715 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 19:
/* 1720 */         for (srcX = 0; srcX < width; srcX++) {
/* 1721 */           src.getPixel(srcX, 0, ps);
/*      */           
/* 1723 */           int val = ps[0];
/* 1724 */           int val2 = this.grayLut[val];
/* 1725 */           pd[0] = val2;
/* 1726 */           pd[1] = val2;
/* 1727 */           pd[2] = val2;
/* 1728 */           if (val == this.grayTransparentAlpha) {
/* 1729 */             pd[3] = 0;
/*      */           } else {
/* 1731 */             pd[3] = this.maxOpacity;
/*      */           } 
/*      */           
/* 1734 */           dst.setPixel(dstX, y, pd);
/* 1735 */           dstX += step;
/*      */         } 
/*      */         break;
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void decodePass(WritableRaster imRas, int xOffset, int yOffset, int xStep, int yStep, int passWidth, int passHeight) {
/* 1749 */     if (passWidth == 0 || passHeight == 0) {
/*      */       return;
/*      */     }
/*      */     
/* 1753 */     int bytesPerRow = (this.inputBands * passWidth * this.bitDepth + 7) / 8;
/* 1754 */     int eltsPerRow = (this.bitDepth == 16) ? (bytesPerRow / 2) : bytesPerRow;
/* 1755 */     byte[] curr = new byte[bytesPerRow];
/* 1756 */     byte[] prior = new byte[bytesPerRow];
/*      */ 
/*      */     
/* 1759 */     WritableRaster passRow = createRaster(passWidth, 1, this.inputBands, eltsPerRow, this.bitDepth);
/*      */ 
/*      */ 
/*      */     
/* 1763 */     DataBuffer dataBuffer = passRow.getDataBuffer();
/* 1764 */     int type = dataBuffer.getDataType();
/* 1765 */     byte[] byteData = null;
/* 1766 */     short[] shortData = null;
/* 1767 */     if (type == 0) {
/* 1768 */       byteData = ((DataBufferByte)dataBuffer).getData();
/*      */     } else {
/* 1770 */       shortData = ((DataBufferUShort)dataBuffer).getData();
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/* 1775 */     int srcY = 0, dstY = yOffset;
/* 1776 */     for (; srcY < passHeight; 
/* 1777 */       srcY++, dstY += yStep) {
/*      */       String msg;
/* 1779 */       int filter = 0;
/*      */       try {
/* 1781 */         filter = this.dataStream.read();
/* 1782 */         this.dataStream.readFully(curr, 0, bytesPerRow);
/*      */       } catch (Exception e) {
/* 1784 */         e.printStackTrace();
/*      */       } 
/*      */       
/* 1787 */       switch (filter) {
/*      */         case 0:
/*      */           break;
/*      */         case 1:
/* 1791 */           decodeSubFilter(curr, bytesPerRow, this.bytesPerPixel);
/*      */           break;
/*      */         case 2:
/* 1794 */           decodeUpFilter(curr, prior, bytesPerRow);
/*      */           break;
/*      */         case 3:
/* 1797 */           decodeAverageFilter(curr, prior, bytesPerRow, this.bytesPerPixel);
/*      */           break;
/*      */         case 4:
/* 1800 */           decodePaethFilter(curr, prior, bytesPerRow, this.bytesPerPixel);
/*      */           break;
/*      */         
/*      */         default:
/* 1804 */           msg = PropertyUtil.getString("PNGImageDecoder16");
/* 1805 */           throw new RuntimeException(msg);
/*      */       } 
/*      */ 
/*      */       
/* 1809 */       if (this.bitDepth < 16) {
/* 1810 */         System.arraycopy(curr, 0, byteData, 0, bytesPerRow);
/*      */       } else {
/* 1812 */         int idx = 0;
/* 1813 */         for (int j = 0; j < eltsPerRow; j++) {
/* 1814 */           shortData[j] = (short)(curr[idx] << 8 | curr[idx + 1] & 0xFF);
/*      */           
/* 1816 */           idx += 2;
/*      */         } 
/*      */       } 
/*      */       
/* 1820 */       processPixels(this.postProcess, passRow, imRas, xOffset, xStep, dstY, passWidth);
/*      */ 
/*      */ 
/*      */       
/* 1824 */       byte[] tmp = prior;
/* 1825 */       prior = curr;
/* 1826 */       curr = tmp;
/*      */     } 
/*      */   }
/*      */   
/*      */   private void decodeImage(boolean useInterlacing) {
/* 1831 */     int width = this.bounds.width;
/* 1832 */     int height = this.bounds.height;
/*      */     
/* 1834 */     if (!useInterlacing) {
/* 1835 */       decodePass(this.theTile, 0, 0, 1, 1, width, height);
/*      */     } else {
/* 1837 */       decodePass(this.theTile, 0, 0, 8, 8, (width + 7) / 8, (height + 7) / 8);
/* 1838 */       decodePass(this.theTile, 4, 0, 8, 8, (width + 3) / 8, (height + 7) / 8);
/* 1839 */       decodePass(this.theTile, 0, 4, 4, 8, (width + 3) / 4, (height + 3) / 8);
/* 1840 */       decodePass(this.theTile, 2, 0, 4, 4, (width + 1) / 4, (height + 3) / 4);
/* 1841 */       decodePass(this.theTile, 0, 2, 2, 4, (width + 1) / 2, (height + 1) / 4);
/* 1842 */       decodePass(this.theTile, 1, 0, 2, 2, width / 2, (height + 1) / 2);
/* 1843 */       decodePass(this.theTile, 0, 1, 1, 2, width, height / 2);
/*      */     } 
/*      */   }
/*      */   
/*      */   public WritableRaster copyData(WritableRaster wr) {
/* 1848 */     GraphicsUtil.copyData(this.theTile, wr);
/* 1849 */     return wr;
/*      */   }
/*      */ 
/*      */   
/*      */   public Raster getTile(int tileX, int tileY) {
/* 1854 */     if (tileX != 0 || tileY != 0) {
/*      */       
/* 1856 */       String msg = PropertyUtil.getString("PNGImageDecoder17");
/* 1857 */       throw new IllegalArgumentException(msg);
/*      */     } 
/* 1859 */     return this.theTile;
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\png\PNGRed.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */