/*     */ package org.apache.batik.ext.awt.image.codec.tiff;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.io.Serializable;
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashMap;
/*     */ import java.util.Iterator;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import org.apache.batik.ext.awt.image.codec.util.SeekableStream;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class TIFFDirectory
/*     */   implements Serializable
/*     */ {
/*     */   boolean isBigEndian;
/*     */   int numEntries;
/*     */   TIFFField[] fields;
/*  65 */   Map fieldIndex = new HashMap();
/*     */ 
/*     */   
/*  68 */   long IFDOffset = 8L;
/*     */ 
/*     */   
/*  71 */   long nextIFDOffset = 0L;
/*     */ 
/*     */   
/*     */   TIFFDirectory() {}
/*     */ 
/*     */   
/*  77 */   private static boolean isValidEndianTag(int endian) { return (endian == 18761 || endian == 19789); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public TIFFDirectory(SeekableStream stream, int directory) throws IOException {
/*  93 */     long global_save_offset = stream.getFilePointer();
/*     */ 
/*     */ 
/*     */     
/*  97 */     stream.seek(0L);
/*  98 */     int endian = stream.readUnsignedShort();
/*  99 */     if (!isValidEndianTag(endian)) {
/* 100 */       throw new IllegalArgumentException("TIFFDirectory1");
/*     */     }
/*     */     
/* 103 */     this.isBigEndian = (endian == 19789);
/*     */     
/* 105 */     int magic = readUnsignedShort(stream);
/* 106 */     if (magic != 42) {
/* 107 */       throw new IllegalArgumentException("TIFFDirectory2");
/*     */     }
/*     */ 
/*     */ 
/*     */     
/* 112 */     long ifd_offset = readUnsignedInt(stream);
/*     */     
/* 114 */     for (int i = 0; i < directory; i++) {
/* 115 */       if (ifd_offset == 0L) {
/* 116 */         throw new IllegalArgumentException("TIFFDirectory3");
/*     */       }
/*     */ 
/*     */       
/* 120 */       stream.seek(ifd_offset);
/* 121 */       long entries = readUnsignedShort(stream);
/* 122 */       stream.skip(12L * entries);
/*     */       
/* 124 */       ifd_offset = readUnsignedInt(stream);
/*     */     } 
/*     */     
/* 127 */     stream.seek(ifd_offset);
/* 128 */     initialize(stream);
/* 129 */     stream.seek(global_save_offset);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public TIFFDirectory(SeekableStream stream, long ifd_offset, int directory) throws IOException {
/* 148 */     long global_save_offset = stream.getFilePointer();
/* 149 */     stream.seek(0L);
/* 150 */     int endian = stream.readUnsignedShort();
/* 151 */     if (!isValidEndianTag(endian)) {
/* 152 */       throw new IllegalArgumentException("TIFFDirectory1");
/*     */     }
/*     */     
/* 155 */     this.isBigEndian = (endian == 19789);
/*     */ 
/*     */     
/* 158 */     stream.seek(ifd_offset);
/*     */ 
/*     */     
/* 161 */     int dirNum = 0;
/* 162 */     while (dirNum < directory) {
/*     */       
/* 164 */       long numEntries = readUnsignedShort(stream);
/*     */ 
/*     */       
/* 167 */       stream.seek(ifd_offset + 12L * numEntries);
/*     */ 
/*     */       
/* 170 */       ifd_offset = readUnsignedInt(stream);
/*     */ 
/*     */       
/* 173 */       stream.seek(ifd_offset);
/*     */ 
/*     */       
/* 176 */       dirNum++;
/*     */     } 
/*     */     
/* 179 */     initialize(stream);
/* 180 */     stream.seek(global_save_offset);
/*     */   }
/*     */   private static final int[] sizeOfType = { 
/* 183 */       0, 1, 1, 2, 4, 8, 1, 1, 2, 4, 8, 4, 8 };
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private void initialize(SeekableStream stream) throws IOException {
/* 203 */     this.IFDOffset = stream.getFilePointer();
/*     */     
/* 205 */     this.numEntries = readUnsignedShort(stream);
/* 206 */     this.fields = new TIFFField[this.numEntries];
/*     */     
/* 208 */     for (int i = 0; i < this.numEntries; i++) {
/* 209 */       double[] dvalues; float[] fvalues; int iivalues[][], ivalues[]; short[] svalues; long llvalues[][], lvalues[]; char[] cvalues; byte[] bvalues; char[] arrayOfChar; int[][] arrayOfInt; long[] arrayOfLong1; int[] arrayOfInt1; float[] arrayOfFloat; short[] arrayOfShort; byte[] arrayOfByte; long[][] arrayOfLong; double[] arrayOfDouble; int j, tag = readUnsignedShort(stream);
/* 210 */       int type = readUnsignedShort(stream);
/* 211 */       int count = (int)readUnsignedInt(stream);
/* 212 */       int value = 0;
/*     */ 
/*     */       
/* 215 */       long nextTagOffset = stream.getFilePointer() + 4L;
/*     */ 
/*     */ 
/*     */       
/*     */       try {
/* 220 */         if (count * sizeOfType[type] > 4) {
/* 221 */           value = (int)readUnsignedInt(stream);
/* 222 */           stream.seek(value);
/*     */         } 
/*     */       } catch (ArrayIndexOutOfBoundsException ae) {
/*     */         
/* 226 */         System.err.println(tag + " " + "TIFFDirectory4");
/*     */         
/* 228 */         stream.seek(nextTagOffset);
/*     */       } 
/*     */ 
/*     */       
/* 232 */       this.fieldIndex.put(new Integer(tag), new Integer(i));
/* 233 */       Object obj = null;
/*     */       
/* 235 */       switch (type) {
/*     */         case 1:
/*     */         case 2:
/*     */         case 6:
/*     */         case 7:
/* 240 */           bvalues = new byte[count];
/* 241 */           stream.readFully(bvalues, 0, count);
/*     */           
/* 243 */           if (type == 2) {
/*     */ 
/*     */             
/* 246 */             int index = 0, prevIndex = 0;
/* 247 */             List v = new ArrayList();
/*     */             
/* 249 */             while (index < count) { do {
/*     */               
/* 251 */               } while (index < count && bvalues[index++] != 0);
/*     */ 
/*     */               
/* 254 */               v.add(new String(bvalues, prevIndex, index - prevIndex));
/*     */               
/* 256 */               prevIndex = index; }
/*     */ 
/*     */             
/* 259 */             count = v.size();
/* 260 */             String[] strings = new String[count];
/* 261 */             v.toArray(strings);
/* 262 */             String[] arrayOfString = strings; break;
/*     */           } 
/* 264 */           arrayOfByte = bvalues;
/*     */           break;
/*     */ 
/*     */ 
/*     */         
/*     */         case 3:
/* 270 */           cvalues = new char[count];
/* 271 */           for (j = 0; j < count; j++) {
/* 272 */             cvalues[j] = (char)readUnsignedShort(stream);
/*     */           }
/* 274 */           arrayOfChar = cvalues;
/*     */           break;
/*     */         
/*     */         case 4:
/* 278 */           lvalues = new long[count];
/* 279 */           for (j = 0; j < count; j++) {
/* 280 */             lvalues[j] = readUnsignedInt(stream);
/*     */           }
/* 282 */           arrayOfLong1 = lvalues;
/*     */           break;
/*     */         
/*     */         case 5:
/* 286 */           llvalues = new long[count][2];
/* 287 */           for (j = 0; j < count; j++) {
/* 288 */             llvalues[j][0] = readUnsignedInt(stream);
/* 289 */             llvalues[j][1] = readUnsignedInt(stream);
/*     */           } 
/* 291 */           arrayOfLong = llvalues;
/*     */           break;
/*     */         
/*     */         case 8:
/* 295 */           svalues = new short[count];
/* 296 */           for (j = 0; j < count; j++) {
/* 297 */             svalues[j] = readShort(stream);
/*     */           }
/* 299 */           arrayOfShort = svalues;
/*     */           break;
/*     */         
/*     */         case 9:
/* 303 */           ivalues = new int[count];
/* 304 */           for (j = 0; j < count; j++) {
/* 305 */             ivalues[j] = readInt(stream);
/*     */           }
/* 307 */           arrayOfInt1 = ivalues;
/*     */           break;
/*     */         
/*     */         case 10:
/* 311 */           iivalues = new int[count][2];
/* 312 */           for (j = 0; j < count; j++) {
/* 313 */             iivalues[j][0] = readInt(stream);
/* 314 */             iivalues[j][1] = readInt(stream);
/*     */           } 
/* 316 */           arrayOfInt = iivalues;
/*     */           break;
/*     */         
/*     */         case 11:
/* 320 */           fvalues = new float[count];
/* 321 */           for (j = 0; j < count; j++) {
/* 322 */             fvalues[j] = readFloat(stream);
/*     */           }
/* 324 */           arrayOfFloat = fvalues;
/*     */           break;
/*     */         
/*     */         case 12:
/* 328 */           dvalues = new double[count];
/* 329 */           for (j = 0; j < count; j++) {
/* 330 */             dvalues[j] = readDouble(stream);
/*     */           }
/* 332 */           arrayOfDouble = dvalues;
/*     */           break;
/*     */         
/*     */         default:
/* 336 */           System.err.println("TIFFDirectory0");
/*     */           break;
/*     */       } 
/*     */       
/* 340 */       this.fields[i] = new TIFFField(tag, type, count, arrayOfDouble);
/* 341 */       stream.seek(nextTagOffset);
/*     */     } 
/*     */ 
/*     */     
/* 345 */     this.nextIFDOffset = readUnsignedInt(stream);
/*     */   }
/*     */ 
/*     */ 
/*     */   
/* 350 */   public int getNumEntries() { return this.numEntries; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public TIFFField getField(int tag) {
/* 358 */     Integer i = (Integer)this.fieldIndex.get(new Integer(tag));
/* 359 */     if (i == null) {
/* 360 */       return null;
/*     */     }
/* 362 */     return this.fields[i.intValue()];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 370 */   public boolean isTagPresent(int tag) { return this.fieldIndex.containsKey(new Integer(tag)); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public int[] getTags() {
/* 378 */     int[] tags = new int[this.fieldIndex.size()];
/* 379 */     Iterator iter = this.fieldIndex.keySet().iterator();
/* 380 */     int i = 0;
/*     */     
/* 382 */     while (iter.hasNext()) {
/* 383 */       tags[i++] = ((Integer)iter.next()).intValue();
/*     */     }
/*     */     
/* 386 */     return tags;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 394 */   public TIFFField[] getFields() { return this.fields; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public byte getFieldAsByte(int tag, int index) {
/* 404 */     Integer i = (Integer)this.fieldIndex.get(new Integer(tag));
/* 405 */     byte[] b = this.fields[i.intValue()].getAsBytes();
/* 406 */     return b[index];
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 416 */   public byte getFieldAsByte(int tag) { return getFieldAsByte(tag, 0); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public long getFieldAsLong(int tag, int index) {
/* 426 */     Integer i = (Integer)this.fieldIndex.get(new Integer(tag));
/* 427 */     return this.fields[i.intValue()].getAsLong(index);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 437 */   public long getFieldAsLong(int tag) { return getFieldAsLong(tag, 0); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public float getFieldAsFloat(int tag, int index) {
/* 447 */     Integer i = (Integer)this.fieldIndex.get(new Integer(tag));
/* 448 */     return this.fields[i.intValue()].getAsFloat(index);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 457 */   public float getFieldAsFloat(int tag) { return getFieldAsFloat(tag, 0); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public double getFieldAsDouble(int tag, int index) {
/* 467 */     Integer i = (Integer)this.fieldIndex.get(new Integer(tag));
/* 468 */     return this.fields[i.intValue()].getAsDouble(index);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 477 */   public double getFieldAsDouble(int tag) { return getFieldAsDouble(tag, 0); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private short readShort(SeekableStream stream) throws IOException {
/* 484 */     if (this.isBigEndian) {
/* 485 */       return stream.readShort();
/*     */     }
/* 487 */     return stream.readShortLE();
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   private int readUnsignedShort(SeekableStream stream) throws IOException {
/* 493 */     if (this.isBigEndian) {
/* 494 */       return stream.readUnsignedShort();
/*     */     }
/* 496 */     return stream.readUnsignedShortLE();
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   private int readInt(SeekableStream stream) throws IOException {
/* 502 */     if (this.isBigEndian) {
/* 503 */       return stream.readInt();
/*     */     }
/* 505 */     return stream.readIntLE();
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   private long readUnsignedInt(SeekableStream stream) throws IOException {
/* 511 */     if (this.isBigEndian) {
/* 512 */       return stream.readUnsignedInt();
/*     */     }
/* 514 */     return stream.readUnsignedIntLE();
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   private long readLong(SeekableStream stream) throws IOException {
/* 520 */     if (this.isBigEndian) {
/* 521 */       return stream.readLong();
/*     */     }
/* 523 */     return stream.readLongLE();
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   private float readFloat(SeekableStream stream) throws IOException {
/* 529 */     if (this.isBigEndian) {
/* 530 */       return stream.readFloat();
/*     */     }
/* 532 */     return stream.readFloatLE();
/*     */   }
/*     */ 
/*     */ 
/*     */   
/*     */   private double readDouble(SeekableStream stream) throws IOException {
/* 538 */     if (this.isBigEndian) {
/* 539 */       return stream.readDouble();
/*     */     }
/* 541 */     return stream.readDoubleLE();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private static int readUnsignedShort(SeekableStream stream, boolean isBigEndian) throws IOException {
/* 548 */     if (isBigEndian) {
/* 549 */       return stream.readUnsignedShort();
/*     */     }
/* 551 */     return stream.readUnsignedShortLE();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   private static long readUnsignedInt(SeekableStream stream, boolean isBigEndian) throws IOException {
/* 558 */     if (isBigEndian) {
/* 559 */       return stream.readUnsignedInt();
/*     */     }
/* 561 */     return stream.readUnsignedIntLE();
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int getNumDirectories(SeekableStream stream) throws IOException {
/* 573 */     long pointer = stream.getFilePointer();
/*     */     
/* 575 */     stream.seek(0L);
/* 576 */     int endian = stream.readUnsignedShort();
/* 577 */     if (!isValidEndianTag(endian)) {
/* 578 */       throw new IllegalArgumentException("TIFFDirectory1");
/*     */     }
/*     */     
/* 581 */     boolean isBigEndian = (endian == 19789);
/* 582 */     int magic = readUnsignedShort(stream, isBigEndian);
/* 583 */     if (magic != 42) {
/* 584 */       throw new IllegalArgumentException("TIFFDirectory2");
/*     */     }
/*     */ 
/*     */     
/* 588 */     stream.seek(4L);
/* 589 */     long offset = readUnsignedInt(stream, isBigEndian);
/*     */     
/* 591 */     int numDirectories = 0;
/* 592 */     while (offset != 0L) {
/* 593 */       numDirectories++;
/*     */       
/* 595 */       stream.seek(offset);
/* 596 */       long entries = readUnsignedShort(stream, isBigEndian);
/* 597 */       stream.skip(12L * entries);
/* 598 */       offset = readUnsignedInt(stream, isBigEndian);
/*     */     } 
/*     */     
/* 601 */     stream.seek(pointer);
/* 602 */     return numDirectories;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 611 */   public boolean isBigEndian() { return this.isBigEndian; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 619 */   public long getIFDOffset() { return this.IFDOffset; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 627 */   public long getNextIFDOffset() { return this.nextIFDOffset; }
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\tiff\TIFFDirectory.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */