/*      */ package org.apache.batik.ext.awt.image.codec.tiff;
/*      */ 
/*      */ import com.sun.image.codec.jpeg.JPEGCodec;
/*      */ import com.sun.image.codec.jpeg.JPEGDecodeParam;
/*      */ import com.sun.image.codec.jpeg.JPEGImageDecoder;
/*      */ import java.awt.Rectangle;
/*      */ import java.awt.color.ColorSpace;
/*      */ import java.awt.image.ColorModel;
/*      */ import java.awt.image.ComponentColorModel;
/*      */ import java.awt.image.DataBuffer;
/*      */ import java.awt.image.DataBufferByte;
/*      */ import java.awt.image.DataBufferInt;
/*      */ import java.awt.image.DataBufferShort;
/*      */ import java.awt.image.DataBufferUShort;
/*      */ import java.awt.image.IndexColorModel;
/*      */ import java.awt.image.MultiPixelPackedSampleModel;
/*      */ import java.awt.image.PixelInterleavedSampleModel;
/*      */ import java.awt.image.Raster;
/*      */ import java.awt.image.SampleModel;
/*      */ import java.awt.image.WritableRaster;
/*      */ import java.io.ByteArrayInputStream;
/*      */ import java.io.IOException;
/*      */ import java.util.HashMap;
/*      */ import java.util.Map;
/*      */ import java.util.zip.DataFormatException;
/*      */ import java.util.zip.Inflater;
/*      */ import org.apache.batik.ext.awt.image.codec.util.SeekableStream;
/*      */ import org.apache.batik.ext.awt.image.rendered.AbstractRed;
/*      */ import org.apache.batik.ext.awt.image.rendered.CachableRed;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class TIFFImage
/*      */   extends AbstractRed
/*      */ {
/*      */   public static final int COMP_NONE = 1;
/*      */   public static final int COMP_FAX_G3_1D = 2;
/*      */   public static final int COMP_FAX_G3_2D = 3;
/*      */   public static final int COMP_FAX_G4_2D = 4;
/*      */   public static final int COMP_LZW = 5;
/*      */   public static final int COMP_JPEG_OLD = 6;
/*      */   public static final int COMP_JPEG_TTN2 = 7;
/*      */   public static final int COMP_PACKBITS = 32773;
/*      */   public static final int COMP_DEFLATE = 32946;
/*      */   private static final int TYPE_UNSUPPORTED = -1;
/*      */   private static final int TYPE_BILEVEL = 0;
/*      */   private static final int TYPE_GRAY_4BIT = 1;
/*      */   private static final int TYPE_GRAY = 2;
/*      */   private static final int TYPE_GRAY_ALPHA = 3;
/*      */   private static final int TYPE_PALETTE = 4;
/*      */   private static final int TYPE_RGB = 5;
/*      */   private static final int TYPE_RGB_ALPHA = 6;
/*      */   private static final int TYPE_YCBCR_SUB = 7;
/*      */   private static final int TYPE_GENERIC = 8;
/*      */   private static final int TIFF_JPEG_TABLES = 347;
/*      */   private static final int TIFF_YCBCR_SUBSAMPLING = 530;
/*      */   SeekableStream stream;
/*      */   int tileSize;
/*      */   int tilesX;
/*      */   int tilesY;
/*      */   long[] tileOffsets;
/*      */   long[] tileByteCounts;
/*      */   char[] colormap;
/*      */   int sampleSize;
/*      */   int compression;
/*      */   byte[] palette;
/*      */   int numBands;
/*      */   int chromaSubH;
/*      */   int chromaSubV;
/*      */   long tiffT4Options;
/*      */   long tiffT6Options;
/*      */   int fillOrder;
/*      */   int predictor;
/*      */   JPEGDecodeParam decodeParam;
/*      */   boolean colorConvertJPEG;
/*      */   Inflater inflater;
/*      */   boolean isBigEndian;
/*      */   int imageType;
/*      */   boolean isWhiteZero;
/*      */   int dataType;
/*      */   boolean decodePaletteAsShorts;
/*      */   boolean tiled;
/*      */   private TIFFFaxDecoder decoder;
/*      */   private TIFFLZWDecoder lzwDecoder;
/*      */   
/*      */   private static final Raster decodeJPEG(byte[] data, JPEGDecodeParam decodeParam, boolean colorConvert, int minX, int minY) {
/*      */     Raster jpegRaster;
/*  146 */     ByteArrayInputStream jpegStream = new ByteArrayInputStream(data);
/*      */ 
/*      */     
/*  149 */     JPEGImageDecoder decoder = (decodeParam == null) ? JPEGCodec.createJPEGDecoder(jpegStream) : JPEGCodec.createJPEGDecoder(jpegStream, decodeParam);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/*  157 */       jpegRaster = colorConvert ? decoder.decodeAsBufferedImage().getWritableTile(0, 0) : decoder.decodeAsRaster();
/*      */     }
/*      */     catch (IOException ioe) {
/*      */       
/*  161 */       throw new RuntimeException("TIFFImage13");
/*      */     } 
/*      */ 
/*      */     
/*  165 */     return jpegRaster.createTranslatedChild(minX, minY);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private final void inflate(byte[] deflated, byte[] inflated) {
/*  173 */     this.inflater.setInput(deflated);
/*      */     try {
/*  175 */       this.inflater.inflate(inflated);
/*      */     } catch (DataFormatException dfe) {
/*  177 */       throw new RuntimeException("TIFFImage17: " + dfe.getMessage());
/*      */     } 
/*      */     
/*  180 */     this.inflater.reset();
/*      */   }
/*      */ 
/*      */   
/*      */   private static SampleModel createPixelInterleavedSampleModel(int dataType, int tileWidth, int tileHeight, int bands) {
/*  185 */     int[] bandOffsets = new int[bands];
/*  186 */     for (int i = 0; i < bands; i++)
/*  187 */       bandOffsets[i] = i; 
/*  188 */     return new PixelInterleavedSampleModel(dataType, tileWidth, tileHeight, bands, tileWidth * bands, bandOffsets);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private long[] getFieldAsLongs(TIFFField field) {
/*  197 */     long[] value = null;
/*      */     
/*  199 */     if (field.getType() == 3) {
/*  200 */       char[] charValue = field.getAsChars();
/*  201 */       value = new long[charValue.length];
/*  202 */       for (int i = 0; i < charValue.length; i++) {
/*  203 */         value[i] = (charValue[i] & 0xFFFF);
/*      */       }
/*  205 */     } else if (field.getType() == 4) {
/*  206 */       value = field.getAsLongs();
/*      */     } else {
/*  208 */       throw new RuntimeException();
/*      */     } 
/*      */     
/*  211 */     return value;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public TIFFImage(SeekableStream stream, TIFFDecodeParam param, int directory) throws IOException {
/*      */     this.decodeParam = null;
/*      */     this.colorConvertJPEG = false;
/*      */     this.inflater = null;
/*      */     this.isWhiteZero = false;
/*      */     this.decoder = null;
/*      */     this.lzwDecoder = null;
/*  228 */     this.stream = stream;
/*  229 */     if (param == null) {
/*  230 */       param = new TIFFDecodeParam();
/*      */     }
/*      */     
/*  233 */     this.decodePaletteAsShorts = param.getDecodePaletteAsShorts();
/*      */ 
/*      */     
/*  236 */     TIFFDirectory dir = (param.getIFDOffset() == null) ? new TIFFDirectory(stream, directory) : new TIFFDirectory(stream, param.getIFDOffset().longValue(), directory);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  242 */     TIFFField sfield = dir.getField(277);
/*  243 */     int samplesPerPixel = (sfield == null) ? 1 : (int)sfield.getAsLong(0);
/*      */ 
/*      */     
/*  246 */     TIFFField planarConfigurationField = dir.getField(284);
/*      */     
/*  248 */     new char[1][0] = '\001'; char[] planarConfiguration = (planarConfigurationField == null) ? new char[1] : planarConfigurationField.getAsChars();
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  253 */     if (planarConfiguration[0] != '\001' && samplesPerPixel != 1) {
/*  254 */       throw new RuntimeException("TIFFImage0");
/*      */     }
/*      */ 
/*      */     
/*  258 */     TIFFField bitsField = dir.getField(258);
/*      */     
/*  260 */     char[] bitsPerSample = null;
/*  261 */     if (bitsField != null) {
/*  262 */       bitsPerSample = bitsField.getAsChars();
/*      */     } else {
/*  264 */       bitsPerSample = new char[] { '\001' };
/*      */ 
/*      */       
/*  267 */       for (int i = 1; i < bitsPerSample.length; i++) {
/*  268 */         if (bitsPerSample[i] != bitsPerSample[0]) {
/*  269 */           throw new RuntimeException("TIFFImage1");
/*      */         }
/*      */       } 
/*      */     } 
/*  273 */     this.sampleSize = bitsPerSample[0];
/*      */ 
/*      */ 
/*      */     
/*  277 */     TIFFField sampleFormatField = dir.getField(339);
/*      */ 
/*      */     
/*  280 */     char[] sampleFormat = null;
/*  281 */     if (sampleFormatField != null) {
/*  282 */       sampleFormat = sampleFormatField.getAsChars();
/*      */ 
/*      */       
/*  285 */       for (int l = 1; l < sampleFormat.length; l++) {
/*  286 */         if (sampleFormat[l] != sampleFormat[0]) {
/*  287 */           throw new RuntimeException("TIFFImage2");
/*      */         }
/*      */       } 
/*      */     } else {
/*      */       
/*  292 */       sampleFormat = new char[] { '\001' };
/*      */     } 
/*      */ 
/*      */     
/*  296 */     boolean isValidDataFormat = false;
/*  297 */     switch (this.sampleSize) {
/*      */       case 1:
/*      */       case 4:
/*      */       case 8:
/*  301 */         if (sampleFormat[0] != '\003') {
/*      */           
/*  303 */           this.dataType = 0;
/*  304 */           isValidDataFormat = true;
/*      */         } 
/*      */         break;
/*      */       case 16:
/*  308 */         if (sampleFormat[0] != '\003') {
/*  309 */           this.dataType = (sampleFormat[0] == '\002') ? 2 : 1;
/*      */           
/*  311 */           isValidDataFormat = true;
/*      */         } 
/*      */         break;
/*      */       case 32:
/*  315 */         if (sampleFormat[0] == '\003') {
/*  316 */           isValidDataFormat = false; break;
/*      */         } 
/*  318 */         this.dataType = 3;
/*  319 */         isValidDataFormat = true;
/*      */         break;
/*      */     } 
/*      */ 
/*      */     
/*  324 */     if (!isValidDataFormat) {
/*  325 */       throw new RuntimeException("TIFFImage3");
/*      */     }
/*      */ 
/*      */     
/*  329 */     TIFFField compField = dir.getField(259);
/*  330 */     this.compression = (compField == null) ? 1 : compField.getAsInt(0);
/*      */ 
/*      */     
/*  333 */     int photometricType = (int)dir.getFieldAsLong(262);
/*      */ 
/*      */ 
/*      */     
/*  337 */     this.imageType = -1;
/*  338 */     switch (photometricType) {
/*      */       case 0:
/*  340 */         this.isWhiteZero = true;
/*      */       case 1:
/*  342 */         if (this.sampleSize == 1 && samplesPerPixel == 1) {
/*  343 */           this.imageType = 0; break;
/*  344 */         }  if (this.sampleSize == 4 && samplesPerPixel == 1) {
/*  345 */           this.imageType = 1; break;
/*  346 */         }  if (this.sampleSize % 8 == 0) {
/*  347 */           if (samplesPerPixel == 1) {
/*  348 */             this.imageType = 2; break;
/*  349 */           }  if (samplesPerPixel == 2) {
/*  350 */             this.imageType = 3; break;
/*      */           } 
/*  352 */           this.imageType = 8;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 2:
/*  357 */         if (this.sampleSize % 8 == 0) {
/*  358 */           if (samplesPerPixel == 3) {
/*  359 */             this.imageType = 5; break;
/*  360 */           }  if (samplesPerPixel == 4) {
/*  361 */             this.imageType = 6; break;
/*      */           } 
/*  363 */           this.imageType = 8;
/*      */         } 
/*      */         break;
/*      */       
/*      */       case 3:
/*  368 */         if (samplesPerPixel == 1 && (this.sampleSize == 4 || this.sampleSize == 8 || this.sampleSize == 16))
/*      */         {
/*  370 */           this.imageType = 4;
/*      */         }
/*      */         break;
/*      */       case 4:
/*  374 */         if (this.sampleSize == 1 && samplesPerPixel == 1) {
/*  375 */           this.imageType = 0;
/*      */         }
/*      */         break;
/*      */       case 6:
/*  379 */         if (this.compression == 7 && this.sampleSize == 8 && samplesPerPixel == 3) {
/*      */ 
/*      */           
/*  382 */           this.colorConvertJPEG = param.getJPEGDecompressYCbCrToRGB();
/*      */ 
/*      */           
/*  385 */           this.imageType = this.colorConvertJPEG ? 5 : 8; break;
/*      */         } 
/*  387 */         chromaField = dir.getField(530);
/*  388 */         if (chromaField != null) {
/*  389 */           this.chromaSubH = chromaField.getAsInt(0);
/*  390 */           this.chromaSubV = chromaField.getAsInt(1);
/*      */         } else {
/*  392 */           this.chromaSubH = this.chromaSubV = 2;
/*      */         } 
/*      */         
/*  395 */         if (this.chromaSubH * this.chromaSubV == 1) {
/*  396 */           this.imageType = 8; break;
/*  397 */         }  if (this.sampleSize == 8 && samplesPerPixel == 3) {
/*  398 */           this.imageType = 7;
/*      */         }
/*      */         break;
/*      */       
/*      */       default:
/*  403 */         if (this.sampleSize % 8 == 0) {
/*  404 */           this.imageType = 8;
/*      */         }
/*      */         break;
/*      */     } 
/*      */     
/*  409 */     if (this.imageType == -1) {
/*  410 */       throw new RuntimeException("TIFFImage4");
/*      */     }
/*      */ 
/*      */     
/*  414 */     Rectangle bounds = new Rectangle(0, 0, (int)dir.getFieldAsLong(256), (int)dir.getFieldAsLong(257));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  420 */     this.numBands = samplesPerPixel;
/*      */ 
/*      */     
/*  423 */     TIFFField efield = dir.getField(338);
/*  424 */     int extraSamples = (efield == null) ? 0 : (int)efield.getAsLong(0);
/*      */ 
/*      */     
/*  427 */     if (dir.getField('ń') != null) {
/*  428 */       this.tiled = true;
/*      */       
/*  430 */       j = (int)dir.getFieldAsLong(322);
/*      */       
/*  432 */       k = (int)dir.getFieldAsLong(323);
/*      */       
/*  434 */       this.tileOffsets = dir.getField(324).getAsLongs();
/*      */       
/*  436 */       this.tileByteCounts = getFieldAsLongs(dir.getField(325));
/*      */     }
/*      */     else {
/*      */       
/*  440 */       this.tiled = false;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  447 */       j = (dir.getField('ł') != null) ? (int)dir.getFieldAsLong(322) : bounds.width;
/*      */ 
/*      */ 
/*      */       
/*  451 */       TIFFField field = dir.getField(278);
/*      */       
/*  453 */       if (field == null) {
/*      */ 
/*      */         
/*  456 */         k = (dir.getField('Ń') != null) ? (int)dir.getFieldAsLong(323) : bounds.height;
/*      */       
/*      */       }
/*      */       else {
/*      */         
/*  461 */         long l = field.getAsLong(0);
/*  462 */         long infinity = 1L;
/*  463 */         infinity = (infinity << 32) - 1L;
/*  464 */         if (l == infinity) {
/*      */           
/*  466 */           k = bounds.height;
/*      */         } else {
/*  468 */           k = (int)l;
/*      */         } 
/*      */       } 
/*      */       
/*  472 */       TIFFField tileOffsetsField = dir.getField(273);
/*      */       
/*  474 */       if (tileOffsetsField == null) {
/*  475 */         throw new RuntimeException("TIFFImage5");
/*      */       }
/*  477 */       this.tileOffsets = getFieldAsLongs(tileOffsetsField);
/*      */ 
/*      */       
/*  480 */       TIFFField tileByteCountsField = dir.getField(279);
/*      */       
/*  482 */       if (tileByteCountsField == null) {
/*  483 */         throw new RuntimeException("TIFFImage6");
/*      */       }
/*  485 */       this.tileByteCounts = getFieldAsLongs(tileByteCountsField);
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  490 */     this.tilesX = (bounds.width + j - 1) / j;
/*  491 */     this.tilesY = (bounds.height + k - 1) / k;
/*  492 */     this.tileSize = j * k * this.numBands;
/*      */ 
/*      */     
/*  495 */     this.isBigEndian = dir.isBigEndian();
/*      */     
/*  497 */     TIFFField fillOrderField = dir.getField(266);
/*      */     
/*  499 */     if (fillOrderField != null) {
/*  500 */       this.fillOrder = fillOrderField.getAsInt(0);
/*      */     } else {
/*      */       
/*  503 */       this.fillOrder = 1;
/*      */     } 
/*      */     
/*  506 */     switch (this.compression) {
/*      */       case 1:
/*      */       case 32773:
/*      */         break;
/*      */       
/*      */       case 32946:
/*  512 */         this.inflater = new Inflater();
/*      */         break;
/*      */       case 2:
/*      */       case 3:
/*      */       case 4:
/*  517 */         if (this.sampleSize != 1) {
/*  518 */           throw new RuntimeException("TIFFImage7");
/*      */         }
/*      */ 
/*      */         
/*  522 */         if (this.compression == 3) {
/*  523 */           TIFFField t4OptionsField = dir.getField(292);
/*      */           
/*  525 */           if (t4OptionsField != null) {
/*  526 */             this.tiffT4Options = t4OptionsField.getAsLong(0);
/*      */           } else {
/*      */             
/*  529 */             this.tiffT4Options = 0L;
/*      */           } 
/*      */         } 
/*      */ 
/*      */         
/*  534 */         if (this.compression == 4) {
/*  535 */           TIFFField t6OptionsField = dir.getField(293);
/*      */           
/*  537 */           if (t6OptionsField != null) {
/*  538 */             this.tiffT6Options = t6OptionsField.getAsLong(0);
/*      */           } else {
/*      */             
/*  541 */             this.tiffT6Options = 0L;
/*      */           } 
/*      */         } 
/*      */ 
/*      */         
/*  546 */         this.decoder = new TIFFFaxDecoder(this.fillOrder, j, k);
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       case 5:
/*  552 */         predictorField = dir.getField(317);
/*      */ 
/*      */         
/*  555 */         if (predictorField == null) {
/*  556 */           this.predictor = 1;
/*      */         } else {
/*  558 */           this.predictor = predictorField.getAsInt(0);
/*      */           
/*  560 */           if (this.predictor != 1 && this.predictor != 2) {
/*  561 */             throw new RuntimeException("TIFFImage8");
/*      */           }
/*      */           
/*  564 */           if (this.predictor == 2 && this.sampleSize != 8) {
/*  565 */             throw new RuntimeException(this.sampleSize + "TIFFImage9");
/*      */           }
/*      */         } 
/*      */ 
/*      */         
/*  570 */         this.lzwDecoder = new TIFFLZWDecoder(j, this.predictor, samplesPerPixel);
/*      */         break;
/*      */ 
/*      */       
/*      */       case 6:
/*  575 */         throw new RuntimeException("TIFFImage15");
/*      */       
/*      */       case 7:
/*  578 */         if (this.sampleSize != 8 || ((this.imageType != 2 || samplesPerPixel != 1) && (this.imageType != 4 || samplesPerPixel != 1) && (this.imageType != 5 || samplesPerPixel != 3)))
/*      */         {
/*      */ 
/*      */           
/*  582 */           throw new RuntimeException("TIFFImage16");
/*      */         }
/*      */ 
/*      */         
/*  586 */         if (dir.isTagPresent(347)) {
/*  587 */           TIFFField jpegTableField = dir.getField(347);
/*  588 */           byte[] jpegTable = jpegTableField.getAsBytes();
/*  589 */           ByteArrayInputStream tableStream = new ByteArrayInputStream(jpegTable);
/*      */           
/*  591 */           JPEGImageDecoder decoder = JPEGCodec.createJPEGDecoder(tableStream);
/*      */           
/*  593 */           decoder.decodeAsRaster();
/*  594 */           this.decodeParam = decoder.getJPEGDecodeParam();
/*      */         } 
/*      */         break;
/*      */       
/*      */       default:
/*  599 */         throw new RuntimeException("TIFFImage10");
/*      */     } 
/*      */     
/*  602 */     ColorModel colorModel = null;
/*  603 */     SampleModel sampleModel = null;
/*  604 */     switch (this.imageType) {
/*      */       case 0:
/*      */       case 1:
/*  607 */         sampleModel = new MultiPixelPackedSampleModel(this.dataType, j, k, this.sampleSize);
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  612 */         if (this.imageType == 0) {
/*  613 */           byte[] map = { this.isWhiteZero ? -1 : 0, this.isWhiteZero ? 0 : -1 };
/*      */           
/*  615 */           colorModel = new IndexColorModel(1, 2, map, map, map); break;
/*      */         } 
/*  617 */         map = new byte[16];
/*  618 */         if (this.isWhiteZero) {
/*  619 */           for (int i = 0; i < map.length; i++)
/*  620 */             map[i] = (byte)(255 - 16 * i); 
/*      */         } else {
/*  622 */           for (int i = 0; i < map.length; i++)
/*  623 */             map[i] = (byte)(16 * i); 
/*      */         } 
/*  625 */         colorModel = new IndexColorModel(4, 16, map, map, map);
/*      */         break;
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*      */       case 2:
/*      */       case 3:
/*      */       case 5:
/*      */       case 6:
/*  635 */         reverseOffsets = new int[this.numBands];
/*  636 */         for (i = 0; i < this.numBands; i++) {
/*  637 */           reverseOffsets[i] = this.numBands - 1 - i;
/*      */         }
/*  639 */         sampleModel = new PixelInterleavedSampleModel(this.dataType, j, k, this.numBands, this.numBands * j, reverseOffsets);
/*      */ 
/*      */ 
/*      */         
/*  643 */         if (this.imageType == 2) {
/*  644 */           colorModel = new ComponentColorModel(ColorSpace.getInstance(1003), new int[] { this.sampleSize }, false, false, 1, this.dataType);
/*      */           
/*      */           break;
/*      */         } 
/*  648 */         if (this.imageType == 5) {
/*  649 */           colorModel = new ComponentColorModel(ColorSpace.getInstance(1000), new int[] { this.sampleSize, this.sampleSize, this.sampleSize }, false, false, 1, this.dataType);
/*      */ 
/*      */ 
/*      */           
/*      */           break;
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/*  658 */         transparency = 1;
/*  659 */         if (extraSamples == 1) {
/*  660 */           transparency = 3;
/*  661 */         } else if (extraSamples == 2) {
/*  662 */           transparency = 2;
/*      */         } 
/*      */         
/*  665 */         colorModel = createAlphaComponentColorModel(this.dataType, this.numBands, (extraSamples == 1), transparency);
/*      */         break;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*      */       case 7:
/*      */       case 8:
/*  679 */         bandOffsets = new int[this.numBands];
/*  680 */         for (i = 0; i < this.numBands; i++) {
/*  681 */           bandOffsets[i] = i;
/*      */         }
/*      */         
/*  684 */         sampleModel = new PixelInterleavedSampleModel(this.dataType, j, k, this.numBands, this.numBands * j, bandOffsets);
/*      */ 
/*      */         
/*  687 */         colorModel = null;
/*      */         break;
/*      */ 
/*      */       
/*      */       case 4:
/*  692 */         cfield = dir.getField(320);
/*  693 */         if (cfield == null) {
/*  694 */           throw new RuntimeException("TIFFImage11");
/*      */         }
/*  696 */         this.colormap = cfield.getAsChars();
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  701 */         if (this.decodePaletteAsShorts) {
/*  702 */           this.numBands = 3;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  709 */           if (this.dataType == 0) {
/*  710 */             this.dataType = 1;
/*      */           }
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  716 */           sampleModel = createPixelInterleavedSampleModel(this.dataType, j, k, this.numBands);
/*      */ 
/*      */           
/*  719 */           colorModel = new ComponentColorModel(ColorSpace.getInstance(1000), new int[] { 16, 16, 16 }, false, false, 1, this.dataType);
/*      */ 
/*      */           
/*      */           break;
/*      */         } 
/*      */ 
/*      */         
/*  726 */         this.numBands = 1;
/*      */         
/*  728 */         if (this.sampleSize == 4) {
/*      */ 
/*      */ 
/*      */           
/*  732 */           sampleModel = new MultiPixelPackedSampleModel(0, j, k, this.sampleSize);
/*      */         
/*      */         }
/*  735 */         else if (this.sampleSize == 8) {
/*      */           
/*  737 */           sampleModel = createPixelInterleavedSampleModel(0, j, k, this.numBands);
/*      */         
/*      */         }
/*  740 */         else if (this.sampleSize == 16) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  747 */           this.dataType = 1;
/*  748 */           sampleModel = createPixelInterleavedSampleModel(1, j, k, this.numBands);
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/*  753 */         bandLength = this.colormap.length / 3;
/*  754 */         r = new byte[bandLength];
/*  755 */         g = new byte[bandLength];
/*  756 */         b = new byte[bandLength];
/*      */         
/*  758 */         gIndex = bandLength;
/*  759 */         bIndex = bandLength * 2;
/*      */         
/*  761 */         if (this.dataType == 2) {
/*      */           
/*  763 */           for (int i = 0; i < bandLength; i++) {
/*  764 */             r[i] = param.decodeSigned16BitsTo8Bits((short)this.colormap[i]);
/*      */             
/*  766 */             g[i] = param.decodeSigned16BitsTo8Bits((short)this.colormap[gIndex + i]);
/*      */             
/*  768 */             b[i] = param.decodeSigned16BitsTo8Bits((short)this.colormap[bIndex + i]);
/*      */           }
/*      */         
/*      */         }
/*      */         else {
/*      */           
/*  774 */           for (int i = 0; i < bandLength; i++) {
/*  775 */             r[i] = param.decode16BitsTo8Bits(this.colormap[i] & 0xFFFF);
/*      */             
/*  777 */             g[i] = param.decode16BitsTo8Bits(this.colormap[gIndex + i] & 0xFFFF);
/*      */             
/*  779 */             b[i] = param.decode16BitsTo8Bits(this.colormap[bIndex + i] & 0xFFFF);
/*      */           } 
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/*  785 */         colorModel = new IndexColorModel(this.sampleSize, bandLength, r, g, b);
/*      */         break;
/*      */ 
/*      */ 
/*      */       
/*      */       default:
/*  791 */         throw new RuntimeException("TIFFImage4");
/*      */     } 
/*      */     
/*  794 */     Map properties = new HashMap();
/*      */     
/*  796 */     properties.put("tiff_directory", dir);
/*      */ 
/*      */ 
/*      */     
/*  800 */     init((CachableRed)null, bounds, colorModel, sampleModel, 0, 0, properties);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*  810 */   public TIFFDirectory getPrivateIFD(long offset) throws IOException { return new TIFFDirectory(this.stream, offset, 0); }
/*      */ 
/*      */ 
/*      */   
/*      */   public WritableRaster copyData(WritableRaster wr) {
/*  815 */     copyToRaster(wr);
/*  816 */     return wr;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public Raster getTile(int tileX, int tileY) {
/*      */     Rectangle rectangle;
/*  824 */     if (tileX < 0 || tileX >= this.tilesX || tileY < 0 || tileY >= this.tilesY)
/*      */     {
/*  826 */       throw new IllegalArgumentException("TIFFImage12");
/*      */     }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  833 */     byte[] bdata = null;
/*  834 */     short[] sdata = null;
/*  835 */     int[] idata = null;
/*      */     
/*  837 */     SampleModel sampleModel = getSampleModel();
/*  838 */     WritableRaster tile = makeTile(tileX, tileY);
/*      */     
/*  840 */     DataBuffer buffer = tile.getDataBuffer();
/*      */     
/*  842 */     int dataType = sampleModel.getDataType();
/*  843 */     if (dataType == 0) {
/*  844 */       bdata = ((DataBufferByte)buffer).getData();
/*  845 */     } else if (dataType == 1) {
/*  846 */       sdata = ((DataBufferUShort)buffer).getData();
/*  847 */     } else if (dataType == 2) {
/*  848 */       sdata = ((DataBufferShort)buffer).getData();
/*  849 */     } else if (dataType == 3) {
/*  850 */       idata = ((DataBufferInt)buffer).getData();
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  859 */     long save_offset = 0L;
/*      */     try {
/*  861 */       save_offset = this.stream.getFilePointer();
/*  862 */       this.stream.seek(this.tileOffsets[tileY * this.tilesX + tileX]);
/*      */     } catch (IOException ioe) {
/*  864 */       throw new RuntimeException("TIFFImage13");
/*      */     } 
/*      */ 
/*      */     
/*  868 */     int byteCount = (int)this.tileByteCounts[tileY * this.tilesX + tileX];
/*      */ 
/*      */ 
/*      */     
/*  872 */     if (!this.tiled) {
/*  873 */       rectangle = tile.getBounds();
/*      */     } else {
/*  875 */       rectangle = new Rectangle(tile.getMinX(), tile.getMinY(), this.tileWidth, this.tileHeight);
/*      */     } 
/*      */     
/*  878 */     int unitsInThisTile = rectangle.width * rectangle.height * this.numBands;
/*      */ 
/*      */     
/*  881 */     byte[] data = (this.compression != 1 || this.imageType == 4) ? new byte[byteCount] : null;
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  886 */     if (this.imageType == 0) {
/*      */       try {
/*  888 */         if (this.compression == 32773) {
/*  889 */           int m; this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  894 */           if (rectangle.width % 8 == 0) {
/*  895 */             m = rectangle.width / 8 * rectangle.height;
/*      */           } else {
/*  897 */             m = (rectangle.width / 8 + 1) * rectangle.height;
/*      */           } 
/*      */           
/*  900 */           decodePackbits(data, m, bdata);
/*  901 */         } else if (this.compression == 5) {
/*  902 */           this.stream.readFully(data, 0, byteCount);
/*  903 */           this.lzwDecoder.decode(data, bdata, rectangle.height);
/*  904 */         } else if (this.compression == 2) {
/*  905 */           this.stream.readFully(data, 0, byteCount);
/*  906 */           this.decoder.decode1D(bdata, data, 0, rectangle.height);
/*  907 */         } else if (this.compression == 3) {
/*  908 */           this.stream.readFully(data, 0, byteCount);
/*  909 */           this.decoder.decode2D(bdata, data, 0, rectangle.height, this.tiffT4Options);
/*      */         }
/*  911 */         else if (this.compression == 4) {
/*  912 */           this.stream.readFully(data, 0, byteCount);
/*  913 */           this.decoder.decodeT6(bdata, data, 0, rectangle.height, this.tiffT6Options);
/*      */         }
/*  915 */         else if (this.compression == 32946) {
/*  916 */           this.stream.readFully(data, 0, byteCount);
/*  917 */           inflate(data, bdata);
/*  918 */         } else if (this.compression == 1) {
/*  919 */           this.stream.readFully(bdata, 0, byteCount);
/*      */         } 
/*      */         
/*  922 */         this.stream.seek(save_offset);
/*      */       } catch (IOException ioe) {
/*  924 */         throw new RuntimeException("TIFFImage13");
/*      */       } 
/*  926 */     } else if (this.imageType == 4) {
/*  927 */       if (this.sampleSize == 16) {
/*      */         
/*  929 */         if (this.decodePaletteAsShorts) {
/*      */           
/*  931 */           short[] tempData = null;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  937 */           int unitsBeforeLookup = unitsInThisTile / 3;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  943 */           int entries = unitsBeforeLookup * 2;
/*      */ 
/*      */ 
/*      */           
/*      */           try {
/*  948 */             if (this.compression == 32773) {
/*      */               
/*  950 */               this.stream.readFully(data, 0, byteCount);
/*      */               
/*  952 */               byte[] byteArray = new byte[entries];
/*  953 */               decodePackbits(data, entries, byteArray);
/*  954 */               tempData = new short[unitsBeforeLookup];
/*  955 */               interpretBytesAsShorts(byteArray, tempData, unitsBeforeLookup);
/*      */             
/*      */             }
/*  958 */             else if (this.compression == 5) {
/*      */ 
/*      */               
/*  961 */               this.stream.readFully(data, 0, byteCount);
/*      */               
/*  963 */               byte[] byteArray = new byte[entries];
/*  964 */               this.lzwDecoder.decode(data, byteArray, rectangle.height);
/*  965 */               tempData = new short[unitsBeforeLookup];
/*  966 */               interpretBytesAsShorts(byteArray, tempData, unitsBeforeLookup);
/*      */             
/*      */             }
/*  969 */             else if (this.compression == 32946) {
/*      */               
/*  971 */               this.stream.readFully(data, 0, byteCount);
/*  972 */               byte[] byteArray = new byte[entries];
/*  973 */               inflate(data, byteArray);
/*  974 */               tempData = new short[unitsBeforeLookup];
/*  975 */               interpretBytesAsShorts(byteArray, tempData, unitsBeforeLookup);
/*      */             
/*      */             }
/*  978 */             else if (this.compression == 1) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */               
/*  984 */               tempData = new short[byteCount / 2];
/*  985 */               readShorts(byteCount / 2, tempData);
/*      */             } 
/*      */             
/*  988 */             this.stream.seek(save_offset);
/*      */           } catch (IOException ioe) {
/*      */             
/*  991 */             throw new RuntimeException("TIFFImage13");
/*      */           } 
/*      */           
/*  994 */           if (dataType == 1) {
/*      */ 
/*      */ 
/*      */ 
/*      */             
/*  999 */             int count = 0, len = this.colormap.length / 3;
/* 1000 */             int len2 = len * 2;
/* 1001 */             for (int i = 0; i < unitsBeforeLookup; i++)
/*      */             {
/* 1003 */               int lookup = tempData[i] & 0xFFFF;
/*      */               
/* 1005 */               int cmapValue = this.colormap[lookup + len2];
/* 1006 */               sdata[count++] = (short)(cmapValue & 0xFFFF);
/*      */               
/* 1008 */               cmapValue = this.colormap[lookup + len];
/* 1009 */               sdata[count++] = (short)(cmapValue & 0xFFFF);
/*      */               
/* 1011 */               cmapValue = this.colormap[lookup];
/* 1012 */               sdata[count++] = (short)(cmapValue & 0xFFFF);
/*      */             }
/*      */           
/* 1015 */           } else if (dataType == 2) {
/*      */ 
/*      */ 
/*      */ 
/*      */             
/* 1020 */             int count = 0, len = this.colormap.length / 3;
/* 1021 */             int len2 = len * 2;
/* 1022 */             for (int i = 0; i < unitsBeforeLookup; i++) {
/*      */               
/* 1024 */               int lookup = tempData[i] & 0xFFFF;
/*      */               
/* 1026 */               int cmapValue = this.colormap[lookup + len2];
/* 1027 */               sdata[count++] = (short)cmapValue;
/*      */               
/* 1029 */               cmapValue = this.colormap[lookup + len];
/* 1030 */               sdata[count++] = (short)cmapValue;
/*      */               
/* 1032 */               cmapValue = this.colormap[lookup];
/* 1033 */               sdata[count++] = (short)cmapValue;
/*      */             } 
/*      */           } 
/*      */         } else {
/*      */ 
/*      */           
/*      */           try {
/*      */ 
/*      */ 
/*      */ 
/*      */             
/* 1044 */             if (this.compression == 32773) {
/*      */               
/* 1046 */               this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */               
/* 1053 */               int bytesInThisTile = unitsInThisTile * 2;
/*      */               
/* 1055 */               byte[] byteArray = new byte[bytesInThisTile];
/* 1056 */               decodePackbits(data, bytesInThisTile, byteArray);
/* 1057 */               interpretBytesAsShorts(byteArray, sdata, unitsInThisTile);
/*      */             
/*      */             }
/* 1060 */             else if (this.compression == 5) {
/*      */               
/* 1062 */               this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */               
/* 1069 */               byte[] byteArray = new byte[unitsInThisTile * 2];
/* 1070 */               this.lzwDecoder.decode(data, byteArray, rectangle.height);
/* 1071 */               interpretBytesAsShorts(byteArray, sdata, unitsInThisTile);
/*      */             
/*      */             }
/* 1074 */             else if (this.compression == 32946) {
/*      */               
/* 1076 */               this.stream.readFully(data, 0, byteCount);
/* 1077 */               byte[] byteArray = new byte[unitsInThisTile * 2];
/* 1078 */               inflate(data, byteArray);
/* 1079 */               interpretBytesAsShorts(byteArray, sdata, unitsInThisTile);
/*      */             
/*      */             }
/* 1082 */             else if (this.compression == 1) {
/*      */               
/* 1084 */               readShorts(byteCount / 2, sdata);
/*      */             } 
/*      */             
/* 1087 */             this.stream.seek(save_offset);
/*      */           } catch (IOException ioe) {
/*      */             
/* 1090 */             throw new RuntimeException("TIFFImage13");
/*      */           }
/*      */         
/*      */         } 
/* 1094 */       } else if (this.sampleSize == 8) {
/*      */         
/* 1096 */         if (this.decodePaletteAsShorts) {
/*      */           
/* 1098 */           byte[] tempData = null;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 1104 */           int unitsBeforeLookup = unitsInThisTile / 3;
/*      */ 
/*      */ 
/*      */           
/*      */           try {
/* 1109 */             if (this.compression == 32773) {
/*      */               
/* 1111 */               this.stream.readFully(data, 0, byteCount);
/* 1112 */               tempData = new byte[unitsBeforeLookup];
/* 1113 */               decodePackbits(data, unitsBeforeLookup, tempData);
/*      */             }
/* 1115 */             else if (this.compression == 5) {
/*      */               
/* 1117 */               this.stream.readFully(data, 0, byteCount);
/* 1118 */               tempData = new byte[unitsBeforeLookup];
/* 1119 */               this.lzwDecoder.decode(data, tempData, rectangle.height);
/*      */             }
/* 1121 */             else if (this.compression == 7) {
/*      */               
/* 1123 */               this.stream.readFully(data, 0, byteCount);
/* 1124 */               Raster tempTile = decodeJPEG(data, this.decodeParam, this.colorConvertJPEG, tile.getMinX(), tile.getMinY());
/*      */ 
/*      */ 
/*      */ 
/*      */               
/* 1129 */               int[] tempPixels = new int[unitsBeforeLookup];
/* 1130 */               tempTile.getPixels(tile.getMinX(), tile.getMinY(), tile.getWidth(), tile.getHeight(), tempPixels);
/*      */ 
/*      */ 
/*      */ 
/*      */               
/* 1135 */               tempData = new byte[unitsBeforeLookup];
/* 1136 */               for (int i = 0; i < unitsBeforeLookup; i++) {
/* 1137 */                 tempData[i] = (byte)tempPixels[i];
/*      */               }
/*      */             }
/* 1140 */             else if (this.compression == 32946) {
/*      */               
/* 1142 */               this.stream.readFully(data, 0, byteCount);
/* 1143 */               tempData = new byte[unitsBeforeLookup];
/* 1144 */               inflate(data, tempData);
/*      */             }
/* 1146 */             else if (this.compression == 1) {
/*      */               
/* 1148 */               tempData = new byte[byteCount];
/* 1149 */               this.stream.readFully(tempData, 0, byteCount);
/*      */             } 
/*      */             
/* 1152 */             this.stream.seek(save_offset);
/*      */           } catch (IOException ioe) {
/*      */             
/* 1155 */             throw new RuntimeException("TIFFImage13");
/*      */           } 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 1161 */           int count = 0, len = this.colormap.length / 3;
/* 1162 */           int len2 = len * 2;
/* 1163 */           for (int i = 0; i < unitsBeforeLookup; i++) {
/*      */             
/* 1165 */             int lookup = tempData[i] & 0xFF;
/*      */             
/* 1167 */             int cmapValue = this.colormap[lookup + len2];
/* 1168 */             sdata[count++] = (short)(cmapValue & 0xFFFF);
/*      */             
/* 1170 */             cmapValue = this.colormap[lookup + len];
/* 1171 */             sdata[count++] = (short)(cmapValue & 0xFFFF);
/*      */             
/* 1173 */             cmapValue = this.colormap[lookup];
/* 1174 */             sdata[count++] = (short)(cmapValue & 0xFFFF);
/*      */           } 
/*      */         } else {
/*      */ 
/*      */           
/*      */           try {
/*      */ 
/*      */ 
/*      */             
/* 1183 */             if (this.compression == 32773) {
/*      */               
/* 1185 */               this.stream.readFully(data, 0, byteCount);
/* 1186 */               decodePackbits(data, unitsInThisTile, bdata);
/*      */             }
/* 1188 */             else if (this.compression == 5) {
/*      */               
/* 1190 */               this.stream.readFully(data, 0, byteCount);
/* 1191 */               this.lzwDecoder.decode(data, bdata, rectangle.height);
/*      */             }
/* 1193 */             else if (this.compression == 7) {
/*      */               
/* 1195 */               this.stream.readFully(data, 0, byteCount);
/* 1196 */               tile.setRect(decodeJPEG(data, this.decodeParam, this.colorConvertJPEG, tile.getMinX(), tile.getMinY()));
/*      */ 
/*      */ 
/*      */ 
/*      */             
/*      */             }
/* 1202 */             else if (this.compression == 32946) {
/*      */               
/* 1204 */               this.stream.readFully(data, 0, byteCount);
/* 1205 */               inflate(data, bdata);
/*      */             }
/* 1207 */             else if (this.compression == 1) {
/*      */               
/* 1209 */               this.stream.readFully(bdata, 0, byteCount);
/*      */             } 
/*      */             
/* 1212 */             this.stream.seek(save_offset);
/*      */           } catch (IOException ioe) {
/*      */             
/* 1215 */             throw new RuntimeException("TIFFImage13");
/*      */           }
/*      */         
/*      */         } 
/* 1219 */       } else if (this.sampleSize == 4) {
/*      */         
/* 1221 */         int padding = (rectangle.width % 2 == 0) ? 0 : 1;
/* 1222 */         int bytesPostDecoding = (rectangle.width / 2 + padding) * rectangle.height;
/*      */ 
/*      */ 
/*      */         
/* 1226 */         if (this.decodePaletteAsShorts) {
/*      */           
/* 1228 */           byte[] tempData = null;
/*      */           
/*      */           try {
/* 1231 */             this.stream.readFully(data, 0, byteCount);
/* 1232 */             this.stream.seek(save_offset);
/*      */           } catch (IOException ioe) {
/* 1234 */             throw new RuntimeException("TIFFImage13");
/*      */           } 
/*      */ 
/*      */           
/* 1238 */           if (this.compression == 32773) {
/*      */             
/* 1240 */             tempData = new byte[bytesPostDecoding];
/* 1241 */             decodePackbits(data, bytesPostDecoding, tempData);
/*      */           }
/* 1243 */           else if (this.compression == 5) {
/*      */             
/* 1245 */             tempData = new byte[bytesPostDecoding];
/* 1246 */             this.lzwDecoder.decode(data, tempData, rectangle.height);
/*      */           }
/* 1248 */           else if (this.compression == 32946) {
/*      */             
/* 1250 */             tempData = new byte[bytesPostDecoding];
/* 1251 */             inflate(data, tempData);
/*      */           }
/* 1253 */           else if (this.compression == 1) {
/*      */             
/* 1255 */             tempData = data;
/*      */           } 
/*      */           
/* 1258 */           int bytes = unitsInThisTile / 3;
/*      */ 
/*      */           
/* 1261 */           data = new byte[bytes];
/*      */           
/* 1263 */           int srcCount = 0, dstCount = 0;
/* 1264 */           for (int j = 0; j < rectangle.height; j++) {
/* 1265 */             for (int i = 0; i < rectangle.width / 2; i++) {
/* 1266 */               data[dstCount++] = (byte)((tempData[srcCount] & 0xF0) >> 4);
/*      */               
/* 1268 */               data[dstCount++] = (byte)(tempData[srcCount++] & 0xF);
/*      */             } 
/*      */ 
/*      */             
/* 1272 */             if (padding == 1) {
/* 1273 */               data[dstCount++] = (byte)((tempData[srcCount++] & 0xF0) >> 4);
/*      */             }
/*      */           } 
/*      */ 
/*      */           
/* 1278 */           int len = this.colormap.length / 3;
/* 1279 */           int len2 = len * 2;
/*      */           
/* 1281 */           int count = 0;
/* 1282 */           for (int i = 0; i < bytes; i++) {
/* 1283 */             int lookup = data[i] & 0xFF;
/* 1284 */             int cmapValue = this.colormap[lookup + len2];
/* 1285 */             sdata[count++] = (short)(cmapValue & 0xFFFF);
/* 1286 */             cmapValue = this.colormap[lookup + len];
/* 1287 */             sdata[count++] = (short)(cmapValue & 0xFFFF);
/* 1288 */             cmapValue = this.colormap[lookup];
/* 1289 */             sdata[count++] = (short)(cmapValue & 0xFFFF);
/*      */           } 
/*      */         } else {
/*      */ 
/*      */           
/*      */           try {
/*      */ 
/*      */             
/* 1297 */             if (this.compression == 32773) {
/*      */               
/* 1299 */               this.stream.readFully(data, 0, byteCount);
/* 1300 */               decodePackbits(data, bytesPostDecoding, bdata);
/*      */             }
/* 1302 */             else if (this.compression == 5) {
/*      */               
/* 1304 */               this.stream.readFully(data, 0, byteCount);
/* 1305 */               this.lzwDecoder.decode(data, bdata, rectangle.height);
/*      */             }
/* 1307 */             else if (this.compression == 32946) {
/*      */               
/* 1309 */               this.stream.readFully(data, 0, byteCount);
/* 1310 */               inflate(data, bdata);
/*      */             }
/* 1312 */             else if (this.compression == 1) {
/*      */               
/* 1314 */               this.stream.readFully(bdata, 0, byteCount);
/*      */             } 
/*      */             
/* 1317 */             this.stream.seek(save_offset);
/*      */           } catch (IOException ioe) {
/*      */             
/* 1320 */             throw new RuntimeException("TIFFImage13");
/*      */           } 
/*      */         } 
/*      */       } 
/* 1324 */     } else if (this.imageType == 1) {
/*      */       try {
/* 1326 */         if (this.compression == 32773) {
/*      */           int m;
/* 1328 */           this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */           
/* 1333 */           if (rectangle.width % 8 == 0) {
/* 1334 */             m = rectangle.width / 2 * rectangle.height;
/*      */           } else {
/* 1336 */             m = (rectangle.width / 2 + 1) * rectangle.height;
/*      */           } 
/*      */ 
/*      */           
/* 1340 */           decodePackbits(data, m, bdata);
/*      */         }
/* 1342 */         else if (this.compression == 5) {
/*      */           
/* 1344 */           this.stream.readFully(data, 0, byteCount);
/* 1345 */           this.lzwDecoder.decode(data, bdata, rectangle.height);
/*      */         }
/* 1347 */         else if (this.compression == 32946) {
/*      */           
/* 1349 */           this.stream.readFully(data, 0, byteCount);
/* 1350 */           inflate(data, bdata);
/*      */         }
/*      */         else {
/*      */           
/* 1354 */           this.stream.readFully(bdata, 0, byteCount);
/*      */         } 
/*      */         
/* 1357 */         this.stream.seek(save_offset);
/*      */       } catch (IOException ioe) {
/* 1359 */         throw new RuntimeException("TIFFImage13");
/*      */       } 
/*      */     } else {
/*      */       int j; int y; int offsetCr; int offsetCb; int bOffset; int[] pixels; int samplesPerDataUnit; byte[] tempData; int numV; int numH; int pixelsPerDataUnit;
/*      */       try {
/* 1364 */         if (this.sampleSize == 8) {
/*      */           
/* 1366 */           if (this.compression == 1) {
/* 1367 */             this.stream.readFully(bdata, 0, byteCount);
/*      */           }
/* 1369 */           else if (this.compression == 5) {
/*      */             
/* 1371 */             this.stream.readFully(data, 0, byteCount);
/* 1372 */             this.lzwDecoder.decode(data, bdata, rectangle.height);
/*      */           }
/* 1374 */           else if (this.compression == 32773) {
/*      */             
/* 1376 */             this.stream.readFully(data, 0, byteCount);
/* 1377 */             decodePackbits(data, unitsInThisTile, bdata);
/*      */           }
/* 1379 */           else if (this.compression == 7) {
/*      */             
/* 1381 */             this.stream.readFully(data, 0, byteCount);
/* 1382 */             tile.setRect(decodeJPEG(data, this.decodeParam, this.colorConvertJPEG, tile.getMinX(), tile.getMinY()));
/*      */ 
/*      */ 
/*      */           
/*      */           }
/* 1387 */           else if (this.compression == 32946) {
/*      */             
/* 1389 */             this.stream.readFully(data, 0, byteCount);
/* 1390 */             inflate(data, bdata);
/*      */           }
/*      */         
/* 1393 */         } else if (this.sampleSize == 16) {
/*      */           
/* 1395 */           if (this.compression == 1)
/*      */           {
/* 1397 */             readShorts(byteCount / 2, sdata);
/*      */           }
/* 1399 */           else if (this.compression == 5)
/*      */           {
/* 1401 */             this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             
/* 1408 */             byte[] byteArray = new byte[unitsInThisTile * 2];
/* 1409 */             this.lzwDecoder.decode(data, byteArray, rectangle.height);
/* 1410 */             interpretBytesAsShorts(byteArray, sdata, unitsInThisTile);
/*      */           
/*      */           }
/* 1413 */           else if (this.compression == 32773)
/*      */           {
/* 1415 */             this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             
/* 1422 */             int bytesInThisTile = unitsInThisTile * 2;
/*      */             
/* 1424 */             byte[] byteArray = new byte[bytesInThisTile];
/* 1425 */             decodePackbits(data, bytesInThisTile, byteArray);
/* 1426 */             interpretBytesAsShorts(byteArray, sdata, unitsInThisTile);
/*      */           }
/* 1428 */           else if (this.compression == 32946)
/*      */           {
/* 1430 */             this.stream.readFully(data, 0, byteCount);
/* 1431 */             byte[] byteArray = new byte[unitsInThisTile * 2];
/* 1432 */             inflate(data, byteArray);
/* 1433 */             interpretBytesAsShorts(byteArray, sdata, unitsInThisTile);
/*      */           }
/*      */         
/*      */         }
/* 1437 */         else if (this.sampleSize == 32 && dataType == 3) {
/*      */           
/* 1439 */           if (this.compression == 1) {
/*      */             
/* 1441 */             readInts(byteCount / 4, idata);
/*      */           }
/* 1443 */           else if (this.compression == 5) {
/*      */             
/* 1445 */             this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             
/* 1452 */             byte[] byteArray = new byte[unitsInThisTile * 4];
/* 1453 */             this.lzwDecoder.decode(data, byteArray, rectangle.height);
/* 1454 */             interpretBytesAsInts(byteArray, idata, unitsInThisTile);
/*      */           
/*      */           }
/* 1457 */           else if (this.compression == 32773) {
/*      */             
/* 1459 */             this.stream.readFully(data, 0, byteCount);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */             
/* 1466 */             int bytesInThisTile = unitsInThisTile * 4;
/*      */             
/* 1468 */             byte[] byteArray = new byte[bytesInThisTile];
/* 1469 */             decodePackbits(data, bytesInThisTile, byteArray);
/* 1470 */             interpretBytesAsInts(byteArray, idata, unitsInThisTile);
/*      */           }
/* 1472 */           else if (this.compression == 32946) {
/*      */             
/* 1474 */             this.stream.readFully(data, 0, byteCount);
/* 1475 */             byte[] byteArray = new byte[unitsInThisTile * 4];
/* 1476 */             inflate(data, byteArray);
/* 1477 */             interpretBytesAsInts(byteArray, idata, unitsInThisTile);
/*      */           } 
/*      */         } 
/*      */ 
/*      */ 
/*      */         
/* 1483 */         this.stream.seek(save_offset);
/*      */       } catch (IOException ioe) {
/*      */         
/* 1486 */         throw new RuntimeException("TIFFImage13");
/*      */       } 
/*      */ 
/*      */       
/* 1490 */       switch (this.imageType) {
/*      */         case 2:
/*      */         case 3:
/* 1493 */           if (this.isWhiteZero) {
/*      */ 
/*      */ 
/*      */             
/* 1497 */             if (dataType == 0 && !(getColorModel() instanceof IndexColorModel)) {
/*      */ 
/*      */               
/* 1500 */               for (int l = 0; l < bdata.length; l += this.numBands)
/* 1501 */                 bdata[l] = (byte)('ÿ' - bdata[l]);  break;
/*      */             } 
/* 1503 */             if (dataType == 1) {
/*      */               
/* 1505 */               int ushortMax = 65535;
/* 1506 */               for (int l = 0; l < sdata.length; l += this.numBands)
/* 1507 */                 sdata[l] = (short)(ushortMax - sdata[l]); 
/*      */               break;
/*      */             } 
/* 1510 */             if (dataType == 2) {
/*      */               
/* 1512 */               for (int l = 0; l < sdata.length; l += this.numBands)
/* 1513 */                 sdata[l] = (short)(sdata[l] ^ 0xFFFFFFFF);  break;
/*      */             } 
/* 1515 */             if (dataType == 3) {
/*      */               
/* 1517 */               long uintMax = 4294967295L;
/*      */               
/* 1519 */               for (int l = 0; l < idata.length; l += this.numBands) {
/* 1520 */                 idata[l] = (int)(uintMax - idata[l]);
/*      */               }
/*      */             } 
/*      */           } 
/*      */           break;
/*      */ 
/*      */         
/*      */         case 5:
/* 1528 */           if (this.sampleSize == 8 && this.compression != 7) {
/* 1529 */             for (int i = 0; i < unitsInThisTile; i += 3) {
/* 1530 */               byte bswap = bdata[i];
/* 1531 */               bdata[i] = bdata[i + 2];
/* 1532 */               bdata[i + 2] = bswap;
/*      */             }  break;
/* 1534 */           }  if (this.sampleSize == 16) {
/* 1535 */             for (int i = 0; i < unitsInThisTile; i += 3) {
/* 1536 */               short sswap = sdata[i];
/* 1537 */               sdata[i] = sdata[i + 2];
/* 1538 */               sdata[i + 2] = sswap;
/*      */             }  break;
/* 1540 */           }  if (this.sampleSize == 32 && 
/* 1541 */             dataType == 3) {
/* 1542 */             for (int i = 0; i < unitsInThisTile; i += 3) {
/* 1543 */               int iswap = idata[i];
/* 1544 */               idata[i] = idata[i + 2];
/* 1545 */               idata[i + 2] = iswap;
/*      */             } 
/*      */           }
/*      */           break;
/*      */ 
/*      */         
/*      */         case 6:
/* 1552 */           if (this.sampleSize == 8) {
/* 1553 */             for (int i = 0; i < unitsInThisTile; i += 4) {
/*      */               
/* 1555 */               byte b = bdata[i];
/* 1556 */               bdata[i] = bdata[i + 3];
/* 1557 */               bdata[i + 3] = b;
/*      */ 
/*      */               
/* 1560 */               b = bdata[i + 1];
/* 1561 */               bdata[i + 1] = bdata[i + 2];
/* 1562 */               bdata[i + 2] = b;
/*      */             }  break;
/* 1564 */           }  if (this.sampleSize == 16) {
/* 1565 */             for (int i = 0; i < unitsInThisTile; i += 4) {
/*      */               
/* 1567 */               short s = sdata[i];
/* 1568 */               sdata[i] = sdata[i + 3];
/* 1569 */               sdata[i + 3] = s;
/*      */ 
/*      */               
/* 1572 */               s = sdata[i + 1];
/* 1573 */               sdata[i + 1] = sdata[i + 2];
/* 1574 */               sdata[i + 2] = s;
/*      */             }  break;
/* 1576 */           }  if (this.sampleSize == 32 && 
/* 1577 */             dataType == 3) {
/* 1578 */             for (int i = 0; i < unitsInThisTile; i += 4) {
/*      */               
/* 1580 */               int m = idata[i];
/* 1581 */               idata[i] = idata[i + 3];
/* 1582 */               idata[i + 3] = m;
/*      */ 
/*      */               
/* 1585 */               m = idata[i + 1];
/* 1586 */               idata[i + 1] = idata[i + 2];
/* 1587 */               idata[i + 2] = m;
/*      */             } 
/*      */           }
/*      */           break;
/*      */ 
/*      */ 
/*      */         
/*      */         case 7:
/* 1595 */           pixelsPerDataUnit = this.chromaSubH * this.chromaSubV;
/*      */           
/* 1597 */           numH = rectangle.width / this.chromaSubH;
/* 1598 */           numV = rectangle.height / this.chromaSubV;
/*      */           
/* 1600 */           tempData = new byte[numH * numV * (pixelsPerDataUnit + 2)];
/* 1601 */           System.arraycopy(bdata, 0, tempData, 0, tempData.length);
/*      */           
/* 1603 */           samplesPerDataUnit = pixelsPerDataUnit * 3;
/* 1604 */           pixels = new int[samplesPerDataUnit];
/*      */           
/* 1606 */           bOffset = 0;
/* 1607 */           offsetCb = pixelsPerDataUnit;
/* 1608 */           offsetCr = offsetCb + 1;
/*      */           
/* 1610 */           y = rectangle.y;
/* 1611 */           for (j = 0; j < numV; j++) {
/* 1612 */             int x = rectangle.x;
/* 1613 */             for (int i = 0; i < numH; i++) {
/* 1614 */               int Cb = tempData[bOffset + offsetCb];
/* 1615 */               int Cr = tempData[bOffset + offsetCr];
/* 1616 */               int k = 0;
/* 1617 */               while (k < samplesPerDataUnit) {
/* 1618 */                 pixels[k++] = tempData[bOffset++];
/* 1619 */                 pixels[k++] = Cb;
/* 1620 */                 pixels[k++] = Cr;
/*      */               } 
/* 1622 */               bOffset += 2;
/* 1623 */               tile.setPixels(x, y, this.chromaSubH, this.chromaSubV, pixels);
/* 1624 */               x += this.chromaSubH;
/*      */             } 
/* 1626 */             y += this.chromaSubV;
/*      */           } 
/*      */           break;
/*      */       } 
/*      */ 
/*      */     
/*      */     } 
/* 1633 */     return tile;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void readShorts(int shortCount, short[] shortArray) {
/* 1640 */     int byteCount = 2 * shortCount;
/* 1641 */     byte[] byteArray = new byte[byteCount];
/*      */     
/*      */     try {
/* 1644 */       this.stream.readFully(byteArray, 0, byteCount);
/*      */     } catch (IOException ioe) {
/* 1646 */       throw new RuntimeException("TIFFImage13");
/*      */     } 
/*      */     
/* 1649 */     interpretBytesAsShorts(byteArray, shortArray, shortCount);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void readInts(int intCount, int[] intArray) {
/* 1656 */     int byteCount = 4 * intCount;
/* 1657 */     byte[] byteArray = new byte[byteCount];
/*      */     
/*      */     try {
/* 1660 */       this.stream.readFully(byteArray, 0, byteCount);
/*      */     } catch (IOException ioe) {
/* 1662 */       throw new RuntimeException("TIFFImage13");
/*      */     } 
/*      */     
/* 1665 */     interpretBytesAsInts(byteArray, intArray, intCount);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void interpretBytesAsShorts(byte[] byteArray, short[] shortArray, int shortCount) {
/* 1674 */     int j = 0;
/*      */ 
/*      */     
/* 1677 */     if (this.isBigEndian) {
/*      */       
/* 1679 */       for (int i = 0; i < shortCount; i++) {
/* 1680 */         int firstByte = byteArray[j++] & 0xFF;
/* 1681 */         int secondByte = byteArray[j++] & 0xFF;
/* 1682 */         shortArray[i] = (short)((firstByte << 8) + secondByte);
/*      */       }
/*      */     
/*      */     } else {
/*      */       
/* 1687 */       for (int i = 0; i < shortCount; i++) {
/* 1688 */         byte b1 = byteArray[j++] & 0xFF;
/* 1689 */         byte b2 = byteArray[j++] & 0xFF;
/* 1690 */         shortArray[i] = (short)((b2 << 8) + b1);
/*      */       } 
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void interpretBytesAsInts(byte[] byteArray, int[] intArray, int intCount) {
/* 1701 */     int j = 0;
/*      */     
/* 1703 */     if (this.isBigEndian) {
/*      */       
/* 1705 */       for (int i = 0; i < intCount; i++) {
/* 1706 */         intArray[i] = (byteArray[j++] & 0xFF) << 24 | (byteArray[j++] & 0xFF) << 16 | (byteArray[j++] & 0xFF) << 8 | byteArray[j++] & 0xFF;
/*      */       
/*      */       }
/*      */     
/*      */     }
/*      */     else {
/*      */ 
/*      */       
/* 1714 */       for (int i = 0; i < intCount; i++) {
/* 1715 */         intArray[i] = byteArray[j++] & 0xFF | (byteArray[j++] & 0xFF) << 8 | (byteArray[j++] & 0xFF) << 16 | (byteArray[j++] & 0xFF) << 24;
/*      */       }
/*      */     } 
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private byte[] decodePackbits(byte[] data, int arraySize, byte[] dst) {
/* 1726 */     if (dst == null) {
/* 1727 */       dst = new byte[arraySize];
/*      */     }
/*      */     
/* 1730 */     int srcCount = 0, dstCount = 0;
/*      */ 
/*      */ 
/*      */     
/*      */     try {
/* 1735 */       while (dstCount < arraySize) {
/*      */         
/* 1737 */         byte b = data[srcCount++];
/*      */         
/* 1739 */         if (b >= 0 && b <= Byte.MAX_VALUE) {
/*      */ 
/*      */           
/* 1742 */           for (int i = 0; i < b + 1; i++)
/* 1743 */             dst[dstCount++] = data[srcCount++]; 
/*      */           continue;
/*      */         } 
/* 1746 */         if (b <= -1 && b >= -127) {
/*      */ 
/*      */           
/* 1749 */           byte repeat = data[srcCount++];
/* 1750 */           for (int i = 0; i < -b + 1; i++) {
/* 1751 */             dst[dstCount++] = repeat;
/*      */           }
/*      */           
/*      */           continue;
/*      */         } 
/* 1756 */         srcCount++;
/*      */       } 
/*      */     } catch (ArrayIndexOutOfBoundsException ae) {
/*      */       
/* 1760 */       throw new RuntimeException("TIFFImage14");
/*      */     } 
/*      */     
/* 1763 */     return dst;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private ComponentColorModel createAlphaComponentColorModel(int dataType, int numBands, boolean isAlphaPremultiplied, int transparency) {
/* 1772 */     ccm = null;
/* 1773 */     int[] RGBBits = null;
/* 1774 */     ColorSpace cs = null;
/* 1775 */     switch (numBands) {
/*      */       case 2:
/* 1777 */         cs = ColorSpace.getInstance(1003);
/*      */         break;
/*      */       case 4:
/* 1780 */         cs = ColorSpace.getInstance(1000);
/*      */         break;
/*      */       default:
/* 1783 */         throw new IllegalArgumentException();
/*      */     } 
/*      */     
/* 1786 */     int componentSize = 0;
/* 1787 */     switch (dataType) {
/*      */       case 0:
/* 1789 */         componentSize = 8;
/*      */         break;
/*      */       case 1:
/*      */       case 2:
/* 1793 */         componentSize = 16;
/*      */         break;
/*      */       case 3:
/* 1796 */         componentSize = 32;
/*      */         break;
/*      */       default:
/* 1799 */         throw new IllegalArgumentException();
/*      */     } 
/*      */     
/* 1802 */     RGBBits = new int[numBands];
/* 1803 */     for (int i = 0; i < numBands; i++) {
/* 1804 */       RGBBits[i] = componentSize;
/*      */     }
/*      */     
/* 1807 */     return new ComponentColorModel(cs, RGBBits, true, isAlphaPremultiplied, transparency, dataType);
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\tiff\TIFFImage.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */