/*      */ package org.apache.batik.ext.awt.image.codec.tiff;
/*      */ 
/*      */ import com.sun.image.codec.jpeg.JPEGCodec;
/*      */ import com.sun.image.codec.jpeg.JPEGEncodeParam;
/*      */ import com.sun.image.codec.jpeg.JPEGImageEncoder;
/*      */ import com.sun.image.codec.jpeg.JPEGQTable;
/*      */ import java.awt.Rectangle;
/*      */ import java.awt.color.ColorSpace;
/*      */ import java.awt.image.BufferedImage;
/*      */ import java.awt.image.ColorModel;
/*      */ import java.awt.image.ComponentSampleModel;
/*      */ import java.awt.image.DataBufferByte;
/*      */ import java.awt.image.IndexColorModel;
/*      */ import java.awt.image.MultiPixelPackedSampleModel;
/*      */ import java.awt.image.Raster;
/*      */ import java.awt.image.RenderedImage;
/*      */ import java.awt.image.SampleModel;
/*      */ import java.awt.image.WritableRaster;
/*      */ import java.io.ByteArrayOutputStream;
/*      */ import java.io.File;
/*      */ import java.io.FileInputStream;
/*      */ import java.io.IOException;
/*      */ import java.io.OutputStream;
/*      */ import java.io.RandomAccessFile;
/*      */ import java.util.ArrayList;
/*      */ import java.util.Iterator;
/*      */ import java.util.List;
/*      */ import java.util.SortedSet;
/*      */ import java.util.TreeSet;
/*      */ import java.util.zip.Deflater;
/*      */ import org.apache.batik.ext.awt.image.codec.util.ImageEncodeParam;
/*      */ import org.apache.batik.ext.awt.image.codec.util.ImageEncoderImpl;
/*      */ import org.apache.batik.ext.awt.image.codec.util.SeekableOutputStream;
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ public class TIFFImageEncoder
/*      */   extends ImageEncoderImpl
/*      */ {
/*      */   private static final int TIFF_UNSUPPORTED = -1;
/*      */   private static final int TIFF_BILEVEL_WHITE_IS_ZERO = 0;
/*      */   private static final int TIFF_BILEVEL_BLACK_IS_ZERO = 1;
/*      */   private static final int TIFF_GRAY = 2;
/*      */   private static final int TIFF_PALETTE = 3;
/*      */   private static final int TIFF_RGB = 4;
/*      */   private static final int TIFF_CMYK = 5;
/*      */   private static final int TIFF_YCBCR = 6;
/*      */   private static final int TIFF_CIELAB = 7;
/*      */   private static final int TIFF_GENERIC = 8;
/*      */   private static final int COMP_NONE = 1;
/*      */   private static final int COMP_JPEG_TTN2 = 7;
/*      */   private static final int COMP_PACKBITS = 32773;
/*      */   private static final int COMP_DEFLATE = 32946;
/*      */   private static final int TIFF_JPEG_TABLES = 347;
/*      */   private static final int TIFF_YCBCR_SUBSAMPLING = 530;
/*      */   private static final int TIFF_YCBCR_POSITIONING = 531;
/*      */   private static final int TIFF_REF_BLACK_WHITE = 532;
/*      */   private static final int EXTRA_SAMPLE_UNSPECIFIED = 0;
/*      */   private static final int EXTRA_SAMPLE_ASSOCIATED_ALPHA = 1;
/*      */   private static final int EXTRA_SAMPLE_UNASSOCIATED_ALPHA = 2;
/*      */   private static final int DEFAULT_ROWS_PER_STRIP = 8;
/*      */   
/*      */   public TIFFImageEncoder(OutputStream output, ImageEncodeParam param) {
/*   95 */     super(output, param);
/*   96 */     if (this.param == null) {
/*   97 */       this.param = new TIFFEncodeParam();
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   public void encode(RenderedImage im) throws IOException {
/*  107 */     writeFileHeader();
/*      */ 
/*      */     
/*  110 */     TIFFEncodeParam encodeParam = (TIFFEncodeParam)this.param;
/*      */     
/*  112 */     Iterator iter = encodeParam.getExtraImages();
/*  113 */     if (iter != null) {
/*  114 */       boolean hasNext; int ifdOffset = 8;
/*  115 */       RenderedImage nextImage = im;
/*  116 */       TIFFEncodeParam nextParam = encodeParam;
/*      */       
/*      */       do {
/*  119 */         hasNext = iter.hasNext();
/*  120 */         ifdOffset = encode(nextImage, nextParam, ifdOffset, !hasNext);
/*  121 */         if (!hasNext)
/*  122 */           continue;  Object obj = iter.next();
/*  123 */         if (obj instanceof RenderedImage) {
/*  124 */           nextImage = (RenderedImage)obj;
/*  125 */           nextParam = encodeParam;
/*  126 */         } else if (obj instanceof Object[]) {
/*  127 */           Object[] o = (Object[])obj;
/*  128 */           nextImage = (RenderedImage)o[0];
/*  129 */           nextParam = (TIFFEncodeParam)o[1];
/*      */         }
/*      */       
/*  132 */       } while (hasNext);
/*      */     } else {
/*  134 */       encode(im, encodeParam, 8, true);
/*      */     } 
/*      */   }
/*      */   
/*      */   private int encode(RenderedImage im, TIFFEncodeParam encodeParam, int ifdOffset, boolean isLast) throws IOException {
/*      */     int i1, n, m, i, blueIndex, greenIndex, redIndex;
/*      */     byte[] b, g, r;
/*  141 */     int compression = encodeParam.getCompression();
/*      */ 
/*      */     
/*  144 */     boolean isTiled = encodeParam.getWriteTiled();
/*      */ 
/*      */     
/*  147 */     int minX = im.getMinX();
/*  148 */     int minY = im.getMinY();
/*  149 */     int width = im.getWidth();
/*  150 */     int height = im.getHeight();
/*      */ 
/*      */     
/*  153 */     SampleModel sampleModel = im.getSampleModel();
/*      */ 
/*      */     
/*  156 */     int[] sampleSize = sampleModel.getSampleSize();
/*  157 */     for (int i = 1; i < sampleSize.length; i++) {
/*  158 */       if (sampleSize[i] != sampleSize[0]) {
/*  159 */         throw new Error("TIFFImageEncoder0");
/*      */       }
/*      */     } 
/*      */ 
/*      */     
/*  164 */     int numBands = sampleModel.getNumBands();
/*  165 */     if ((sampleSize[0] == 1 || sampleSize[0] == 4) && numBands != 1) {
/*  166 */       throw new Error("TIFFImageEncoder1");
/*      */     }
/*      */ 
/*      */     
/*  170 */     int dataType = sampleModel.getDataType();
/*  171 */     switch (dataType) {
/*      */       case 0:
/*  173 */         if (sampleSize[0] != 1 && sampleSize[0] == 4 && sampleSize[0] != 8)
/*      */         {
/*  175 */           throw new Error("TIFFImageEncoder2");
/*      */         }
/*      */         break;
/*      */       case 1:
/*      */       case 2:
/*  180 */         if (sampleSize[0] != 16) {
/*  181 */           throw new Error("TIFFImageEncoder3");
/*      */         }
/*      */         break;
/*      */       case 3:
/*      */       case 4:
/*  186 */         if (sampleSize[0] != 32) {
/*  187 */           throw new Error("TIFFImageEncoder4");
/*      */         }
/*      */         break;
/*      */       default:
/*  191 */         throw new Error("TIFFImageEncoder5");
/*      */     } 
/*      */     
/*  194 */     boolean dataTypeIsShort = (dataType == 2 || dataType == 1);
/*      */ 
/*      */ 
/*      */     
/*  198 */     ColorModel colorModel = im.getColorModel();
/*  199 */     if (colorModel != null && colorModel instanceof IndexColorModel && dataType != 0)
/*      */     {
/*      */ 
/*      */       
/*  203 */       throw new Error("TIFFImageEncoder6");
/*      */     }
/*  205 */     IndexColorModel icm = null;
/*  206 */     int sizeOfColormap = 0;
/*  207 */     char[] colormap = null;
/*      */ 
/*      */     
/*  210 */     int imageType = -1;
/*  211 */     int numExtraSamples = 0;
/*  212 */     int extraSampleType = 0;
/*  213 */     if (colorModel instanceof IndexColorModel) {
/*  214 */       icm = (IndexColorModel)colorModel;
/*  215 */       int mapSize = icm.getMapSize();
/*      */       
/*  217 */       if (sampleSize[0] == 1 && numBands == 1) {
/*      */         
/*  219 */         if (mapSize != 2) {
/*  220 */           throw new IllegalArgumentException("TIFFImageEncoder7");
/*      */         }
/*      */ 
/*      */         
/*  224 */         byte[] r = new byte[mapSize];
/*  225 */         icm.getReds(r);
/*  226 */         byte[] g = new byte[mapSize];
/*  227 */         icm.getGreens(g);
/*  228 */         byte[] b = new byte[mapSize];
/*  229 */         icm.getBlues(b);
/*      */         
/*  231 */         if ((r[0] & 0xFF) == 0 && (r[1] & 0xFF) == 255 && (g[0] & 0xFF) == 0 && (g[1] & 0xFF) == 255 && (b[0] & 0xFF) == 0 && (b[1] & 0xFF) == 255) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  238 */           imageType = 1;
/*      */         }
/*  240 */         else if ((r[0] & 0xFF) == 255 && (r[1] & 0xFF) == 0 && (g[0] & 0xFF) == 255 && (g[1] & 0xFF) == 0 && (b[0] & 0xFF) == 255 && (b[1] & 0xFF) == 0) {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */           
/*  247 */           imageType = 0;
/*      */         } else {
/*      */           
/*  250 */           imageType = 3;
/*      */         }
/*      */       
/*  253 */       } else if (numBands == 1) {
/*      */         
/*  255 */         imageType = 3;
/*      */       } 
/*  257 */     } else if (colorModel == null) {
/*      */       
/*  259 */       if (sampleSize[0] == 1 && numBands == 1) {
/*  260 */         imageType = 1;
/*      */       } else {
/*  262 */         imageType = 8;
/*  263 */         if (numBands > 1) {
/*  264 */           numExtraSamples = numBands - 1;
/*      */         }
/*      */       } 
/*      */     } else {
/*      */       
/*  269 */       ColorSpace colorSpace = colorModel.getColorSpace();
/*      */       
/*  271 */       switch (colorSpace.getType()) {
/*      */         case 9:
/*  273 */           imageType = 5;
/*      */           break;
/*      */         case 6:
/*  276 */           imageType = 2;
/*      */           break;
/*      */         case 1:
/*  279 */           imageType = 7;
/*      */           break;
/*      */         case 5:
/*  282 */           if (compression == 7 && encodeParam.getJPEGCompressRGBToYCbCr()) {
/*      */             
/*  284 */             imageType = 6; break;
/*      */           } 
/*  286 */           imageType = 4;
/*      */           break;
/*      */         
/*      */         case 3:
/*  290 */           imageType = 6;
/*      */           break;
/*      */         default:
/*  293 */           imageType = 8;
/*      */           break;
/*      */       } 
/*      */       
/*  297 */       if (imageType == 8) {
/*  298 */         numExtraSamples = numBands - 1;
/*  299 */       } else if (numBands > 1) {
/*  300 */         numExtraSamples = numBands - colorSpace.getNumComponents();
/*      */       } 
/*      */       
/*  303 */       if (numExtraSamples == 1 && colorModel.hasAlpha()) {
/*  304 */         extraSampleType = colorModel.isAlphaPremultiplied() ? 1 : 2;
/*      */       }
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  310 */     if (imageType == -1) {
/*  311 */       throw new Error("TIFFImageEncoder8");
/*      */     }
/*      */ 
/*      */     
/*  315 */     if (compression == 7) {
/*  316 */       if (imageType == 3)
/*  317 */         throw new Error("TIFFImageEncoder11"); 
/*  318 */       if (sampleSize[0] != 8 || (imageType != 2 && imageType != 4 && imageType != 6))
/*      */       {
/*      */ 
/*      */         
/*  322 */         throw new Error("TIFFImageEncoder9");
/*      */       }
/*      */     } 
/*      */     
/*  326 */     int photometricInterpretation = -1;
/*  327 */     switch (imageType) {
/*      */       
/*      */       case 0:
/*  330 */         photometricInterpretation = 0;
/*      */         break;
/*      */       
/*      */       case 1:
/*  334 */         photometricInterpretation = 1;
/*      */         break;
/*      */ 
/*      */       
/*      */       case 2:
/*      */       case 8:
/*  340 */         photometricInterpretation = 1;
/*      */         break;
/*      */       
/*      */       case 3:
/*  344 */         photometricInterpretation = 3;
/*      */         
/*  346 */         icm = (IndexColorModel)colorModel;
/*  347 */         sizeOfColormap = icm.getMapSize();
/*      */         
/*  349 */         r = new byte[sizeOfColormap];
/*  350 */         icm.getReds(r);
/*  351 */         g = new byte[sizeOfColormap];
/*  352 */         icm.getGreens(g);
/*  353 */         b = new byte[sizeOfColormap];
/*  354 */         icm.getBlues(b);
/*      */         
/*  356 */         redIndex = 0; greenIndex = sizeOfColormap;
/*  357 */         blueIndex = 2 * sizeOfColormap;
/*  358 */         colormap = new char[sizeOfColormap * 3];
/*  359 */         for (i = 0; i < sizeOfColormap; i++) {
/*  360 */           m = 0xFF & r[i];
/*  361 */           colormap[redIndex++] = (char)(m << 8 | m);
/*  362 */           m = 0xFF & g[i];
/*  363 */           colormap[greenIndex++] = (char)(m << 8 | m);
/*  364 */           m = 0xFF & b[i];
/*  365 */           colormap[blueIndex++] = (char)(m << 8 | m);
/*      */         } 
/*      */         
/*  368 */         sizeOfColormap *= 3;
/*      */         break;
/*      */ 
/*      */       
/*      */       case 4:
/*  373 */         photometricInterpretation = 2;
/*      */         break;
/*      */       
/*      */       case 5:
/*  377 */         photometricInterpretation = 5;
/*      */         break;
/*      */       
/*      */       case 6:
/*  381 */         photometricInterpretation = 6;
/*      */         break;
/*      */       
/*      */       case 7:
/*  385 */         photometricInterpretation = 8;
/*      */         break;
/*      */       
/*      */       default:
/*  389 */         throw new Error("TIFFImageEncoder8");
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  395 */     if (isTiled) {
/*  396 */       m = (encodeParam.getTileWidth() > 0) ? encodeParam.getTileWidth() : im.getTileWidth();
/*      */       
/*  398 */       n = (encodeParam.getTileHeight() > 0) ? encodeParam.getTileHeight() : im.getTileHeight();
/*      */     } else {
/*      */       
/*  401 */       m = width;
/*      */       
/*  403 */       n = (encodeParam.getTileHeight() > 0) ? encodeParam.getTileHeight() : 8;
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  408 */     JPEGEncodeParam jep = null;
/*  409 */     if (compression == 7) {
/*      */       
/*  411 */       jep = encodeParam.getJPEGEncodeParam();
/*      */ 
/*      */       
/*  414 */       i1 = jep.getHorizontalSubsampling(0);
/*  415 */       int maxSubV = jep.getVerticalSubsampling(0);
/*  416 */       for (int i = 1; i < numBands; i++) {
/*  417 */         int subH = jep.getHorizontalSubsampling(i);
/*  418 */         if (subH > i1) {
/*  419 */           i1 = subH;
/*      */         }
/*  421 */         int subV = jep.getVerticalSubsampling(i);
/*  422 */         if (subV > maxSubV) {
/*  423 */           maxSubV = subV;
/*      */         }
/*      */       } 
/*      */       
/*  427 */       int factorV = 8 * maxSubV;
/*  428 */       n = (int)(n / factorV + 0.5F) * factorV;
/*      */       
/*  430 */       if (n < factorV) {
/*  431 */         n = factorV;
/*      */       }
/*      */       
/*  434 */       if (isTiled) {
/*  435 */         int factorH = 8 * i1;
/*  436 */         m = (int)(m / factorH + 0.5F) * factorH;
/*      */         
/*  438 */         if (m < factorH) {
/*  439 */           m = factorH;
/*      */         }
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/*  445 */     if (isTiled) {
/*      */       
/*  447 */       i1 = (width + m - 1) / m * (height + n - 1) / n;
/*      */     }
/*      */     else {
/*      */       
/*  451 */       i1 = (int)Math.ceil(height / n);
/*      */     } 
/*      */     
/*  454 */     long[] tileByteCounts = new long[i1];
/*      */     
/*  456 */     long bytesPerRow = (long)Math.ceil(sampleSize[0] / 8.0D * m * numBands);
/*      */ 
/*      */     
/*  459 */     long bytesPerTile = bytesPerRow * n;
/*      */     
/*  461 */     for (int i = 0; i < i1; i++) {
/*  462 */       tileByteCounts[i] = bytesPerTile;
/*      */     }
/*      */     
/*  465 */     if (!isTiled) {
/*      */       
/*  467 */       long lastStripRows = (height - n * (i1 - 1));
/*  468 */       tileByteCounts[i1 - 1] = lastStripRows * bytesPerRow;
/*      */     } 
/*      */     
/*  471 */     long totalBytesOfData = bytesPerTile * (i1 - 1) + tileByteCounts[i1 - 1];
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  476 */     long[] tileOffsets = new long[i1];
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  492 */     SortedSet fields = new TreeSet();
/*      */ 
/*      */     
/*  495 */     fields.add(new TIFFField(256, 4, 1, new long[] { width }));
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  500 */     fields.add(new TIFFField(257, 4, 1, new long[] { height }));
/*      */ 
/*      */ 
/*      */     
/*  504 */     char[] shortSampleSize = new char[numBands];
/*  505 */     for (int i = 0; i < numBands; i++)
/*  506 */       shortSampleSize[i] = (char)sampleSize[i]; 
/*  507 */     fields.add(new TIFFField(258, 3, numBands, shortSampleSize));
/*      */ 
/*      */ 
/*      */     
/*  511 */     fields.add(new TIFFField(259, 3, 1, new char[] { (char)compression }));
/*      */ 
/*      */ 
/*      */     
/*  515 */     fields.add(new TIFFField(262, 3, 1, new char[] { (char)photometricInterpretation }));
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  520 */     if (!isTiled) {
/*  521 */       fields.add(new TIFFField(273, 4, i1, tileOffsets));
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  526 */     fields.add(new TIFFField(277, 3, 1, new char[] { (char)numBands }));
/*      */ 
/*      */ 
/*      */     
/*  530 */     if (!isTiled) {
/*  531 */       fields.add(new TIFFField(278, 4, 1, new long[] { n }));
/*      */ 
/*      */ 
/*      */       
/*  535 */       fields.add(new TIFFField(279, 4, i1, tileByteCounts));
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  540 */     if (colormap != null) {
/*  541 */       fields.add(new TIFFField(320, 3, sizeOfColormap, colormap));
/*      */     }
/*      */ 
/*      */ 
/*      */     
/*  546 */     if (isTiled) {
/*  547 */       fields.add(new TIFFField(322, 4, 1, new long[] { m }));
/*      */ 
/*      */ 
/*      */       
/*  551 */       fields.add(new TIFFField(323, 4, 1, new long[] { n }));
/*      */ 
/*      */ 
/*      */       
/*  555 */       fields.add(new TIFFField(324, 4, i1, tileOffsets));
/*      */ 
/*      */ 
/*      */       
/*  559 */       fields.add(new TIFFField(325, 4, i1, tileByteCounts));
/*      */     } 
/*      */ 
/*      */ 
/*      */     
/*  564 */     if (numExtraSamples > 0) {
/*  565 */       char[] extraSamples = new char[numExtraSamples];
/*  566 */       for (int i = 0; i < numExtraSamples; i++) {
/*  567 */         extraSamples[i] = (char)extraSampleType;
/*      */       }
/*  569 */       fields.add(new TIFFField(338, 3, numExtraSamples, extraSamples));
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  575 */     if (dataType != 0) {
/*      */       
/*  577 */       char[] sampleFormat = new char[numBands];
/*  578 */       if (dataType == 4) {
/*  579 */         sampleFormat[0] = '\003';
/*  580 */       } else if (dataType == 1) {
/*  581 */         sampleFormat[0] = '\001';
/*      */       } else {
/*  583 */         sampleFormat[0] = '\002';
/*      */       } 
/*  585 */       for (int b = 1; b < numBands; b++) {
/*  586 */         sampleFormat[b] = sampleFormat[0];
/*      */       }
/*  588 */       fields.add(new TIFFField(339, 3, numBands, sampleFormat));
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  600 */     JPEGEncodeParam jpegEncodeParam = null;
/*  601 */     JPEGImageEncoder jpegEncoder = null;
/*  602 */     int jpegColorID = 0;
/*      */     
/*  604 */     if (compression == 7) {
/*      */ 
/*      */       
/*  607 */       jpegColorID = 0;
/*      */       
/*  609 */       switch (imageType) {
/*      */         case 2:
/*      */         case 3:
/*  612 */           jpegColorID = 1;
/*      */           break;
/*      */         
/*      */         case 4:
/*  616 */           jpegColorID = 2;
/*      */           break;
/*      */         
/*      */         case 6:
/*  620 */           jpegColorID = 3;
/*      */           break;
/*      */       } 
/*      */ 
/*      */ 
/*      */       
/*  626 */       Raster tile00 = im.getTile(0, 0);
/*  627 */       jpegEncodeParam = JPEGCodec.getDefaultJPEGEncodeParam(tile00, jpegColorID);
/*      */ 
/*      */ 
/*      */       
/*  631 */       modifyEncodeParam(jep, jpegEncodeParam, numBands);
/*      */ 
/*      */       
/*  634 */       jpegEncodeParam.setImageInfoValid(false);
/*  635 */       jpegEncodeParam.setTableInfoValid(true);
/*  636 */       ByteArrayOutputStream tableStream = new ByteArrayOutputStream();
/*      */       
/*  638 */       jpegEncoder = JPEGCodec.createJPEGEncoder(tableStream, jpegEncodeParam);
/*      */ 
/*      */ 
/*      */       
/*  642 */       jpegEncoder.encode(tile00);
/*  643 */       byte[] tableData = tableStream.toByteArray();
/*  644 */       fields.add(new TIFFField(347, 7, tableData.length, tableData));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  650 */       jpegEncoder = null;
/*      */     } 
/*      */     
/*  653 */     if (imageType == 6) {
/*      */       long[][] arrayOfLong;
/*      */       
/*  656 */       char subsampleH = '\001';
/*  657 */       char subsampleV = '\001';
/*      */ 
/*      */       
/*  660 */       if (compression == 7) {
/*      */         
/*  662 */         subsampleH = (char)jep.getHorizontalSubsampling(0);
/*  663 */         subsampleV = (char)jep.getVerticalSubsampling(0);
/*  664 */         for (int i = 1; i < numBands; i++) {
/*  665 */           char subH = (char)jep.getHorizontalSubsampling(i);
/*  666 */           if (subH > subsampleH) {
/*  667 */             subsampleH = subH;
/*      */           }
/*  669 */           char subV = (char)jep.getVerticalSubsampling(i);
/*  670 */           if (subV > subsampleV) {
/*  671 */             subsampleV = subV;
/*      */           }
/*      */         } 
/*      */       } 
/*      */       
/*  676 */       fields.add(new TIFFField(530, 3, 2, new char[] { subsampleH, subsampleV }));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  682 */       fields.add(new TIFFField(531, 3, 1, new char[] { (compression == 7) ? '\001' : '\002' }));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  689 */       if (compression == 7) {
/*  690 */         arrayOfLong = new long[][] { { 0L, 1L }, { 255L, 1L }, { 128L, 1L }, { 255L, 1L }, { 128L, 1L }, { 255L, 1L } };
/*      */       
/*      */       }
/*      */       else {
/*      */         
/*  695 */         arrayOfLong = new long[][] { { 15L, 1L }, { 235L, 1L }, { 128L, 1L }, { 240L, 1L }, { 128L, 1L }, { 240L, 1L } };
/*      */       } 
/*      */ 
/*      */ 
/*      */       
/*  700 */       fields.add(new TIFFField(532, 5, 6, arrayOfLong));
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  709 */     TIFFField[] extraFields = encodeParam.getExtraFields();
/*  710 */     if (extraFields != null) {
/*  711 */       List extantTags = new ArrayList(fields.size());
/*  712 */       Iterator fieldIter = fields.iterator();
/*  713 */       while (fieldIter.hasNext()) {
/*  714 */         TIFFField fld = (TIFFField)fieldIter.next();
/*  715 */         extantTags.add(new Integer(fld.getTag()));
/*      */       } 
/*      */       
/*  718 */       int numExtraFields = extraFields.length;
/*  719 */       for (int i = 0; i < numExtraFields; i++) {
/*  720 */         TIFFField fld = extraFields[i];
/*  721 */         Integer tagValue = new Integer(fld.getTag());
/*  722 */         if (!extantTags.contains(tagValue)) {
/*  723 */           fields.add(fld);
/*  724 */           extantTags.add(tagValue);
/*      */         } 
/*      */       } 
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  734 */     int dirSize = getDirectorySize(fields);
/*      */ 
/*      */ 
/*      */     
/*  738 */     tileOffsets[0] = (ifdOffset + dirSize);
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  749 */     OutputStream outCache = null;
/*  750 */     byte[] compressBuf = null;
/*  751 */     File tempFile = null;
/*      */     
/*  753 */     int nextIFDOffset = 0;
/*  754 */     boolean skipByte = false;
/*      */     
/*  756 */     Deflater deflater = null;
/*  757 */     boolean jpegRGBToYCbCr = false;
/*      */     
/*  759 */     if (compression == 1) {
/*      */ 
/*      */ 
/*      */ 
/*      */       
/*  764 */       int numBytesPadding = 0;
/*  765 */       if (sampleSize[0] == 16 && tileOffsets[0] % 2L != 0L) {
/*  766 */         numBytesPadding = 1;
/*  767 */         tileOffsets[0] = tileOffsets[0] + 1L;
/*  768 */       } else if (sampleSize[0] == 32 && tileOffsets[0] % 4L != 0L) {
/*  769 */         numBytesPadding = (int)(4L - tileOffsets[0] % 4L);
/*  770 */         tileOffsets[0] = tileOffsets[0] + numBytesPadding;
/*      */       } 
/*      */ 
/*      */       
/*  774 */       for (int i = 1; i < i1; i++) {
/*  775 */         tileOffsets[i] = tileOffsets[i - 1] + tileByteCounts[i - 1];
/*      */       }
/*      */       
/*  778 */       if (!isLast) {
/*      */         
/*  780 */         nextIFDOffset = (int)(tileOffsets[0] + totalBytesOfData);
/*      */ 
/*      */         
/*  783 */         if ((nextIFDOffset & true) != 0) {
/*  784 */           nextIFDOffset++;
/*  785 */           skipByte = true;
/*      */         } 
/*      */       } 
/*      */ 
/*      */       
/*  790 */       writeDirectory(ifdOffset, fields, nextIFDOffset);
/*      */ 
/*      */ 
/*      */       
/*  794 */       if (numBytesPadding != 0) {
/*  795 */         for (int padding = 0; padding < numBytesPadding; padding++) {
/*  796 */           this.output.write(0);
/*      */         
/*      */         }
/*      */       }
/*      */     }
/*      */     else {
/*      */       
/*  803 */       if (this.output instanceof SeekableOutputStream) {
/*      */         
/*  805 */         ((SeekableOutputStream)this.output).seek(tileOffsets[0]);
/*      */       } else {
/*      */         
/*  808 */         outCache = this.output;
/*      */ 
/*      */         
/*      */         try {
/*  812 */           tempFile = File.createTempFile("jai-SOS-", ".tmp");
/*  813 */           tempFile.deleteOnExit();
/*  814 */           RandomAccessFile raFile = new RandomAccessFile(tempFile, "rw");
/*      */           
/*  816 */           this.output = new SeekableOutputStream(raFile);
/*      */         
/*      */         }
/*      */         catch (Exception e) {
/*      */           
/*  821 */           this.output = new ByteArrayOutputStream((int)totalBytesOfData);
/*      */         } 
/*      */       } 
/*      */       
/*  825 */       int bufSize = 0;
/*  826 */       switch (compression) {
/*      */         case 32773:
/*  828 */           bufSize = (int)(bytesPerTile + (bytesPerRow + 127L) / 128L * n);
/*      */           break;
/*      */         
/*      */         case 7:
/*  832 */           bufSize = 0;
/*      */ 
/*      */           
/*  835 */           if (imageType == 6 && colorModel != null && colorModel.getColorSpace().getType() == 5)
/*      */           {
/*      */ 
/*      */             
/*  839 */             jpegRGBToYCbCr = true;
/*      */           }
/*      */           break;
/*      */         case 32946:
/*  843 */           bufSize = (int)bytesPerTile;
/*  844 */           deflater = new Deflater(encodeParam.getDeflateLevel());
/*      */           break;
/*      */         default:
/*  847 */           bufSize = 0; break;
/*      */       } 
/*  849 */       if (bufSize != 0) {
/*  850 */         compressBuf = new byte[bufSize];
/*      */       }
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  857 */     int[] pixels = null;
/*  858 */     float[] fpixels = null;
/*      */ 
/*      */     
/*  861 */     boolean checkContiguous = ((sampleSize[0] == 1 && sampleModel instanceof MultiPixelPackedSampleModel && dataType == 0) || (sampleSize[0] == 8 && sampleModel instanceof ComponentSampleModel));
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/*  870 */     byte[] bpixels = null;
/*  871 */     if (compression != 7) {
/*  872 */       if (dataType == 0) {
/*  873 */         bpixels = new byte[n * m * numBands];
/*  874 */       } else if (dataTypeIsShort) {
/*  875 */         bpixels = new byte[2 * n * m * numBands];
/*  876 */       } else if (dataType == 3 || dataType == 4) {
/*      */         
/*  878 */         bpixels = new byte[4 * n * m * numBands];
/*      */       } 
/*      */     }
/*      */ 
/*      */     
/*  883 */     int lastRow = minY + height;
/*  884 */     int lastCol = minX + width;
/*  885 */     int tileNum = 0;
/*  886 */     for (int row = minY; row < lastRow; row += n) {
/*  887 */       int rows = isTiled ? n : Math.min(n, lastRow - row);
/*      */       
/*  889 */       int size = rows * m * numBands;
/*      */       
/*  891 */       for (int col = minX; col < lastCol; col += m) {
/*      */         int i, ls, i; byte b1;
/*  893 */         Raster src = im.getData(new Rectangle(col, row, m, rows));
/*      */ 
/*      */         
/*  896 */         boolean useDataBuffer = false;
/*  897 */         if (compression != 7) {
/*  898 */           if (checkContiguous) {
/*  899 */             if (sampleSize[0] == 8) {
/*  900 */               ComponentSampleModel csm = (ComponentSampleModel)src.getSampleModel();
/*      */               
/*  902 */               int[] bankIndices = csm.getBankIndices();
/*  903 */               int[] bandOffsets = csm.getBandOffsets();
/*  904 */               int pixelStride = csm.getPixelStride();
/*  905 */               int lineStride = csm.getScanlineStride();
/*      */               
/*  907 */               if (pixelStride != numBands || lineStride != bytesPerRow) {
/*      */                 
/*  909 */                 useDataBuffer = false;
/*      */               } else {
/*  911 */                 useDataBuffer = true;
/*  912 */                 int i = 0;
/*  913 */                 for (; useDataBuffer && i < numBands; 
/*  914 */                   i++) {
/*  915 */                   if (bankIndices[i] != 0 || bandOffsets[i] != i)
/*      */                   {
/*  917 */                     useDataBuffer = false;
/*      */                   }
/*      */                 } 
/*      */               } 
/*      */             } else {
/*  922 */               MultiPixelPackedSampleModel mpp = (MultiPixelPackedSampleModel)src.getSampleModel();
/*      */               
/*  924 */               if (mpp.getNumBands() == 1 && mpp.getDataBitOffset() == 0 && mpp.getPixelBitStride() == 1)
/*      */               {
/*      */                 
/*  927 */                 useDataBuffer = true;
/*      */               }
/*      */             } 
/*      */           }
/*      */           
/*  932 */           if (!useDataBuffer) {
/*  933 */             if (dataType == 4) {
/*  934 */               fpixels = src.getPixels(col, row, m, rows, fpixels);
/*      */             } else {
/*      */               
/*  937 */               pixels = src.getPixels(col, row, m, rows, pixels);
/*      */             } 
/*      */           }
/*      */         } 
/*      */ 
/*      */ 
/*      */ 
/*      */         
/*  945 */         int pixel = 0;
/*  946 */         int k = 0;
/*  947 */         switch (sampleSize[0]) {
/*      */ 
/*      */           
/*      */           case 1:
/*  951 */             if (useDataBuffer) {
/*  952 */               byte[] btmp = ((DataBufferByte)src.getDataBuffer()).getData();
/*      */               
/*  954 */               MultiPixelPackedSampleModel mpp = (MultiPixelPackedSampleModel)src.getSampleModel();
/*      */               
/*  956 */               int lineStride = mpp.getScanlineStride();
/*  957 */               int inOffset = mpp.getOffset(col - src.getSampleModelTranslateX(), row - src.getSampleModelTranslateY());
/*      */ 
/*      */ 
/*      */ 
/*      */               
/*  962 */               if (lineStride == (int)bytesPerRow) {
/*  963 */                 System.arraycopy(btmp, inOffset, bpixels, 0, (int)bytesPerRow * rows);
/*      */               }
/*      */               else {
/*      */                 
/*  967 */                 int outOffset = 0;
/*  968 */                 for (int j = 0; j < rows; j++) {
/*  969 */                   System.arraycopy(btmp, inOffset, bpixels, outOffset, (int)bytesPerRow);
/*      */ 
/*      */                   
/*  972 */                   inOffset += lineStride;
/*  973 */                   outOffset += (int)bytesPerRow;
/*      */                 } 
/*      */               } 
/*      */             } else {
/*  977 */               int index = 0;
/*      */ 
/*      */               
/*  980 */               for (int i = 0; i < rows; i++) {
/*      */ 
/*      */                 
/*  983 */                 for (int j = 0; j < m / 8; j++) {
/*      */                   
/*  985 */                   pixel = pixels[index++] << 7 | pixels[index++] << 6 | pixels[index++] << 5 | pixels[index++] << 4 | pixels[index++] << 3 | pixels[index++] << 2 | pixels[index++] << 1 | pixels[index++];
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */                   
/*  994 */                   bpixels[k++] = (byte)pixel;
/*      */                 } 
/*      */ 
/*      */                 
/*  998 */                 if (m % 8 > 0) {
/*  999 */                   pixel = 0;
/* 1000 */                   for (int j = 0; j < m % 8; j++) {
/* 1001 */                     pixel |= pixels[index++] << 7 - j;
/*      */                   }
/* 1003 */                   bpixels[k++] = (byte)pixel;
/*      */                 } 
/*      */               } 
/*      */             } 
/*      */             
/* 1008 */             if (compression == 1) {
/* 1009 */               this.output.write(bpixels, 0, rows * (m + 7) / 8); break;
/* 1010 */             }  if (compression == 32773) {
/* 1011 */               int numCompressedBytes = compressPackBits(bpixels, rows, (int)bytesPerRow, compressBuf);
/*      */ 
/*      */ 
/*      */               
/* 1015 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1016 */               this.output.write(compressBuf, 0, numCompressedBytes); break;
/* 1017 */             }  if (compression == 32946) {
/* 1018 */               int numCompressedBytes = deflate(deflater, bpixels, compressBuf);
/*      */               
/* 1020 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1021 */               this.output.write(compressBuf, 0, numCompressedBytes);
/*      */             } 
/*      */             break;
/*      */ 
/*      */ 
/*      */           
/*      */           case 4:
/* 1028 */             b1 = 0;
/*      */ 
/*      */             
/* 1031 */             for (i = 0; i < rows; i++) {
/*      */ 
/*      */ 
/*      */               
/* 1035 */               for (int j = 0; j < m / 2; j++) {
/* 1036 */                 pixel = pixels[b1++] << 4 | pixels[b1++];
/* 1037 */                 bpixels[k++] = (byte)pixel;
/*      */               } 
/*      */ 
/*      */               
/* 1041 */               if ((m & true) == 1) {
/* 1042 */                 pixel = pixels[b1++] << 4;
/* 1043 */                 bpixels[k++] = (byte)pixel;
/*      */               } 
/*      */             } 
/*      */             
/* 1047 */             if (compression == 1) {
/* 1048 */               this.output.write(bpixels, 0, rows * (m + 1) / 2); break;
/* 1049 */             }  if (compression == 32773) {
/* 1050 */               int numCompressedBytes = compressPackBits(bpixels, rows, (int)bytesPerRow, compressBuf);
/*      */ 
/*      */ 
/*      */               
/* 1054 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1055 */               this.output.write(compressBuf, 0, numCompressedBytes); break;
/* 1056 */             }  if (compression == 32946) {
/* 1057 */               int numCompressedBytes = deflate(deflater, bpixels, compressBuf);
/*      */               
/* 1059 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1060 */               this.output.write(compressBuf, 0, numCompressedBytes);
/*      */             } 
/*      */             break;
/*      */ 
/*      */           
/*      */           case 8:
/* 1066 */             if (compression != 7) {
/* 1067 */               if (useDataBuffer) {
/* 1068 */                 byte[] btmp = ((DataBufferByte)src.getDataBuffer()).getData();
/*      */                 
/* 1070 */                 ComponentSampleModel csm = (ComponentSampleModel)src.getSampleModel();
/*      */                 
/* 1072 */                 int inOffset = csm.getOffset(col - src.getSampleModelTranslateX(), row - src.getSampleModelTranslateY());
/*      */ 
/*      */ 
/*      */ 
/*      */                 
/* 1077 */                 int lineStride = csm.getScanlineStride();
/* 1078 */                 if (lineStride == (int)bytesPerRow) {
/* 1079 */                   System.arraycopy(btmp, inOffset, bpixels, 0, (int)bytesPerRow * rows);
/*      */                 
/*      */                 }
/*      */                 else {
/*      */                   
/* 1084 */                   int outOffset = 0;
/* 1085 */                   for (int j = 0; j < rows; j++) {
/* 1086 */                     System.arraycopy(btmp, inOffset, bpixels, outOffset, (int)bytesPerRow);
/*      */ 
/*      */                     
/* 1089 */                     inOffset += lineStride;
/* 1090 */                     outOffset += (int)bytesPerRow;
/*      */                   } 
/*      */                 } 
/*      */               } else {
/* 1094 */                 for (int i = 0; i < size; i++) {
/* 1095 */                   bpixels[i] = (byte)pixels[i];
/*      */                 }
/*      */               } 
/*      */             }
/*      */             
/* 1100 */             if (compression == 1) {
/* 1101 */               this.output.write(bpixels, 0, size); break;
/* 1102 */             }  if (compression == 32773) {
/* 1103 */               int numCompressedBytes = compressPackBits(bpixels, rows, (int)bytesPerRow, compressBuf);
/*      */ 
/*      */ 
/*      */               
/* 1107 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1108 */               this.output.write(compressBuf, 0, numCompressedBytes); break;
/* 1109 */             }  if (compression == 7) {
/* 1110 */               long startPos = getOffset(this.output);
/*      */ 
/*      */ 
/*      */ 
/*      */               
/* 1115 */               if (jpegEncoder == null || jpegEncodeParam.getWidth() != src.getWidth() || jpegEncodeParam.getHeight() != src.getHeight()) {
/*      */ 
/*      */ 
/*      */                 
/* 1119 */                 jpegEncodeParam = JPEGCodec.getDefaultJPEGEncodeParam(src, jpegColorID);
/*      */ 
/*      */ 
/*      */                 
/* 1123 */                 modifyEncodeParam(jep, jpegEncodeParam, numBands);
/*      */ 
/*      */                 
/* 1126 */                 jpegEncoder = JPEGCodec.createJPEGEncoder(this.output, jpegEncodeParam);
/*      */               } 
/*      */ 
/*      */ 
/*      */               
/* 1131 */               if (jpegRGBToYCbCr) {
/* 1132 */                 WritableRaster wRas = null;
/* 1133 */                 if (src instanceof WritableRaster) {
/* 1134 */                   wRas = (WritableRaster)src;
/*      */                 } else {
/* 1136 */                   wRas = src.createCompatibleWritableRaster();
/* 1137 */                   wRas.setRect(src);
/*      */                 } 
/*      */                 
/* 1140 */                 if (wRas.getMinX() != 0 || wRas.getMinY() != 0) {
/* 1141 */                   wRas = wRas.createWritableTranslatedChild(0, 0);
/*      */                 }
/*      */                 
/* 1144 */                 BufferedImage bi = new BufferedImage(colorModel, wRas, false, null);
/*      */ 
/*      */                 
/* 1147 */                 jpegEncoder.encode(bi);
/*      */               } else {
/* 1149 */                 jpegEncoder.encode(src.createTranslatedChild(0, 0));
/*      */               } 
/*      */ 
/*      */               
/* 1153 */               long endPos = getOffset(this.output);
/* 1154 */               tileByteCounts[tileNum++] = (int)(endPos - startPos); break;
/* 1155 */             }  if (compression == 32946) {
/* 1156 */               int numCompressedBytes = deflate(deflater, bpixels, compressBuf);
/*      */               
/* 1158 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1159 */               this.output.write(compressBuf, 0, numCompressedBytes);
/*      */             } 
/*      */             break;
/*      */ 
/*      */           
/*      */           case 16:
/* 1165 */             ls = 0;
/* 1166 */             for (i = 0; i < size; i++) {
/* 1167 */               int value = pixels[i];
/* 1168 */               bpixels[ls++] = (byte)((value & 0xFF00) >> 8);
/* 1169 */               bpixels[ls++] = (byte)(value & 0xFF);
/*      */             } 
/*      */             
/* 1172 */             if (compression == 1) {
/* 1173 */               this.output.write(bpixels, 0, size * 2); break;
/* 1174 */             }  if (compression == 32773) {
/* 1175 */               int numCompressedBytes = compressPackBits(bpixels, rows, (int)bytesPerRow, compressBuf);
/*      */ 
/*      */ 
/*      */               
/* 1179 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1180 */               this.output.write(compressBuf, 0, numCompressedBytes); break;
/* 1181 */             }  if (compression == 32946) {
/* 1182 */               int numCompressedBytes = deflate(deflater, bpixels, compressBuf);
/*      */               
/* 1184 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1185 */               this.output.write(compressBuf, 0, numCompressedBytes);
/*      */             } 
/*      */             break;
/*      */           
/*      */           case 32:
/* 1190 */             if (dataType == 3) {
/* 1191 */               int li = 0;
/* 1192 */               for (int i = 0; i < size; i++) {
/* 1193 */                 int value = pixels[i];
/* 1194 */                 bpixels[li++] = (byte)((value & 0xFF000000) >>> 24);
/* 1195 */                 bpixels[li++] = (byte)((value & 0xFF0000) >>> 16);
/* 1196 */                 bpixels[li++] = (byte)((value & 0xFF00) >>> 8);
/* 1197 */                 bpixels[li++] = (byte)(value & 0xFF);
/*      */               } 
/*      */             } else {
/* 1200 */               int lf = 0;
/* 1201 */               for (int i = 0; i < size; i++) {
/* 1202 */                 int value = Float.floatToIntBits(fpixels[i]);
/* 1203 */                 bpixels[lf++] = (byte)((value & 0xFF000000) >>> 24);
/* 1204 */                 bpixels[lf++] = (byte)((value & 0xFF0000) >>> 16);
/* 1205 */                 bpixels[lf++] = (byte)((value & 0xFF00) >>> 8);
/* 1206 */                 bpixels[lf++] = (byte)(value & 0xFF);
/*      */               } 
/*      */             } 
/* 1209 */             if (compression == 1) {
/* 1210 */               this.output.write(bpixels, 0, size * 4); break;
/* 1211 */             }  if (compression == 32773) {
/* 1212 */               int numCompressedBytes = compressPackBits(bpixels, rows, (int)bytesPerRow, compressBuf);
/*      */ 
/*      */ 
/*      */               
/* 1216 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1217 */               this.output.write(compressBuf, 0, numCompressedBytes); break;
/* 1218 */             }  if (compression == 32946) {
/* 1219 */               int numCompressedBytes = deflate(deflater, bpixels, compressBuf);
/*      */               
/* 1221 */               tileByteCounts[tileNum++] = numCompressedBytes;
/* 1222 */               this.output.write(compressBuf, 0, numCompressedBytes);
/*      */             } 
/*      */             break;
/*      */         } 
/*      */ 
/*      */       
/*      */       } 
/*      */     } 
/* 1230 */     if (compression == 1) {
/*      */       
/* 1232 */       if (skipByte) {
/* 1233 */         this.output.write(0);
/*      */       }
/*      */     } else {
/*      */       
/* 1237 */       int totalBytes = 0;
/* 1238 */       for (int i = 1; i < i1; i++) {
/* 1239 */         int numBytes = (int)tileByteCounts[i - 1];
/* 1240 */         totalBytes += numBytes;
/* 1241 */         tileOffsets[i] = tileOffsets[i - 1] + numBytes;
/*      */       } 
/* 1243 */       totalBytes += (int)tileByteCounts[i1 - 1];
/*      */       
/* 1245 */       nextIFDOffset = isLast ? 0 : (ifdOffset + dirSize + totalBytes);
/*      */       
/* 1247 */       if ((nextIFDOffset & true) != 0) {
/* 1248 */         nextIFDOffset++;
/* 1249 */         skipByte = true;
/*      */       } 
/*      */       
/* 1252 */       if (outCache == null) {
/*      */ 
/*      */ 
/*      */         
/* 1256 */         if (skipByte) {
/* 1257 */           this.output.write(0);
/*      */         }
/*      */         
/* 1260 */         SeekableOutputStream sos = (SeekableOutputStream)this.output;
/*      */ 
/*      */         
/* 1263 */         long savePos = sos.getFilePointer();
/*      */ 
/*      */         
/* 1266 */         sos.seek(ifdOffset);
/* 1267 */         writeDirectory(ifdOffset, fields, nextIFDOffset);
/*      */ 
/*      */         
/* 1270 */         sos.seek(savePos);
/* 1271 */       } else if (tempFile != null) {
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1276 */         FileInputStream fileStream = new FileInputStream(tempFile);
/*      */ 
/*      */         
/* 1279 */         this.output.close();
/*      */ 
/*      */         
/* 1282 */         this.output = outCache;
/*      */ 
/*      */         
/* 1285 */         writeDirectory(ifdOffset, fields, nextIFDOffset);
/*      */ 
/*      */         
/* 1288 */         byte[] copyBuffer = new byte[8192];
/* 1289 */         int bytesCopied = 0;
/* 1290 */         while (bytesCopied < totalBytes) {
/* 1291 */           int bytesRead = fileStream.read(copyBuffer);
/* 1292 */           if (bytesRead == -1) {
/*      */             break;
/*      */           }
/* 1295 */           this.output.write(copyBuffer, 0, bytesRead);
/* 1296 */           bytesCopied += bytesRead;
/*      */         } 
/*      */ 
/*      */         
/* 1300 */         fileStream.close();
/* 1301 */         tempFile.delete();
/*      */ 
/*      */         
/* 1304 */         if (skipByte) {
/* 1305 */           this.output.write(0);
/*      */         }
/* 1307 */       } else if (this.output instanceof ByteArrayOutputStream) {
/*      */ 
/*      */ 
/*      */         
/* 1311 */         ByteArrayOutputStream memoryStream = (ByteArrayOutputStream)this.output;
/*      */ 
/*      */ 
/*      */         
/* 1315 */         this.output = outCache;
/*      */ 
/*      */         
/* 1318 */         writeDirectory(ifdOffset, fields, nextIFDOffset);
/*      */ 
/*      */         
/* 1321 */         memoryStream.writeTo(this.output);
/*      */ 
/*      */         
/* 1324 */         if (skipByte) {
/* 1325 */           this.output.write(0);
/*      */         }
/*      */       } else {
/*      */         
/* 1329 */         throw new IllegalStateException();
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/* 1334 */     return nextIFDOffset;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private int getDirectorySize(SortedSet fields) {
/* 1342 */     int numEntries = fields.size();
/*      */ 
/*      */     
/* 1345 */     int dirSize = 2 + numEntries * 12 + 4;
/*      */ 
/*      */     
/* 1348 */     Iterator iter = fields.iterator();
/* 1349 */     while (iter.hasNext()) {
/*      */       
/* 1351 */       TIFFField field = (TIFFField)iter.next();
/*      */ 
/*      */       
/* 1354 */       int valueSize = field.getCount() * sizeOfType[field.getType()];
/*      */ 
/*      */       
/* 1357 */       if (valueSize > 4) {
/* 1358 */         dirSize += valueSize;
/*      */       }
/*      */     } 
/*      */     
/* 1362 */     return dirSize;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void writeFileHeader() throws IOException {
/* 1369 */     this.output.write(77);
/* 1370 */     this.output.write(77);
/*      */ 
/*      */     
/* 1373 */     this.output.write(0);
/* 1374 */     this.output.write(42);
/*      */ 
/*      */     
/* 1377 */     writeLong(8L);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void writeDirectory(int thisIFDOffset, SortedSet fields, int nextIFDOffset) throws IOException {
/* 1385 */     int numEntries = fields.size();
/*      */     
/* 1387 */     long offsetBeyondIFD = (thisIFDOffset + 12 * numEntries + 4 + 2);
/* 1388 */     List tooBig = new ArrayList();
/*      */ 
/*      */     
/* 1391 */     writeUnsignedShort(numEntries);
/*      */     
/* 1393 */     Iterator iter = fields.iterator();
/* 1394 */     while (iter.hasNext()) {
/*      */ 
/*      */       
/* 1397 */       TIFFField field = (TIFFField)iter.next();
/*      */ 
/*      */       
/* 1400 */       int tag = field.getTag();
/* 1401 */       writeUnsignedShort(tag);
/*      */ 
/*      */       
/* 1404 */       int type = field.getType();
/* 1405 */       writeUnsignedShort(type);
/*      */ 
/*      */ 
/*      */       
/* 1409 */       int count = field.getCount();
/* 1410 */       int valueSize = getValueSize(field);
/* 1411 */       writeLong((type == 2) ? valueSize : count);
/*      */ 
/*      */       
/* 1414 */       if (valueSize > 4) {
/*      */ 
/*      */         
/* 1417 */         writeLong(offsetBeyondIFD);
/* 1418 */         offsetBeyondIFD += valueSize;
/* 1419 */         tooBig.add(field);
/*      */         continue;
/*      */       } 
/* 1422 */       writeValuesAsFourBytes(field);
/*      */     } 
/*      */ 
/*      */ 
/*      */ 
/*      */     
/* 1428 */     writeLong(nextIFDOffset);
/*      */ 
/*      */     
/* 1431 */     for (int i = 0; i < tooBig.size(); i++) {
/* 1432 */       writeValues((TIFFField)tooBig.get(i));
/*      */     }
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private static int getValueSize(TIFFField field) {
/* 1440 */     int type = field.getType();
/* 1441 */     int count = field.getCount();
/* 1442 */     int valueSize = 0;
/* 1443 */     if (type == 2) {
/* 1444 */       for (int i = 0; i < count; i++) {
/* 1445 */         byte[] stringBytes = field.getAsString(i).getBytes();
/* 1446 */         valueSize += stringBytes.length;
/* 1447 */         if (stringBytes[stringBytes.length - 1] != 0) {
/* 1448 */           valueSize++;
/*      */         }
/*      */       } 
/*      */     } else {
/* 1452 */       valueSize = count * sizeOfType[type];
/*      */     } 
/* 1454 */     return valueSize;
/*      */   }
/*      */   private static final int[] sizeOfType = { 
/* 1457 */       0, 1, 1, 2, 4, 8, 1, 1, 2, 4, 8, 4, 8 };
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void writeValuesAsFourBytes(TIFFField field) throws IOException {
/*      */     int i;
/*      */     long[] longs;
/*      */     int i, i;
/*      */     char[] chars;
/*      */     int i, i;
/*      */     byte[] bytes;
/* 1475 */     int dataType = field.getType();
/* 1476 */     int count = field.getCount();
/*      */     
/* 1478 */     switch (dataType) {
/*      */ 
/*      */       
/*      */       case 1:
/* 1482 */         bytes = field.getAsBytes();
/* 1483 */         if (count > 4) count = 4; 
/* 1484 */         for (i = 0; i < count; i++) {
/* 1485 */           this.output.write(bytes[i]);
/*      */         }
/* 1487 */         for (i = 0; i < 4 - count; i++) {
/* 1488 */           this.output.write(0);
/*      */         }
/*      */         break;
/*      */       
/*      */       case 3:
/* 1493 */         chars = field.getAsChars();
/* 1494 */         if (count > 2) count = 2; 
/* 1495 */         for (i = 0; i < count; i++)
/* 1496 */           writeUnsignedShort(chars[i]); 
/* 1497 */         for (i = 0; i < 2 - count; i++) {
/* 1498 */           writeUnsignedShort(0);
/*      */         }
/*      */         break;
/*      */ 
/*      */       
/*      */       case 4:
/* 1504 */         longs = field.getAsLongs();
/*      */         
/* 1506 */         for (i = 0; i < count; i++)
/* 1507 */           writeLong(longs[i]);  break;
/*      */     }  } private void writeValues(TIFFField field) throws IOException { int i, i; long[][] rationals; int i; double[] doubles; int i; float[] floats; int i;
/*      */     long[] longs;
/*      */     int i;
/*      */     short[] shorts;
/*      */     int i;
/*      */     char[] chars;
/*      */     int i;
/*      */     byte[] bytes;
/* 1516 */     int dataType = field.getType();
/* 1517 */     int count = field.getCount();
/*      */     
/* 1519 */     switch (dataType) {
/*      */ 
/*      */       
/*      */       case 1:
/*      */       case 6:
/*      */       case 7:
/* 1525 */         bytes = field.getAsBytes();
/* 1526 */         for (i = 0; i < count; i++) {
/* 1527 */           this.output.write(bytes[i]);
/*      */         }
/*      */         return;
/*      */ 
/*      */       
/*      */       case 3:
/* 1533 */         chars = field.getAsChars();
/* 1534 */         for (i = 0; i < count; i++) {
/* 1535 */           writeUnsignedShort(chars[i]);
/*      */         }
/*      */         return;
/*      */       case 8:
/* 1539 */         shorts = field.getAsShorts();
/* 1540 */         for (i = 0; i < count; i++) {
/* 1541 */           writeUnsignedShort(shorts[i]);
/*      */         }
/*      */         return;
/*      */ 
/*      */       
/*      */       case 4:
/*      */       case 9:
/* 1548 */         longs = field.getAsLongs();
/* 1549 */         for (i = 0; i < count; i++) {
/* 1550 */           writeLong(longs[i]);
/*      */         }
/*      */         return;
/*      */       
/*      */       case 11:
/* 1555 */         floats = field.getAsFloats();
/* 1556 */         for (i = 0; i < count; i++) {
/* 1557 */           int intBits = Float.floatToIntBits(floats[i]);
/* 1558 */           writeLong(intBits);
/*      */         } 
/*      */         return;
/*      */       
/*      */       case 12:
/* 1563 */         doubles = field.getAsDoubles();
/* 1564 */         for (i = 0; i < count; i++) {
/* 1565 */           long longBits = Double.doubleToLongBits(doubles[i]);
/* 1566 */           writeLong(longBits >>> 32);
/* 1567 */           writeLong(longBits & 0xFFFFFFFFL);
/*      */         } 
/*      */         return;
/*      */       
/*      */       case 5:
/*      */       case 10:
/* 1573 */         rationals = field.getAsRationals();
/* 1574 */         for (i = 0; i < count; i++) {
/* 1575 */           writeLong(rationals[i][0]);
/* 1576 */           writeLong(rationals[i][1]);
/*      */         } 
/*      */         return;
/*      */       
/*      */       case 2:
/* 1581 */         for (i = 0; i < count; i++) {
/* 1582 */           byte[] stringBytes = field.getAsString(i).getBytes();
/* 1583 */           this.output.write(stringBytes);
/* 1584 */           if (stringBytes[stringBytes.length - 1] != 0) {
/* 1585 */             this.output.write(0);
/*      */           }
/*      */         } 
/*      */         return;
/*      */     } 
/*      */     
/* 1591 */     throw new Error("TIFFImageEncoder10"); }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void writeUnsignedShort(int s) throws IOException {
/* 1600 */     this.output.write((s & 0xFF00) >>> 8);
/* 1601 */     this.output.write(s & 0xFF);
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private void writeLong(long l) throws IOException {
/* 1610 */     this.output.write((int)((l & 0xFFFFFFFFFF000000L) >>> 24));
/* 1611 */     this.output.write((int)((l & 0xFF0000L) >>> 16));
/* 1612 */     this.output.write((int)((l & 0xFF00L) >>> 8));
/* 1613 */     this.output.write((int)(l & 0xFFL));
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private long getOffset(OutputStream out) throws IOException {
/* 1621 */     if (out instanceof ByteArrayOutputStream)
/* 1622 */       return ((ByteArrayOutputStream)out).size(); 
/* 1623 */     if (out instanceof SeekableOutputStream) {
/* 1624 */       return ((SeekableOutputStream)out).getFilePointer();
/*      */     }
/*      */     
/* 1627 */     throw new IllegalStateException();
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private static int compressPackBits(byte[] data, int numRows, int bytesPerRow, byte[] compData) {
/* 1636 */     int inOffset = 0;
/* 1637 */     int outOffset = 0;
/*      */     
/* 1639 */     for (int i = 0; i < numRows; i++) {
/* 1640 */       outOffset = packBits(data, inOffset, bytesPerRow, compData, outOffset);
/*      */       
/* 1642 */       inOffset += bytesPerRow;
/*      */     } 
/*      */     
/* 1645 */     return outOffset;
/*      */   }
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   
/*      */   private static int packBits(byte[] input, int inOffset, int inCount, byte[] output, int outOffset) {
/* 1655 */     int inMax = inOffset + inCount - 1;
/* 1656 */     int inMaxMinus1 = inMax - 1;
/*      */     
/* 1658 */     while (inOffset <= inMax) {
/* 1659 */       int run = 1;
/* 1660 */       byte replicate = input[inOffset];
/*      */       
/* 1662 */       while (run < 127 && inOffset < inMax && input[inOffset] == input[inOffset + 1]) {
/* 1663 */         run++;
/* 1664 */         inOffset++;
/*      */       } 
/* 1666 */       if (run > 1) {
/* 1667 */         inOffset++;
/* 1668 */         output[outOffset++] = (byte)-(run - 1);
/* 1669 */         output[outOffset++] = replicate;
/*      */       } 
/*      */       
/* 1672 */       run = 0;
/* 1673 */       int saveOffset = outOffset;
/* 1674 */       while (run < 128 && ((inOffset < inMax && input[inOffset] != input[inOffset + 1]) || (inOffset < inMaxMinus1 && input[inOffset] != input[inOffset + 2]))) {
/*      */ 
/*      */ 
/*      */ 
/*      */         
/* 1679 */         run++;
/* 1680 */         output[++outOffset] = input[inOffset++];
/*      */       } 
/* 1682 */       if (run > 0) {
/* 1683 */         output[saveOffset] = (byte)(run - 1);
/* 1684 */         outOffset++;
/*      */       } 
/*      */       
/* 1687 */       if (inOffset == inMax) {
/* 1688 */         if (run > 0 && run < 128) {
/* 1689 */           output[saveOffset] = (byte)(output[saveOffset] + 1);
/* 1690 */           output[outOffset++] = input[inOffset++]; continue;
/*      */         } 
/* 1692 */         output[outOffset++] = 0;
/* 1693 */         output[outOffset++] = input[inOffset++];
/*      */       } 
/*      */     } 
/*      */ 
/*      */     
/* 1698 */     return outOffset;
/*      */   }
/*      */ 
/*      */   
/*      */   private static int deflate(Deflater deflater, byte[] inflated, byte[] deflated) {
/* 1703 */     deflater.setInput(inflated);
/* 1704 */     deflater.finish();
/* 1705 */     int numCompressedBytes = deflater.deflate(deflated);
/* 1706 */     deflater.reset();
/* 1707 */     return numCompressedBytes;
/*      */   }
/*      */ 
/*      */ 
/*      */   
/*      */   private static void modifyEncodeParam(JPEGEncodeParam src, JPEGEncodeParam dst, int nbands) {
/* 1713 */     dst.setDensityUnit(src.getDensityUnit());
/* 1714 */     dst.setXDensity(src.getXDensity());
/* 1715 */     dst.setYDensity(src.getYDensity());
/* 1716 */     dst.setRestartInterval(src.getRestartInterval());
/* 1717 */     for (int i = 0; i < 4; i++) {
/* 1718 */       JPEGQTable tbl = src.getQTable(i);
/* 1719 */       if (tbl != null)
/* 1720 */         dst.setQTable(i, tbl); 
/*      */     } 
/*      */   }
/*      */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\tiff\TIFFImageEncoder.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */