/*     */ package org.apache.batik.ext.awt.image.codec.util;
/*     */ 
/*     */ import java.awt.Point;
/*     */ import java.awt.Rectangle;
/*     */ import java.awt.image.ColorModel;
/*     */ import java.awt.image.Raster;
/*     */ import java.awt.image.RenderedImage;
/*     */ import java.awt.image.SampleModel;
/*     */ import java.awt.image.WritableRaster;
/*     */ import java.util.ArrayList;
/*     */ import java.util.HashMap;
/*     */ import java.util.List;
/*     */ import java.util.Map;
/*     */ import java.util.Vector;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class SimpleRenderedImage
/*     */   implements RenderedImage
/*     */ {
/*     */   protected int minX;
/*     */   protected int minY;
/*     */   protected int width;
/*     */   protected int height;
/*     */   protected int tileWidth;
/*     */   protected int tileHeight;
/*  68 */   protected int tileGridXOffset = 0;
/*     */ 
/*     */   
/*  71 */   protected int tileGridYOffset = 0;
/*     */ 
/*     */   
/*  74 */   protected SampleModel sampleModel = null;
/*     */ 
/*     */   
/*  77 */   protected ColorModel colorModel = null;
/*     */ 
/*     */   
/*  80 */   protected List sources = new ArrayList();
/*     */ 
/*     */   
/*  83 */   protected Map properties = new HashMap();
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  89 */   public int getMinX() { return this.minX; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*  99 */   public final int getMaxX() { return getMinX() + getWidth(); }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 104 */   public int getMinY() { return this.minY; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 114 */   public final int getMaxY() { return getMinY() + getHeight(); }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 119 */   public int getWidth() { return this.width; }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 124 */   public int getHeight() { return this.height; }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 129 */   public Rectangle getBounds() { return new Rectangle(getMinX(), getMinY(), getWidth(), getHeight()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 135 */   public int getTileWidth() { return this.tileWidth; }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 140 */   public int getTileHeight() { return this.tileHeight; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 147 */   public int getTileGridXOffset() { return this.tileGridXOffset; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 154 */   public int getTileGridYOffset() { return this.tileGridYOffset; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 163 */   public int getMinTileX() { return XToTileX(getMinX()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 172 */   public int getMaxTileX() { return XToTileX(getMaxX() - 1); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 182 */   public int getNumXTiles() { return getMaxTileX() - getMinTileX() + 1; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 191 */   public int getMinTileY() { return YToTileY(getMinY()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 200 */   public int getMaxTileY() { return YToTileY(getMaxY() - 1); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 210 */   public int getNumYTiles() { return getMaxTileY() - getMinTileY() + 1; }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 215 */   public SampleModel getSampleModel() { return this.sampleModel; }
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 220 */   public ColorModel getColorModel() { return this.colorModel; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Object getProperty(String name) {
/* 233 */     name = name.toLowerCase();
/* 234 */     return this.properties.get(name);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public String[] getPropertyNames() {
/* 245 */     String[] names = new String[this.properties.size()];
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 253 */     this.properties.keySet().toArray(names);
/* 254 */     return names;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public String[] getPropertyNames(String prefix) {
/* 271 */     String[] propertyNames = getPropertyNames();
/* 272 */     if (propertyNames == null) {
/* 273 */       return null;
/*     */     }
/*     */     
/* 276 */     prefix = prefix.toLowerCase();
/*     */     
/* 278 */     List names = new ArrayList();
/* 279 */     for (int i = 0; i < propertyNames.length; i++) {
/* 280 */       if (propertyNames[i].startsWith(prefix)) {
/* 281 */         names.add(propertyNames[i]);
/*     */       }
/*     */     } 
/*     */     
/* 285 */     if (names.size() == 0) {
/* 286 */       return null;
/*     */     }
/*     */ 
/*     */     
/* 290 */     String[] prefixNames = new String[names.size()];
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 296 */     names.toArray(prefixNames);
/*     */     
/* 298 */     return prefixNames;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int XToTileX(int x, int tileGridXOffset, int tileWidth) {
/* 309 */     x -= tileGridXOffset;
/* 310 */     if (x < 0) {
/* 311 */       x += 1 - tileWidth;
/*     */     }
/* 313 */     return x / tileWidth;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public static int YToTileY(int y, int tileGridYOffset, int tileHeight) {
/* 322 */     y -= tileGridYOffset;
/* 323 */     if (y < 0) {
/* 324 */       y += 1 - tileHeight;
/*     */     }
/* 326 */     return y / tileHeight;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 338 */   public int XToTileX(int x) { return XToTileX(x, getTileGridXOffset(), getTileWidth()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 350 */   public int YToTileY(int y) { return YToTileY(y, getTileGridYOffset(), getTileHeight()); }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 359 */   public static int tileXToX(int tx, int tileGridXOffset, int tileWidth) { return tx * tileWidth + tileGridXOffset; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 368 */   public static int tileYToY(int ty, int tileGridYOffset, int tileHeight) { return ty * tileHeight + tileGridYOffset; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 380 */   public int tileXToX(int tx) { return tx * this.tileWidth + this.tileGridXOffset; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/* 392 */   public int tileYToY(int ty) { return ty * this.tileHeight + this.tileGridYOffset; }
/*     */ 
/*     */ 
/*     */   
/* 396 */   public Vector getSources() { return null; }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Raster getData() {
/* 416 */     Rectangle rect = new Rectangle(getMinX(), getMinY(), getWidth(), getHeight());
/*     */     
/* 418 */     return getData(rect);
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public Raster getData(Rectangle bounds) {
/* 439 */     int startX = XToTileX(bounds.x);
/* 440 */     int startY = YToTileY(bounds.y);
/* 441 */     int endX = XToTileX(bounds.x + bounds.width - 1);
/* 442 */     int endY = YToTileY(bounds.y + bounds.height - 1);
/*     */ 
/*     */     
/* 445 */     if (startX == endX && startY == endY) {
/* 446 */       Raster tile = getTile(startX, startY);
/* 447 */       return tile.createChild(bounds.x, bounds.y, bounds.width, bounds.height, bounds.x, bounds.y, null);
/*     */     } 
/*     */ 
/*     */ 
/*     */     
/* 452 */     SampleModel sm = this.sampleModel.createCompatibleSampleModel(bounds.width, bounds.height);
/*     */ 
/*     */ 
/*     */ 
/*     */     
/* 457 */     WritableRaster dest = Raster.createWritableRaster(sm, bounds.getLocation());
/*     */ 
/*     */     
/* 460 */     for (int j = startY; j <= endY; j++) {
/* 461 */       for (int i = startX; i <= endX; i++) {
/* 462 */         Raster raster = getTile(i, j);
/* 463 */         Rectangle intersectRect = bounds.intersection(raster.getBounds());
/*     */         
/* 465 */         Raster liveRaster = raster.createChild(intersectRect.x, intersectRect.y, intersectRect.width, intersectRect.height, intersectRect.x, intersectRect.y, null);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 472 */         dest.setDataElements(0, 0, liveRaster);
/*     */       } 
/*     */     } 
/* 475 */     return dest;
/*     */   }
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   
/*     */   public WritableRaster copyData(WritableRaster dest) {
/*     */     Rectangle rectangle;
/* 500 */     if (dest == null) {
/* 501 */       rectangle = getBounds();
/* 502 */       Point p = new Point(this.minX, this.minY);
/*     */       
/* 504 */       SampleModel sm = this.sampleModel.createCompatibleSampleModel(this.width, this.height);
/*     */       
/* 506 */       dest = Raster.createWritableRaster(sm, p);
/*     */     } else {
/* 508 */       rectangle = dest.getBounds();
/*     */     } 
/*     */     
/* 511 */     int startX = XToTileX(rectangle.x);
/* 512 */     int startY = YToTileY(rectangle.y);
/* 513 */     int endX = XToTileX(rectangle.x + rectangle.width - 1);
/* 514 */     int endY = YToTileY(rectangle.y + rectangle.height - 1);
/*     */     
/* 516 */     for (int j = startY; j <= endY; j++) {
/* 517 */       for (int i = startX; i <= endX; i++) {
/* 518 */         Raster tile = getTile(i, j);
/* 519 */         Rectangle intersectRect = rectangle.intersection(tile.getBounds());
/*     */         
/* 521 */         Raster liveRaster = tile.createChild(intersectRect.x, intersectRect.y, intersectRect.width, intersectRect.height, intersectRect.x, intersectRect.y, null);
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */         
/* 536 */         dest.setDataElements(0, 0, liveRaster);
/*     */       } 
/*     */     } 
/* 539 */     return dest;
/*     */   }
/*     */   
/*     */   public abstract Raster getTile(int paramInt1, int paramInt2);
/*     */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\code\\util\SimpleRenderedImage.class
 * Java compiler version: 1 (45.3)
 * JD-Core Version:       1.0.7
 */