/*    */ package org.apache.batik.ext.awt.image.codec.imageio;
/*    */ 
/*    */ import java.awt.image.BufferedImage;
/*    */ import java.awt.image.PixelInterleavedSampleModel;
/*    */ import java.awt.image.SampleModel;
/*    */ import java.awt.image.SinglePixelPackedSampleModel;
/*    */ import java.io.IOException;
/*    */ import java.io.OutputStream;
/*    */ import org.apache.batik.ext.awt.image.GraphicsUtil;
/*    */ import org.apache.batik.ext.awt.image.rendered.FormatRed;
/*    */ import org.apache.batik.ext.awt.image.spi.ImageWriter;
/*    */ import org.apache.batik.ext.awt.image.spi.ImageWriterParams;
/*    */ import org.apache.batik.ext.awt.image.spi.ImageWriterRegistry;
/*    */ import org.apache.batik.transcoder.TranscoderException;
/*    */ import org.apache.batik.transcoder.TranscoderOutput;
/*    */ import org.apache.batik.transcoder.TranscodingHints;
/*    */ import org.apache.batik.transcoder.image.TIFFTranscoder;
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ 
/*    */ public class TIFFTranscoderImageIOWriteAdapter
/*    */   implements TIFFTranscoder.WriteAdapter
/*    */ {
/*    */   public void writeImage(TIFFTranscoder transcoder, BufferedImage img, TranscoderOutput output) throws TranscoderException {
/* 56 */     TranscodingHints hints = transcoder.getTranscodingHints();
/*    */     
/* 58 */     ImageWriter writer = ImageWriterRegistry.getInstance().getWriterFor("image/tiff");
/*    */     
/* 60 */     ImageWriterParams params = new ImageWriterParams();
/*    */     
/* 62 */     float PixSzMM = transcoder.getUserAgent().getPixelUnitToMillimeter();
/* 63 */     int PixSzInch = (int)(25.4D / PixSzMM + 0.5D);
/* 64 */     params.setResolution(PixSzInch);
/*    */     
/* 66 */     if (hints.containsKey(TIFFTranscoder.KEY_COMPRESSION_METHOD)) {
/* 67 */       String method = (String)hints.get(TIFFTranscoder.KEY_COMPRESSION_METHOD);
/*    */       
/* 69 */       if ("packbits".equals(method)) {
/* 70 */         params.setCompressionMethod("PackBits");
/* 71 */       } else if ("deflate".equals(method)) {
/* 72 */         params.setCompressionMethod("Deflate");
/* 73 */       } else if ("lzw".equals(method)) {
/* 74 */         params.setCompressionMethod("LZW");
/* 75 */       } else if ("jpeg".equals(method)) {
/* 76 */         params.setCompressionMethod("JPEG");
/*    */       } 
/*    */     } 
/*    */ 
/*    */ 
/*    */     
/*    */     try {
/* 83 */       OutputStream ostream = output.getOutputStream();
/* 84 */       int w = img.getWidth();
/* 85 */       int h = img.getHeight();
/*    */       
/* 87 */       SinglePixelPackedSampleModel sppsm = (SinglePixelPackedSampleModel)img.getSampleModel();
/* 88 */       int bands = sppsm.getNumBands();
/* 89 */       int[] off = new int[bands];
/* 90 */       for (int i = 0; i < bands; i++)
/* 91 */         off[i] = i; 
/* 92 */       SampleModel sm = new PixelInterleavedSampleModel(0, w, h, bands, w * bands, off);
/*    */ 
/*    */       
/* 95 */       FormatRed formatRed = new FormatRed(GraphicsUtil.wrap(img), sm);
/* 96 */       writer.writeImage(formatRed, ostream, params);
/* 97 */       ostream.flush();
/* 98 */     } catch (IOException ex) {
/* 99 */       throw new TranscoderException(ex);
/*    */     } 
/*    */   }
/*    */ }


/* Location:              C:\Program Files (x86)\IBM\WebSphere\AppServer\profiles\ctgAppSrv01\installedApps\ctgCell01\MAXIMO.ear\maximouiweb.war\WEB-INF\classes\!\com\ibm\tivoli\maximo\report\control\svgtools\batik-codec.jar!\org\apache\batik\ext\awt\image\codec\imageio\TIFFTranscoderImageIOWriteAdapter.class
 * Java compiler version: 2 (46.0)
 * JD-Core Version:       1.0.7
 */